@@ -14,52 +14,64 @@
 
 
 # flake8: noqa
+from multipledispatch import halt_ordering, restart_ordering
 
-from ibis.filesystems import HDFS, WebHDFS
-from ibis.common import IbisError
-
+import ibis.config_init
+import ibis.util as util
 import ibis.expr.api as api
 import ibis.expr.types as ir
 
+from ibis.config import options
+from ibis.common import IbisError
+from ibis.compat import suppress
+from ibis.filesystems import HDFS, WebHDFS
+
+# speeds up signature registration
+halt_ordering()
+
 # __all__ is defined
 from ibis.expr.api import *
 
-try:
+# pandas backend is mandatory
+import ibis.pandas.api as pandas
+
+with suppress(ImportError):
+    # pip install ibis-framework[csv]
+    import ibis.file.csv as csv
+
+with suppress(ImportError):
+    # pip install ibis-framework[parquet]
+    import ibis.file.parquet as parquet
+
+with suppress(ImportError):
+    # pip install  ibis-framework[hdf5]
+    import ibis.file.hdf5 as hdf5
+
+with suppress(ImportError):
+    # pip install ibis-framework[impala]
     import ibis.impala.api as impala
-except ImportError:  # pip install ibis-framework[impala]
-    pass
 
-try:
+with suppress(ImportError):
+    # pip install ibis-framework[sqlite]
     import ibis.sql.sqlite.api as sqlite
-except ImportError:  # pip install ibis-framework[sqlite]
-    pass
 
-try:
+with suppress(ImportError):
+    # pip install ibis-framework[postgres]
     import ibis.sql.postgres.api as postgres
-except ImportError:  # pip install ibis-framework[postgres]
-    pass
 
-try:
+with suppress(ImportError):
+    # pip install ibis-framework[mysql]
+    import ibis.sql.mysql.api as mysql
+
+with suppress(ImportError):
+    # pip install ibis-framework[clickhouse]
     import ibis.clickhouse.api as clickhouse
-except ImportError:  # pip install ibis-framework[clickhouse]
-    pass
 
-try:
+with suppress(ImportError):
+    # pip install ibis-framework[bigquery]
     import ibis.bigquery.api as bigquery
-except ImportError:  # pip install ibis-framework[bigquery]
-    pass
 
-try:
-    from multipledispatch import halt_ordering, restart_ordering
-    halt_ordering()
-    import ibis.pandas.api as pandas
-    restart_ordering()
-except ImportError:  # pip install  ibis-framework[pandas]
-    pass
-
-import ibis.config_init
-from ibis.config import options
-import ibis.util as util
+restart_ordering()
 
 
 def hdfs_connect(host='localhost', port=50070, protocol='webhdfs',
@@ -117,17 +129,6 @@ def hdfs_connect(host='localhost', port=50070, protocol='webhdfs',
         hdfs_client = InsecureClient(url, **kwds)
     return WebHDFS(hdfs_client)
 
-def test(impala=False):
-    import pytest
-    import ibis
-    import os
-
-    ibis_dir, _ = os.path.split(ibis.__file__)
-
-    args = ['--pyargs', ibis_dir]
-    if impala:
-        args.append('--impala')
-    pytest.main(args)
 
 from ._version import get_versions
 __version__ = get_versions()['version']
