@@ -8,19 +8,17 @@ import ibis.expr.operations as ops
 import ibis.sql.transforms as transforms
 
 from ibis.clickhouse.identifiers import quote_identifier
-from ibis.clickhouse.types import ibis_to_clickhouse
 
 
 def _cast(translator, expr):
+    from ibis.clickhouse.client import ClickhouseDataType
+
     op = expr.op()
     arg, target = op.args
     arg_ = translator.translate(arg)
+    type_ = str(ClickhouseDataType.from_ibis(target, nullable=False))
 
-    if isinstance(arg, ir.CategoryValue) and target == 'int32':
-        return arg_
-    else:
-        type_ = ibis_to_clickhouse[target.name.lower()]
-        return 'CAST({0!s} AS {1!s})'.format(arg_, type_)
+    return 'CAST({0!s} AS {1!s})'.format(arg_, type_)
 
 
 def _between(translator, expr):
@@ -65,7 +63,7 @@ def fixed_arity(func_name, arity):
         arg_count = len(op.args)
         if arity != arg_count:
             msg = 'Incorrect number of args {0} instead of {1}'
-            raise com.TranslationError(msg.format(arg_count, arity))
+            raise com.UnsupportedOperationError(msg.format(arg_count, arity))
         return _call(translator, func_name, *op.args)
     return formatter
 
@@ -81,7 +79,7 @@ def agg_variance_like(func):
                 'pop': '{0}Pop'.format(func)}
 
     def formatter(translator, expr):
-        arg, where, how = expr.op().args
+        arg, how, where = expr.op().args
         return _aggregate(translator, variants[how], arg, where)
 
     return formatter
@@ -129,6 +127,14 @@ def varargs(func_name):
     return varargs_formatter
 
 
+def _arbitrary(translator, expr):
+    arg, how, where = expr.op().args
+    functions = {'first': 'any',
+                 'last': 'anyLast',
+                 'heavy': 'anyHeavy'}
+    return _aggregate(translator, functions[how], arg, where=where)
+
+
 def _substring(translator, expr):
     # arg_ is the formatted notation
     op = expr.op()
@@ -151,8 +157,9 @@ def _string_find(translator, expr):
     op = expr.op()
     arg, substr, start, _ = op.args
     if start is not None:
-        raise com.TranslationError('String find doesn\'t '
-                                   'support start argument')
+        raise com.UnsupportedOperationError(
+            "String find doesn't support start argument"
+        )
 
     return _call(translator, 'position', arg, substr) + ' - 1'
 
@@ -169,12 +176,6 @@ def _regex_extract(translator, expr):
     return 'extractAll({0}, {1})'.format(arg_, pattern_)
 
 
-def _string_join(translator, expr):
-    op = expr.op()
-    arg, strings = op.args
-    return _call(translator, 'concat_ws', arg, *strings)
-
-
 def _parse_url(translator, expr):
     op = expr.op()
     arg, extract, key = op.args
@@ -191,8 +192,9 @@ def _parse_url(translator, expr):
         else:
             return _call(translator, 'queryString', arg)
     else:
-        raise com.TranslationError('Parse url with extrac {0} is not '
-                                   'supported'.format(extract))
+        raise com.UnsupportedOperationError(
+            'Parse url with extract {0} is not supported'.format(extract)
+        )
 
 
 def _index_of(translator, expr):
@@ -233,8 +235,9 @@ def _hash(translator, expr):
                   'sipHash64', 'sipHash128'}
 
     if how not in algorithms:
-        raise com.TranslationError('Unsupported hash algorithm {0}'
-                                   .format(how))
+        raise com.UnsupportedOperationError(
+            'Unsupported hash algorithm {0}'.format(how)
+        )
 
     return _call(translator, how, arg)
 
@@ -261,6 +264,26 @@ def _value_list(translator, expr):
     return '({0})'.format(', '.join(values_))
 
 
+def _interval_format(translator, expr):
+    if expr.unit in {'ms', 'us', 'ns'}:
+        raise com.UnsupportedOperationError(
+            "Clickhouse doesn't support subsecond interval resolutions")
+
+    return 'INTERVAL {} {}'.format(expr.op().value, expr.resolution.upper())
+
+
+def _interval_from_integer(translator, expr):
+    op = expr.op()
+    arg, unit = op.args
+
+    if expr.unit in {'ms', 'us', 'ns'}:
+        raise com.UnsupportedOperationError(
+            "Clickhouse doesn't support subsecond interval resolutions")
+
+    arg_ = translator.translate(arg)
+    return 'INTERVAL {} {}'.format(arg_, expr.resolution.upper())
+
+
 def literal(translator, expr):
     value = expr.op().value
     if isinstance(expr, ir.BooleanValue):
@@ -269,6 +292,8 @@ def literal(translator, expr):
         return "'{0!s}'".format(value.replace("'", "\\'"))
     elif isinstance(expr, ir.NumericValue):
         return repr(value)
+    elif isinstance(expr, ir.IntervalValue):
+        return _interval_format(translator, expr)
     elif isinstance(expr, ir.TimestampValue):
         if isinstance(value, datetime):
             if value.microsecond != 0:
@@ -280,8 +305,10 @@ def literal(translator, expr):
         if isinstance(value, date):
             value = value.strftime('%Y-%m-%d')
         return "toDate('{0!s}')".format(value)
+    elif isinstance(expr, ir.ArrayValue):
+        return str(list(value))
     else:
-        raise NotImplementedError
+        raise NotImplementedError(type(expr))
 
 
 class CaseFormatter(object):
@@ -359,29 +386,12 @@ def _timestamp_from_unix(translator, expr):
     op = expr.op()
     arg, unit = op.args
 
-    if unit == 'ms':
-        raise ValueError('`ms` unit is not supported!')
-    elif unit == 'us':
-        raise ValueError('`us` unit is not supported!')
+    if unit in {'ms', 'us', 'ns'}:
+        raise ValueError('`{}` unit is not supported!'.format(unit))
 
     return _call(translator, 'toDateTime', arg)
 
 
-def _timestamp_delta(translator, expr):
-    op = expr.op()
-    arg, offset = op.args
-
-    if isinstance(arg, ir.TimestampValue):
-        offset_ = offset.to_unit('s').n
-    elif isinstance(arg, ir.DateValue):
-        offset_ = offset.to_unit('d').n
-    else:
-        raise com.TranslationError('Unsupported timedelta operation')
-
-    arg_ = translator.translate(arg)
-    return '{0} + {1}'.format(arg_, offset_)
-
-
 def _truncate(translator, expr):
     op = expr.op()
     arg, unit = op.args
@@ -389,15 +399,19 @@ def _truncate(translator, expr):
     converters = {
         'Y': 'toStartOfYear',
         'M': 'toStartOfMonth',
+        'W': 'toMonday',
         'D': 'toDate',
-        'H': 'toStartOfHour',
-        'MI': 'toStartOfMinute'
+        'h': 'toStartOfHour',
+        'm': 'toStartOfMinute',
+        's': 'toDateTime'
     }
 
     try:
         converter = converters[unit]
     except KeyError:
-        raise com.TranslationError('Unsupported concat unit {0}'.format(unit))
+        raise com.UnsupportedOperationError(
+            'Unsupported truncate unit {}'.format(unit)
+        )
 
     return _call(translator, converter, arg)
 
@@ -436,7 +450,7 @@ def _table_column(translator, expr):
         proj_expr = table.projection([field_name]).to_array()
         return _table_array_view(translator, proj_expr)
 
-    # TODO: table aliasing is partially supported
+    # TODO(kszucs): table aliasing is partially supported
     # if ctx.need_aliases():
     #     alias = ctx.get_ref(table)
     #     if alias is not None:
@@ -445,7 +459,40 @@ def _table_column(translator, expr):
     return quoted_name
 
 
-# TODO: clickhouse uses differenct string functions
+def _string_split(translator, expr):
+    value, sep = expr.op().args
+    return 'splitByString({}, {})'.format(
+        translator.translate(sep),
+        translator.translate(value)
+    )
+
+
+def _string_join(translator, expr):
+    sep, elements = expr.op().args
+    assert isinstance(elements.op(), ir.ValueList), \
+        'elements must be a ValueList, got {}'.format(type(elements.op()))
+    return 'arrayStringConcat([{}], {})'.format(
+        ', '.join(map(translator.translate, elements)),
+        translator.translate(sep),
+    )
+
+
+def _string_repeat(translator, expr):
+    value, times = expr.op().args
+    result = 'arrayStringConcat(arrayMap(x -> {}, range({})))'.format(
+        translator.translate(value), translator.translate(times)
+    )
+    return result
+
+
+def _string_like(translator, expr):
+    value, pattern = expr.op().args[:2]
+    return '{} LIKE {}'.format(
+        translator.translate(value), translator.translate(pattern)
+    )
+
+
+# TODO: clickhouse uses different string functions
 #       for ascii and utf-8 encodings,
 
 _binary_infix_ops = {
@@ -481,6 +528,9 @@ _operation_registry = {
     # Unary operations
     ops.TypeOf: unary('toTypeName'),
 
+    ops.IsNan: unary('isNaN'),
+    ops.IsInf: unary('isInfinite'),
+
     ops.Abs: unary('abs'),
     ops.Ceil: unary('ceil'),
     ops.Floor: unary('floor'),
@@ -514,6 +564,7 @@ _operation_registry = {
 
     ops.Count: agg('count'),
     ops.CountDistinct: agg('uniq'),
+    ops.Arbitrary: _arbitrary,
 
     # string operations
     ops.StringLength: unary('length'),
@@ -524,26 +575,34 @@ _operation_registry = {
     ops.StringFind: _string_find,
     ops.FindInSet: _index_of,
     ops.StringReplace: fixed_arity('replaceAll', 3),
+    ops.StringJoin: _string_join,
+    ops.StringSplit: _string_split,
+    ops.StringSQLLike: _string_like,
+    ops.Repeat: _string_repeat,
 
-    # TODO: there are no concat_ws in clickhouse
-    # ops.StringJoin: varargs('concat'),
-
-    ops.StringSQLLike: binary_infix_op('LIKE'),
     ops.RegexSearch: fixed_arity('match', 2),
     # TODO: extractAll(haystack, pattern)[index + 1]
     ops.RegexExtract: _regex_extract,
     ops.RegexReplace: fixed_arity('replaceRegexpAll', 3),
     ops.ParseURL: _parse_url,
 
-    # Timestamp operations
+    # Temporal operations
+    ops.Date: unary('toDate'),
+    ops.DateTruncate: _truncate,
+
     ops.TimestampNow: lambda *args: 'now()',
+    ops.TimestampTruncate: _truncate,
+
+    ops.TimeTruncate: _truncate,
+
+    ops.IntervalFromInteger: _interval_from_integer,
+
     ops.ExtractYear: unary('toYear'),
     ops.ExtractMonth: unary('toMonth'),
     ops.ExtractDay: unary('toDayOfMonth'),
     ops.ExtractHour: unary('toHour'),
     ops.ExtractMinute: unary('toMinute'),
     ops.ExtractSecond: unary('toSecond'),
-    ops.Truncate: _truncate,
 
     # Other operations
     ops.E: lambda *args: 'e()',
@@ -553,6 +612,8 @@ _operation_registry = {
 
     ops.Cast: _cast,
 
+    # for more than 2 args this should be arrayGreatest|Least(array([]))
+    # because clickhouse's greatest and least doesn't support varargs
     ops.Greatest: varargs('greatest'),
     ops.Least: varargs('least'),
 
@@ -568,18 +629,25 @@ _operation_registry = {
     ops.TableColumn: _table_column,
     ops.TableArrayView: _table_array_view,
 
-    ops.TimestampDelta: _timestamp_delta,
+    ops.DateAdd: binary_infix_op('+'),
+    ops.DateSub: binary_infix_op('-'),
+    ops.DateDiff: binary_infix_op('-'),
+    ops.TimestampAdd: binary_infix_op('+'),
+    ops.TimestampSub: binary_infix_op('-'),
+    ops.TimestampDiff: binary_infix_op('-'),
     ops.TimestampFromUNIX: _timestamp_from_unix,
 
     transforms.ExistsSubquery: _exists_subquery,
-    transforms.NotExistsSubquery: _exists_subquery
+    transforms.NotExistsSubquery: _exists_subquery,
+
+    ops.ArrayLength: unary('length'),
 }
 
 
 def raise_error(translator, expr, *args):
-    msg = 'Clickhouse backend doesn\'t support {0} operation!'
+    msg = "Clickhouse backend doesn't support {0} operation!"
     op = expr.op()
-    raise com.TranslationError(msg.format(type(op)))
+    raise com.UnsupportedOperationError(msg.format(type(op)))
 
 
 def _null_literal(translator, expr):
