@@ -3,10 +3,10 @@ import ibis
 import pytest
 
 
-CLICKHOUSE_HOST = os.environ.get('IBIS_CLICKHOUSE_HOST', 'localhost')
-CLICKHOUSE_PORT = int(os.environ.get('IBIS_CLICKHOUSE_PORT', 9000))
-CLICKHOUSE_USER = os.environ.get('IBIS_CLICKHOUSE_USER', 'default')
-CLICKHOUSE_PASS = os.environ.get('IBIS_CLICKHOUSE_PASS', '')
+CLICKHOUSE_HOST = os.environ.get('IBIS_TEST_CLICKHOUSE_HOST', 'localhost')
+CLICKHOUSE_PORT = int(os.environ.get('IBIS_TEST_CLICKHOUSE_PORT', 9000))
+CLICKHOUSE_USER = os.environ.get('IBIS_TEST_CLICKHOUSE_USER', 'default')
+CLICKHOUSE_PASS = os.environ.get('IBIS_TEST_CLICKHOUSE_PASSWORD', '')
 IBIS_TEST_CLICKHOUSE_DB = os.environ.get('IBIS_TEST_DATA_DB', 'ibis_testing')
 
 
@@ -14,6 +14,7 @@ IBIS_TEST_CLICKHOUSE_DB = os.environ.get('IBIS_TEST_DATA_DB', 'ibis_testing')
 def con():
     return ibis.clickhouse.connect(
         host=CLICKHOUSE_HOST,
+        port=CLICKHOUSE_PORT,
         user=CLICKHOUSE_USER,
         password=CLICKHOUSE_PASS,
         database=IBIS_TEST_CLICKHOUSE_DB,
@@ -37,5 +38,7 @@ def df(alltypes):
 
 @pytest.fixture
 def translate():
-    from ibis.clickhouse.compiler import ClickhouseExprTranslator
-    return lambda expr: ClickhouseExprTranslator(expr).get_result()
+    from ibis.clickhouse.compiler import ClickhouseDialect
+    dialect = ClickhouseDialect()
+    context = dialect.make_context()
+    return lambda expr: dialect.translator(expr, context).get_result()
