@@ -12,18 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pandas as pd
+import abc
+import six
 
-import toolz
-
-from ibis.compat import zip as czip
 from ibis.config import options
 
+import ibis.util as util
+import ibis.common as com
 import ibis.expr.types as ir
+import ibis.expr.schema as sch
 import ibis.expr.operations as ops
 import ibis.sql.compiler as comp
-import ibis.common as com
-import ibis.util as util
 
 
 class Client(object):
@@ -38,7 +37,7 @@ class Query(object):
     supporting such functionality).
     """
 
-    def __init__(self, client, ddl):
+    def __init__(self, client, ddl, **kwargs):
         self.client = client
         self.expr = getattr(
             ddl, 'parent_expr', getattr(ddl, 'table_set', None)
@@ -50,6 +49,7 @@ class Query(object):
             self.compiled_ddl = ddl
 
         self.result_wrapper = getattr(ddl, 'result_handler', None)
+        self.extra_options = kwargs
 
     def execute(self):
         # synchronous by default
@@ -64,22 +64,18 @@ class Query(object):
         return result
 
     def _fetch(self, cursor):
-        rows = cursor.fetchall()
-        # TODO(wesm): please evaluate/reimpl to optimize for perf/memory
-        dtypes = [self._db_type_to_dtype(x[1]) for x in cursor.description]
-        names = [x[0] for x in cursor.description]
-        cols = {}
-        for (col, name, dtype) in czip(czip(*rows), names, dtypes):
-            try:
-                cols[name] = pd.Series(col, dtype=dtype)
-            except TypeError:
-                # coercing to specified dtype failed, e.g. NULL vals in int col
-                cols[name] = pd.Series(col)
-        return pd.DataFrame(cols, columns=names)
-
-    def _db_type_to_dtype(self, db_type, column):
         raise NotImplementedError
 
+    def schema(self):
+
+        if isinstance(self.expr, (ir.TableExpr, ir.ExprList, sch.HasSchema)):
+            return self.expr.schema()
+        elif isinstance(self.expr, ir.ValueExpr):
+            return sch.schema([(self.expr.get_name(), self.expr.type())])
+        else:
+            raise ValueError('Expression with type {} does not have a '
+                             'schema'.format(type(self.expr)))
+
 
 class AsyncQuery(Query):
 
@@ -98,11 +94,13 @@ class AsyncQuery(Query):
         raise NotImplementedError
 
 
-class SQLClient(Client):
+class SQLClient(six.with_metaclass(abc.ABCMeta, Client)):
 
     sync_query = Query
     async_query = Query
 
+    dialect = comp.Dialect
+
     def table(self, name, database=None):
         """
         Create a table expression that references a particular table in the
@@ -200,7 +198,8 @@ class SQLClient(Client):
         """
         return self._execute(query, results=results)
 
-    def execute(self, expr, params=None, limit='default', async=False):
+    def execute(self, expr, params=None, limit='default', async=False,
+                **kwargs):
         """
         Compile and execute Ibis expression using this backend client
         interface, returning results in-memory in the appropriate object type
@@ -226,11 +225,12 @@ class SQLClient(Client):
         if len(ast.queries) > 1:
             raise NotImplementedError
         else:
-            return self._execute_query(ast.queries[0], async=async)
+            return self._execute_query(ast.queries[0], async=async, **kwargs)
 
-    def _execute_query(self, ddl, async=False):
+    def _execute_query(self, ddl, async=False, **kwargs):
         klass = self.async_query if async else self.sync_query
-        return klass(self, ddl).execute()
+        inst = klass(self, ddl, **kwargs)
+        return inst.execute()
 
     def compile(self, expr, params=None, limit=None):
         """
@@ -245,7 +245,9 @@ class SQLClient(Client):
         return queries[0] if len(queries) == 1 else queries
 
     def _build_ast_ensure_limit(self, expr, limit, params=None):
-        ast = self._build_ast(expr, params=params)
+        context = self.dialect.make_context(params=params)
+
+        ast = self._build_ast(expr, context)
         # note: limit can still be None at this point, if the global
         # default_limit is None
         for query in reversed(ast.queries):
@@ -267,7 +269,7 @@ class SQLClient(Client):
                                    'offset': query.limit['offset']}
         return ast
 
-    def explain(self, expr):
+    def explain(self, expr, params=None):
         """
         Query for and return the query plan associated with the indicated
         expression or SQL query.
@@ -277,7 +279,8 @@ class SQLClient(Client):
         plan : string
         """
         if isinstance(expr, ir.Expr):
-            ast = self._build_ast(expr)
+            context = self.dialect.make_context(params=params)
+            ast = self._build_ast(expr, context)
             if len(ast.queries) > 1:
                 raise Exception('Multi-query expression')
 
@@ -293,9 +296,9 @@ class SQLClient(Client):
         return 'Query:\n{0}\n\n{1}'.format(util.indent(query, 2),
                                            '\n'.join(result))
 
-    def _build_ast(self, expr, params=None):
+    def _build_ast(self, expr, context):
         # Implement in clients
-        raise NotImplementedError
+        raise NotImplementedError(type(self).__name__)
 
 
 class QueryPipeline(object):
@@ -308,18 +311,33 @@ class QueryPipeline(object):
     pass
 
 
-def execute(expr, limit='default', async=False, params=None):
-    backend = find_backend(expr)
-    return backend.execute(expr, limit=limit, async=async, params=params)
+def validate_backends(backends):
+    if not backends:
+        default = options.default_backend
+        if default is None:
+            raise com.IbisError(
+                'Expression depends on no backends, and found no default'
+            )
+        return [default]
+
+    if len(backends) > 1:
+        raise ValueError('Multiple backends found')
+    return backends
 
 
-def compile(expr, limit=None, params=None):
-    backend = find_backend(expr)
-    return backend.compile(expr, limit=limit, params=params)
+def execute(expr, limit='default', async=False, params=None, **kwargs):
+    backend, = validate_backends(list(find_backends(expr)))
+    return backend.execute(expr, limit=limit, async=async, params=params,
+                           **kwargs)
 
 
-def find_backend(expr):
-    backends = []
+def compile(expr, limit=None, params=None, **kwargs):
+    backend, = validate_backends(list(find_backends(expr)))
+    return backend.compile(expr, limit=limit, params=params, **kwargs)
+
+
+def find_backends(expr):
+    seen_backends = set()
 
     stack = [expr.op()]
     seen = set()
@@ -332,24 +350,12 @@ def find_backend(expr):
 
             for arg in node.flat_args():
                 if isinstance(arg, Client):
-                    backends.append(arg)
+                    if arg not in seen_backends:
+                        yield arg
+                        seen_backends.add(arg)
                 elif isinstance(arg, ir.Expr):
                     stack.append(arg.op())
 
-    backends = list(toolz.unique(backends, key=id))
-
-    if len(backends) > 1:
-        raise ValueError('Multiple backends found')
-    elif not backends:
-        default = options.default_backend
-        if default is None:
-            raise com.IbisError(
-                'Expression depends on no backends, and found no default'
-            )
-        return default
-
-    return backends[0]
-
 
 class Database(object):
 
@@ -358,12 +364,12 @@ class Database(object):
         self.client = client
 
     def __repr__(self):
-        return "{0}('{1}')".format('Database', self.name)
+        return '{}({!r})'.format(type(self).__name__, self.name)
 
     def __dir__(self):
         attrs = dir(type(self))
         unqualified_tables = [self._unqualify(x) for x in self.tables]
-        return list(frozenset(attrs + unqualified_tables))
+        return sorted(frozenset(attrs + unqualified_tables))
 
     def __contains__(self, key):
         return key in self.tables
@@ -376,15 +382,7 @@ class Database(object):
         return self.table(key)
 
     def __getattr__(self, key):
-        special_attrs = ['_ipython_display_', 'trait_names',
-                         '_getAttributeNames']
-
-        try:
-            return object.__getattribute__(self, key)
-        except AttributeError:
-            if key in special_attrs:
-                raise
-            return self.table(key)
+        return self.table(key)
 
     def _qualify(self, value):
         return value
