@@ -37,6 +37,18 @@ class TranslationError(IbisError):
     pass
 
 
+class OperationNotDefinedError(TranslationError):
+    pass
+
+
+class UnsupportedOperationError(TranslationError):
+    pass
+
+
+class UnsupportedBackendType(TranslationError):
+    pass
+
+
 class IbisInputError(ValueError, IbisError):
     pass
 
