@@ -15,9 +15,10 @@
 import toolz
 
 import ibis.expr.types as ir
+import ibis.expr.lineage as lin
 import ibis.expr.operations as ops
 
-from ibis.expr.datatypes import HasSchema
+from ibis.expr.schema import HasSchema
 from ibis.expr.window import window
 
 from ibis.common import RelationError, ExpressionError, IbisTypeError
@@ -103,7 +104,6 @@ class ScalarAggregate(object):
 
     def get_result(self):
         expr = self.expr
-
         subbed_expr = self._visit(expr)
 
         try:
@@ -220,22 +220,13 @@ def find_immediate_parent_tables(expr):
             Literal[int8]
               1
     """
-    stack = [expr]
-    seen = set()
+    def finder(expr):
+        if isinstance(expr, ir.TableExpr):
+            return lin.halt, expr
+        else:
+            return lin.proceed, None
 
-    while stack:
-        e = stack.pop()
-        node = e.op()
-
-        if node not in seen:
-            seen.add(node)
-
-            if isinstance(e, ir.TableExpr):
-                yield e
-            else:  # Only traverse into non TableExpr objects
-                stack.extend(
-                    arg for arg in node.flat_args() if isinstance(arg, ir.Expr)
-                )
+    return lin.traverse(finder, expr)
 
 
 def is_scalar_reduce(x):
@@ -494,6 +485,36 @@ def _base_table(table_node):
         return _base_table(table_node.table.op())
 
 
+def has_reduction(expr):
+    """Does `expr` contain a reduction?
+
+    Parameters
+    ----------
+    expr : ibis.expr.types.Expr
+        An ibis expression
+
+    Returns
+    -------
+    truth_value : bool
+        Whether or not there's at least one reduction in `expr`
+
+    Notes
+    -----
+    The ``isinstance(op, ops.TableNode)`` check in this function implies
+    that we only examine every non-table expression that precedes the first
+    table expression.
+    """
+    def fn(expr):
+        op = expr.op()
+        if isinstance(op, ops.TableNode):  # don't go below any table nodes
+            return lin.halt, None
+        if isinstance(op, ops.Reduction):
+            return lin.halt, True
+        return lin.proceed, None
+    reduction_status = lin.traverse(fn, expr)
+    return any(reduction_status)
+
+
 def apply_filter(expr, predicates):
     # This will attempt predicate pushdown in the cases where we can do it
     # easily and safely, to make both cleaner SQL and fewer referential errors
@@ -505,8 +526,11 @@ def apply_filter(expr, predicates):
         return _filter_selection(expr, predicates)
     elif isinstance(op, ops.Aggregation):
         # Potential fusion opportunity
+        # GH1344: We can't sub in things with correlated subqueries
         simplified_predicates = [
-            sub_for(predicate, [(expr, op.table)]) for predicate in predicates
+            sub_for(predicate, [(expr, op.table)])
+            if not has_reduction(predicate) else predicate
+            for predicate in predicates
         ]
 
         if op.table._is_valid(simplified_predicates):
@@ -542,15 +566,18 @@ def _filter_selection(expr, predicates):
         # Potential fusion opportunity. The predicates may need to be
         # rewritten in terms of the child table. This prevents the broken
         # ref issue (described in more detail in #59)
-        simplified_predicates = [sub_for(x, [(expr, op.table)])
-                                 for x in predicates]
+        simplified_predicates = [
+            sub_for(predicate, [(expr, op.table)])
+            if not has_reduction(predicate) else predicate
+            for predicate in predicates
+        ]
 
         if op.table._is_valid(simplified_predicates):
             result = ops.Selection(
                 op.table, [],
                 predicates=op.predicates + simplified_predicates,
                 sort_keys=op.sort_keys)
-            return ir.TableExpr(result)
+            return result.to_expr()
 
     can_pushdown = _can_pushdown(op, predicates)
 
@@ -565,7 +592,7 @@ def _filter_selection(expr, predicates):
         result = ops.Selection(expr, proj_exprs=[],
                                predicates=predicates)
 
-    return ir.TableExpr(result)
+    return result.to_expr()
 
 
 def _can_pushdown(op, predicates):
@@ -582,12 +609,12 @@ def _can_pushdown(op, predicates):
     # 3) Appears in the selections in the projection (either is part of one of
     #    the entire tables or a single column selection)
 
-    can_pushdown = True
     for pred in predicates:
         validator = _PushdownValidate(op, pred)
         predicate_is_valid = validator.get_result()
-        can_pushdown = can_pushdown and predicate_is_valid
-    return can_pushdown
+        if not predicate_is_valid:
+            return False
+    return True
 
 
 class _PushdownValidate(object):
@@ -595,33 +622,20 @@ class _PushdownValidate(object):
     def __init__(self, parent, predicate):
         self.parent = parent
         self.pred = predicate
-
         self.validator = ExprValidator([self.parent.table])
 
-        self.valid = True
-
     def get_result(self):
-        self._walk(self.pred)
-        return self.valid
+        predicate = self.pred
+        return not has_reduction(predicate) and all(self._walk(predicate))
 
     def _walk(self, expr):
-        stack = [expr]
-        seen = set()
+        def validate(expr):
+            op = expr.op()
+            if isinstance(op, ops.TableColumn):
+                return lin.proceed, self._validate_column(expr)
+            return lin.proceed, None
 
-        while stack:
-            e = stack.pop()
-            node = e.op()
-
-            if node not in seen:
-                seen.add(node)
-
-                if isinstance(node, ops.TableColumn):
-                    self.valid = self.valid and self._validate_column(e)
-
-                stack.extend(
-                    arg for arg in node.flat_args()
-                    if isinstance(arg, ir.ValueExpr)
-                )
+        return lin.traverse(validate, expr, type=ir.ValueExpr)
 
     def _validate_column(self, expr):
         if isinstance(self.parent, ops.Selection):
@@ -653,8 +667,6 @@ class _PushdownValidate(object):
                 is_valid = (col_table.is_ancestor(node.table) or
                             col_table.is_ancestor(lifted_node.table))
 
-                # is_valid = True
-
         return is_valid
 
 
@@ -663,16 +675,6 @@ def _is_aliased(col_expr):
 
 
 def windowize_function(expr, w=None):
-    def _check_window(x):
-        # Hmm
-        arg, window = x.op().args
-        if isinstance(arg.op(), ops.RowNumber):
-            if len(window._order_by) == 0:
-                raise ExpressionError('RowNumber requires explicit '
-                                      'window sort')
-
-        return x
-
     def _windowize(x, w):
         if not isinstance(x.op(), ops.WindowOp):
             walked = _walk(x, w)
@@ -687,16 +689,15 @@ def windowize_function(expr, w=None):
                 walked = x
 
         op = walked.op()
-        if (isinstance(op, ops.AnalyticOp) or
-                getattr(op, '_reduction', False)):
+        if isinstance(op, ops.AnalyticOp) or getattr(op, '_reduction', False):
             if w is None:
                 w = window()
-            return _check_window(walked.over(w))
+            return walked.over(w)
         elif isinstance(op, ops.WindowOp):
             if w is not None:
-                return _check_window(walked.over(w))
+                return walked.over(w)
             else:
-                return _check_window(walked)
+                return walked
         else:
             return walked
 
@@ -758,9 +759,10 @@ class Projector(object):
 
     def get_result(self):
         roots = self.parent_roots
+        first_root = roots[0]
 
-        if len(roots) == 1 and isinstance(roots[0], ops.Selection):
-            fused_op = self._check_fusion(roots[0])
+        if len(roots) == 1 and isinstance(first_root, ops.Selection):
+            fused_op = self._check_fusion(first_root)
             if fused_op is not None:
                 return fused_op
 
@@ -820,7 +822,7 @@ class Projector(object):
 def _maybe_resolve_exprs(table, exprs):
     try:
         return table._resolve(exprs)
-    except AttributeError:
+    except (AttributeError, IbisTypeError):
         return None
 
 
@@ -855,11 +857,7 @@ class ExprValidator(object):
         return self.roots_shared(node) > 0
 
     def roots_shared(self, node):
-        count = 0
-        for root in self.roots:
-            if root.is_ancestor(node):
-                count += 1
-        return count
+        return sum(root.is_ancestor(node) for root in self.roots)
 
     def shares_some_roots(self, expr):
         expr_roots = expr._root_tables()
@@ -892,71 +890,22 @@ class ExprValidator(object):
                 'dependencies of the table expression.' % repr(expr))
 
 
-class CommonSubexpr(object):
-
-    def __init__(self, exprs):
-        self.parent_exprs = exprs
-
-    def validate(self, expr):
-        if isinstance(expr, ir.TableExpr):
-            if not self._check(expr):
-                return False
-
+def fully_originate_from(exprs, parents):
+    def finder(expr):
         op = expr.op()
 
-        for arg in op.flat_args():
-            if not isinstance(arg, ir.Expr):
-                continue
-            elif not isinstance(arg, ir.TableExpr):
-                if not self.validate(arg):
-                    return False
-            else:
-                # Table expression. Must be found in a parent table expr a
-                # blocking root of one of the parent tables
-                if not self._check(arg):
-                    return False
+        if isinstance(expr, ir.TableExpr):
+            return lin.proceed, expr
+        elif op.blocks():
+            return lin.halt, None
+        else:
+            return lin.proceed, None
 
-        return True
+    # unique table dependencies of exprs and parents
+    exprs_deps = set(lin.traverse(finder, exprs))
+    parents_deps = set(lin.traverse(finder, parents))
 
-    def _check(self, expr):
-        # Table dependency matches one of the parent exprs
-        is_valid = False
-        for parent in self.parent_exprs:
-            is_valid = is_valid or self._check_table(parent, expr)
-        return is_valid
-
-    def _check_table(self, parent, needle):
-        def _matches(expr):
-            op = expr.op()
-
-            if expr.equals(needle):
-                return True
-
-            if op.blocks():
-                return False
-
-            for arg in op.flat_args():
-                if not isinstance(arg, ir.Expr):
-                    continue
-                if _matches(arg):
-                    return True
-
-            return True
-
-        return _matches(parent)
-
-    def validate_all(self, exprs):
-        for expr in exprs:
-            self.assert_valid(expr)
-
-    def assert_valid(self, expr):
-        if not self.validate(expr):
-            msg = self._error_message(expr)
-            raise RelationError(msg)
-
-    def _error_message(self, expr):
-        return ('The expression %s does not fully originate from '
-                'dependencies of the table expression.' % repr(expr))
+    return exprs_deps <= parents_deps
 
 
 class FilterValidator(ExprValidator):
@@ -976,6 +925,10 @@ class FilterValidator(ExprValidator):
     def validate(self, expr):
         op = expr.op()
 
+        if (isinstance(expr, ir.BooleanColumn) and
+                isinstance(op, ops.TableColumn)):
+            return True
+
         is_valid = True
 
         if isinstance(op, ops.Contains):
@@ -993,7 +946,7 @@ class FilterValidator(ExprValidator):
                 elif isinstance(arg, (ir.ColumnExpr, ir.AnalyticExpr)):
                     roots_valid.append(self.shares_some_roots(arg))
                 elif isinstance(arg, ir.Expr):
-                    raise NotImplementedError
+                    raise NotImplementedError(repr((type(expr), type(arg))))
                 else:
                     # arg_valid = True
                     pass
@@ -1053,29 +1006,13 @@ def find_source_table(expr):
     ...
     NotImplementedError: More than one base table not implemented
     """
-    first_tables = []
-
-    stack = [expr]
-    seen = set()
-
-    while stack:
-        e = stack.pop()
-        op = e.op()
-
-        if op not in seen:
-            seen.add(op)
-
-            arguments = [
-                arg for arg in reversed(list(op.flat_args()))
-                if isinstance(arg, ir.Expr)
-            ]
-            first_tables.extend(
-                arg for arg in arguments if isinstance(arg, ir.TableExpr)
-            )
-            stack.extend(
-                arg for arg in arguments if not isinstance(arg, ir.TableExpr)
-            )
+    def finder(expr):
+        if isinstance(expr, ir.TableExpr):
+            return lin.halt, expr
+        else:
+            return lin.proceed, None
 
+    first_tables = lin.traverse(finder, expr.op().flat_args())
     options = list(toolz.unique(first_tables, key=id))
 
     if len(options) > 1:
@@ -1115,7 +1052,7 @@ def flatten_predicate(expr):
         a = Column[int64*] 'a' from table
           ref_0
       right:
-        Literal[int8]
+        Literal[int64]
           1
     >>> predicates[1]  # doctest: +NORMALIZE_WHITESPACE
     ref_0
@@ -1132,26 +1069,10 @@ def flatten_predicate(expr):
         Literal[string]
           foo
     """
-    predicates = []
-    stack = [expr]
-    seen = set()
+    def predicate(expr):
+        if isinstance(expr.op(), ops.And):
+            return lin.proceed, None
+        else:
+            return lin.halt, expr
 
-    while stack:
-        e = stack.pop()
-
-        if not isinstance(e, ir.BooleanColumn):
-            raise TypeError(
-                'Predicate component is not an instance of ir.BooleanColumn'
-            )
-
-        op = e.op()
-
-        if op not in seen:
-            seen.add(op)
-
-            if isinstance(op, ops.And):
-                stack.append(op.right)
-                stack.append(op.left)
-            else:
-                predicates.append(e)
-    return predicates
+    return list(lin.traverse(predicate, expr, type=ir.BooleanColumn))
