@@ -16,14 +16,17 @@ from __future__ import print_function
 
 import warnings
 import operator
-import functools
+import datetime
 import collections
 
 import six
 import toolz
 
-from ibis.expr.datatypes import Schema  # noqa
+from ibis.compat import functools
+
+from ibis.expr.schema import Schema
 from ibis.expr import datatypes as dt
+from ibis.expr import schema as sch
 from ibis.expr.types import (Expr,  # noqa
                              ValueExpr, ScalarExpr, ColumnExpr,
                              TableExpr,
@@ -35,11 +38,13 @@ from ibis.expr.types import (Expr,  # noqa
                              Int64Value, Int64Scalar, Int64Column,
                              NullScalar,
                              BooleanValue, BooleanScalar, BooleanColumn,
+                             FloatingValue,
                              FloatValue, FloatScalar, FloatColumn,
                              DoubleValue, DoubleScalar, DoubleColumn,
                              StringValue, StringScalar, StringColumn,
                              DecimalValue, DecimalScalar, DecimalColumn,
                              TimestampValue, TimestampScalar, TimestampColumn,
+                             IntervalValue, IntervalScalar, IntervalColumn,
                              DateValue, TimeValue,
                              ArrayValue, ArrayScalar, ArrayColumn,
                              MapValue, MapScalar, MapColumn,
@@ -47,11 +52,8 @@ from ibis.expr.types import (Expr,  # noqa
                              CategoryValue, unnamed, as_value_expr, literal,
                              param, null, sequence)
 
-# __all__ is defined
-from ibis.expr.temporal import *  # noqa
-
 import ibis.common as _com
-from ibis.compat import PY2, to_time
+from ibis.compat import PY2, to_time, to_date
 from ibis.expr.analytics import bucket, histogram
 from ibis.expr.groupby import GroupedTableExpr  # noqa
 from ibis.expr.window import window, trailing_window, cumulative_window
@@ -59,13 +61,15 @@ import ibis.expr.analytics as _analytics
 import ibis.expr.analysis as _L
 import ibis.expr.types as ir
 import ibis.expr.operations as _ops
-import ibis.expr.temporal as _T
 import ibis.util as util
 
 
 __all__ = [
+    'infer_dtype', 'infer_schema',
     'schema', 'table', 'literal', 'expr_list',
-    'timestamp', 'time', 'param',
+    'timestamp', 'time', 'date', 'interval', 'param',
+    'nanosecond', 'microsecond', 'millisecond', 'second',
+    'minute', 'hour', 'day', 'week', 'month', 'year',
     'case', 'where', 'sequence',
     'now', 'desc', 'null', 'NA',
     'cast', 'coalesce', 'greatest', 'least',
@@ -76,7 +80,6 @@ __all__ = [
     'Expr', 'Schema',
     'window', 'trailing_window', 'cumulative_window',
 ]
-__all__ += _T.__all__
 
 
 NA = null()
@@ -97,7 +100,12 @@ double         DOUBLE
 boolean        BOOLEAN
 string         STRING
 timestamp      TIMESTAMP
-decimal(p, s)  DECIMAL(p,s)"""
+decimal(p, s)  DECIMAL(p,s)
+interval(u)    INTERVAL(u)"""
+
+
+infer_dtype = dt.infer
+infer_schema = sch.infer
 
 
 def schema(pairs=None, names=None, types=None):
@@ -180,6 +188,23 @@ def timestamp(value):
     return ir.TimestampScalar(ir.literal(value).op())
 
 
+def date(value):
+    """
+    Returns a date literal if value is likely coercible to a date
+
+    Parameters
+    ----------
+    value : date value as string
+
+    Returns
+    --------
+    result : TimeScalar
+    """
+    if isinstance(value, six.string_types):
+        value = to_date(value)
+    return ir.DateScalar(ir.literal(value).op())
+
+
 def time(value):
     """
     Returns a time literal if value is likely coercible to a time
@@ -192,14 +217,98 @@ def time(value):
     --------
     result : TimeScalar
     """
-    if PY2:
-        raise ValueError("time support is not enabled on python 2")
-
     if isinstance(value, six.string_types):
         value = to_time(value)
     return ir.TimeScalar(ir.literal(value).op())
 
 
+def interval(value=None, unit='s', years=None, quarters=None, months=None,
+             weeks=None, days=None, hours=None, minutes=None, seconds=None,
+             milliseconds=None, microseconds=None, nanoseconds=None):
+    """
+    Returns an interval literal
+
+    Parameters
+    ----------
+    value : int or datetime.timedelta, default None
+    years : int, default None
+    quarters : int, default None
+    months : int, default None
+    days : int, default None
+    weeks : int, default None
+    hours : int, default None
+    minutes : int, default None
+    seconds : int, default None
+    milliseconds : int, default None
+    microseconds : int, default None
+    nanoseconds : int, default None
+
+    Returns
+    --------
+    result : IntervalScalar
+    """
+    if value is not None:
+        if isinstance(value, datetime.timedelta):
+            unit = 's'
+            value = int(value.total_seconds())
+        elif not isinstance(value, six.integer_types):
+            raise ValueError('Interval value must be an integer')
+    else:
+        kwds = [
+            ('Y', years),
+            ('Q', quarters),
+            ('M', months),
+            ('W', weeks),
+            ('D', days),
+            ('h', hours),
+            ('m', minutes),
+            ('s', seconds),
+            ('ms', milliseconds),
+            ('us', microseconds),
+            ('ns', nanoseconds)
+        ]
+        defined_units = [(k, v) for k, v in kwds if v is not None]
+
+        if len(defined_units) != 1:
+            raise ValueError('Exactly one argument is required')
+
+        unit, value = defined_units[0]
+
+    value_type = ir.literal(value).type()
+    type = dt.Interval(unit, value_type)
+
+    return ir.literal(value, type=type).op().to_expr()
+
+
+@functools.wraps(interval)
+def timedelta(*args, **kwargs):
+    warnings.warn('ibis.timedelta is deprecated, use ibis.interval instead',
+                  DeprecationWarning)
+    return interval(*args, **kwargs)
+
+
+def _timedelta(name, unit):
+    def f(value=1):
+        msg = 'ibis.{0} is deprecated, use ibis.interval({0}s=n) instead'
+        warnings.warn(msg.format(name), DeprecationWarning)
+        return interval(value, unit=unit)
+    f.__name__ = name
+    return f
+
+
+year = _timedelta('year', 'Y')
+quarter = _timedelta('quarter', 'Q')
+month = _timedelta('month', 'M')
+week = _timedelta('week', 'W')
+day = _timedelta('day', 'D')
+hour = _timedelta('hour', 'h')
+minute = _timedelta('minute', 'm')
+second = _timedelta('second', 's')
+millisecond = _timedelta('millisecond', 'ms')
+microsecond = _timedelta('microsecond', 'us')
+nanosecond = _timedelta('nanosecond', 'ns')
+
+
 schema.__doc__ = """\
 Validate and return an Ibis Schema object
 
@@ -268,8 +377,10 @@ def now():
 
 
 def row_number():
-    """
-    Analytic function for the current row number, starting at 0
+    """Analytic function for the current row number, starting at 0.
+
+    This function does not require an ORDER BY clause, however, without an
+    ORDER BY clause the order of the result is nondeterministic.
 
     Returns
     -------
@@ -330,16 +441,14 @@ def count(expr, where=None):
     """
     op = expr.op()
     if isinstance(op, _ops.DistinctColumn):
-        if where is not None:
-            raise NotImplementedError
-        result = op.count().to_expr()
+        result = _ops.CountDistinct(op.args[0], where).to_expr()
     else:
         result = _ops.Count(expr, where).to_expr()
 
     return result.name('count')
 
 
-def group_concat(arg, sep=','):
+def group_concat(arg, sep=',', where=None):
     """
     Concatenate values using the indicated separator (comma by default) to
     produce a string
@@ -348,12 +457,32 @@ def group_concat(arg, sep=','):
     ----------
     arg : array expression
     sep : string, default ','
+    where : bool, default None
 
     Returns
     -------
     concatenated : string scalar
     """
-    return _ops.GroupConcat(arg, sep).to_expr()
+    return _ops.GroupConcat(arg, sep, where).to_expr()
+
+
+def arbitrary(arg, where=None, how='first'):
+    """
+    Selects the first / last non-null value in a column
+
+    Parameters
+    ----------
+    arg : array expression
+    where: bool, default None
+    how : {'first', 'last', 'heavy'}, default 'first'
+      Heavy selects a frequently occurring value using the heavy hitters
+      algorithm. Heavy is only supported by Clickhouse backend.
+
+    Returns
+    -------
+    arbitrary element : scalar type of caller
+    """
+    return _ops.Arbitrary(arg, how, where).to_expr()
 
 
 def _binop_expr(name, klass):
@@ -362,7 +491,7 @@ def _binop_expr(name, klass):
             other = as_value_expr(other)
             op = klass(self, other)
             return op.to_expr()
-        except NotImplementedError:
+        except (_com.IbisTypeError, NotImplementedError):
             return NotImplemented
 
     f.__name__ = name
@@ -728,6 +857,7 @@ floordiv = _binop_expr('__floordiv__', _ops.FloorDivide)
 pow = _binop_expr('__pow__', _ops.Power)
 mod = _binop_expr('__mod__', _ops.Modulus)
 
+radd = _rbinop_expr('__radd__', _ops.Add)
 rsub = _rbinop_expr('__rsub__', _ops.Subtract)
 rdiv = _rbinop_expr('__rdiv__', _ops.Divide)
 rfloordiv = _rbinop_expr('__rfloordiv__', _ops.FloorDivide)
@@ -765,23 +895,50 @@ def substitute(arg, value, replacement=None, else_=None):
 
 
 def _case(arg):
-    """
-    Create a new SimpleCaseBuilder to chain multiple if-else
-    statements. Add new search expressions with the .when method. These
-    must be comparable with this array expression. Conclude by calling
-    .end()
-
-    Examples
-    --------
-    >>> case_expr = (expr.case()
-    ...              .when(case1, output1)
-    ...              .when(case2, output2)
-    ...              .default(default_output)
-    ...              .end())  # doctest: +SKIP
+    """Create a new SimpleCaseBuilder to chain multiple if-else statements. Add
+    new search expressions with the .when method. These must be comparable with
+    this array expression. Conclude by calling .end()
 
     Returns
     -------
     builder : CaseBuilder
+
+    Examples
+    --------
+    >>> import ibis
+    >>> t = ibis.table([('string_col', 'string')], name='t')
+    >>> expr = t.string_col
+    >>> case_expr = (expr.case()
+    ...              .when('a', 'an a')
+    ...              .when('b', 'a b')
+    ...              .else_('null or (not a and not b)')
+    ...              .end())
+    >>> case_expr  # doctest: +NORMALIZE_WHITESPACE
+    ref_0
+    UnboundTable[table]
+      name: t
+      schema:
+        string_col : string
+    <BLANKLINE>
+    SimpleCase[string*]
+      base:
+        string_col = Column[string*] 'string_col' from table
+          ref_0
+      cases:
+        ValueList[string*]
+          Literal[string]
+            a
+          Literal[string]
+            b
+      results:
+        ValueList[string*]
+          Literal[string]
+            an a
+          Literal[string]
+            a b
+      default:
+        Literal[string]
+          null or (not a and not b)
     """
     return _ops.SimpleCaseBuilder(arg)
 
@@ -836,6 +993,7 @@ approx_nunique = _agg_function('approx_nunique', _ops.HLLCardinality, True)
 approx_median = _agg_function('approx_median', _ops.CMSMedian, True)
 max = _agg_function('max', _ops.Max, True)
 min = _agg_function('min', _ops.Min, True)
+nunique = _agg_function('nunique', _ops.CountDistinct, True)
 
 
 def lag(arg, offset=None, default=None):
@@ -886,18 +1044,8 @@ def distinct(arg):
     return op.to_expr()
 
 
-def nunique(arg):
-    """
-    Shorthand for foo.distinct().count(); computing the number of unique
-    values in an array.
-    """
-    return _ops.CountDistinct(arg).to_expr()
-
-
 def topk(arg, k, by=None):
     """
-    Produces
-
     Returns
     -------
     topk : TopK filter expression
@@ -993,6 +1141,7 @@ _generic_column_methods = dict(
     topk=topk,
     summary=_generic_summary,
     count=count,
+    arbitrary=arbitrary,
     min=min,
     max=max,
     approx_median=approx_median,
@@ -1127,6 +1276,22 @@ def _integer_to_timestamp(arg, unit='s'):
     return op.to_expr()
 
 
+def _integer_to_interval(arg, unit='s'):
+    """
+    Convert integer interval with the same inner type
+
+    Parameters
+    ----------
+    unit : {'Y', 'M', 'W', 'D', 'h', 'm', s', 'ms', 'us', 'ns'}
+
+    Returns
+    -------
+    interval : interval value expression
+    """
+    op = _ops.IntervalFromInteger(arg, unit)
+    return op.to_expr()
+
+
 abs = _unary_op('abs', _ops.Abs)
 ceil = _unary_op('ceil', _ops.Ceil)
 exp = _unary_op('exp', _ops.Exp)
@@ -1180,12 +1345,13 @@ _numeric_value_methods = dict(
     pow=pow,
 
     __radd__=add,
+    radd=add,
 
     __rsub__=rsub,
     rsub=rsub,
 
     __rmul__=_rbinop_expr('__rmul__', _ops.Multiply),
-    __rpow__=_binop_expr('__rpow__', _ops.Power),
+    __rpow__=_rbinop_expr('__rpow__', _ops.Power),
 
     __mod__=mod,
     __rmod__=_rbinop_expr('__rmod__', _ops.Modulus),
@@ -1211,6 +1377,7 @@ def convert_base(arg, from_base, to_base):
 
 _integer_value_methods = dict(
     to_timestamp=_integer_to_timestamp,
+    to_interval=_integer_to_interval,
     convert_base=convert_base
 )
 
@@ -1234,7 +1401,7 @@ def std(arg, where=None, how='sample'):
     -------
     stdev : double scalar
     """
-    expr = _ops.StandardDev(arg, where, how).to_expr()
+    expr = _ops.StandardDev(arg, how, where).to_expr()
     expr = expr.name('std')
     return expr
 
@@ -1251,7 +1418,7 @@ def variance(arg, where=None, how='sample'):
     -------
     stdev : double scalar
     """
-    expr = _ops.Variance(arg, where, how).to_expr()
+    expr = _ops.Variance(arg, how, where).to_expr()
     expr = expr.name('var')
     return expr
 
@@ -1273,8 +1440,14 @@ _numeric_column_methods = dict(
     summary=_numeric_summary,
 )
 
+_floating_value_methods = dict(
+    isnan=_unary_op('isnull', _ops.IsNan),
+    isinf=_unary_op('isinf', _ops.IsInf),
+)
+
 _add_methods(NumericValue, _numeric_value_methods)
 _add_methods(IntegerValue, _integer_value_methods)
+_add_methods(FloatingValue, _floating_value_methods)
 
 _add_methods(NumericColumn, _numeric_column_methods)
 
@@ -1365,7 +1538,7 @@ def _string_left(self, nchars):
 
 def _string_right(self, nchars):
     """
-    Split up to nchars starting from end of each string.
+    Return up to nchars starting from end of each string.
 
     Returns
     -------
@@ -1680,6 +1853,25 @@ def _string_contains(arg, substr):
     return arg.find(substr) >= 0
 
 
+def _string_split(arg, delimiter):
+    """Split `arg` on `delimiter`.
+
+    Parameters
+    ----------
+    arg : str or ibis.expr.types.StringValue
+    delimiter : str or ibis.expr.types.StringValue
+
+    Returns
+    -------
+    splitsville : Array[String]
+    """
+    return _ops.StringSplit(arg, delimiter).to_expr()
+
+
+def _string_concat(*args):
+    return _ops.StringConcat(*args).to_expr()
+
+
 def _string_dunder_contains(arg, substr):
     raise TypeError('Use val.contains(arg)')
 
@@ -1696,8 +1888,12 @@ def _string_getitem(self, key):
             raise ValueError('negative slicing not yet supported')
 
         return self.substr(start, stop - start)
+    elif isinstance(key, six.integer_types):
+        return self.substr(key, 1)
     else:
-        raise NotImplementedError
+        raise NotImplementedError(
+            'string __getitem__[{}]'.format(type(key).__name__)
+        )
 
 
 _string_value_methods = dict(
@@ -1732,10 +1928,12 @@ _string_value_methods = dict(
     find=_string_find,
     translate=_translate,
     find_in_set=_find_in_set,
+    split=_string_split,
     join=_string_join,
     lpad=_lpad,
     rpad=_rpad,
-    __add__=add,
+    __add__=_string_concat,
+    __radd__=lambda *args: _string_concat(*args[::-1]),
     __mul__=mul,
     __rmul__=mul,
 )
@@ -1795,13 +1993,27 @@ _array_column_methods = dict(
 
 _add_methods(ArrayValue, _array_column_methods)
 
+
 # ---------------------------------------------------------------------
 # Map API
 
+def get(expr, key, default):
+    """
+    Return the mapped value for this key, or the default
+    if the key does not exist
+
+    Parameters
+    ----------
+    key : any
+    default : any
+    """
+    return _ops.MapValueOrDefaultForKey(expr, key, default).to_expr()
+
 
 _map_column_methods = dict(
     length=_unary_op('length', _ops.MapLength),
     __getitem__=_binop_expr('__getitem__', _ops.MapValueForKey),
+    get=get,
     keys=_unary_op('keys', _ops.MapKeys),
     values=_unary_op('values', _ops.MapValues),
     __add__=_binop_expr('__add__', _ops.MapConcat),
@@ -1829,7 +2041,7 @@ def _struct_get_field(expr, field_name):
     value_expr : ibis.expr.types.ValueExpr
         An expression with the type of the field being accessed.
     """
-    return _ops.StructField(expr, field_name).to_expr()
+    return _ops.StructField(expr, field_name).to_expr().name(field_name)
 
 
 _struct_column_methods = dict(
@@ -1854,16 +2066,20 @@ def _timestamp_truncate(arg, unit):
       'Y': year
       'Q': quarter
       'M': month
-      'D': day
       'W': week
-      'H': hour
-      'MI': minute
+      'D': day
+      'h': hour
+      'm': minute
+      's': second
+      'ms': millisecond
+      'us': microsecond
+      'ns': nanosecond
 
     Returns
     -------
     truncated : timestamp
     """
-    return _ops.Truncate(arg, unit).to_expr()
+    return _ops.TimestampTruncate(arg, unit).to_expr()
 
 
 def _timestamp_strftime(arg, format_str):
@@ -1899,6 +2115,35 @@ def _timestamp_time(arg):
     return _ops.Time(arg).to_expr()
 
 
+def _timestamp_date(arg):
+    """
+    Return a Date node for a Timestamp
+    We can then perform certain operations on this node
+    w/o actually instantiating the underlying structure
+    (which is inefficient in pandas/numpy)
+
+    Returns
+    -------
+    Date node
+    """
+    return _ops.Date(arg).to_expr()
+
+
+def _timestamp_sub(left, right):
+    right = as_value_expr(right)
+
+    if isinstance(right, ir.TimestampValue):
+        op = _ops.TimestampDiff(left, right)
+    else:
+        op = _ops.TimestampSub(left, right)  # let the operation validate
+
+    return op.to_expr()
+
+
+_timestamp_add = _binop_expr('__add__', _ops.TimestampAdd)
+_timestamp_radd = _binop_expr('__radd__', _ops.TimestampAdd)
+
+
 _timestamp_value_methods = dict(
     strftime=_timestamp_strftime,
     year=_extract_field('year', _ops.ExtractYear),
@@ -1910,21 +2155,158 @@ _timestamp_value_methods = dict(
     millisecond=_extract_field('millisecond', _ops.ExtractMillisecond),
     truncate=_timestamp_truncate,
     time=_timestamp_time,
+    date=_timestamp_date,
+
+    __sub__=_timestamp_sub,
+    sub=_timestamp_sub,
+
+    __add__=_timestamp_add,
+    add=_timestamp_add,
+
+    __radd__=_timestamp_radd,
+    radd=_timestamp_radd,
+    day_of_week=property(lambda self: _ops.DayOfWeekNode([self]).to_expr()),
 )
 
 
+# ---------------------------------------------------------------------
+# Date API
+
+
+def _date_truncate(arg, unit):
+    """
+    Zero out smaller-size units beyond indicated unit. Commonly used for time
+    series resampling.
+
+    Parameters
+    ----------
+    unit : string, one of below table
+      'Y': year
+      'Q': quarter
+      'M': month
+      'W': week
+      'D': day
+
+    Returns
+    -------
+    truncated : date
+    """
+    return _ops.DateTruncate(arg, unit).to_expr()
+
+
+def _date_sub(left, right):
+    right = as_value_expr(right)
+
+    if isinstance(right, ir.DateValue):
+        op = _ops.DateDiff(left, right)
+    else:
+        op = _ops.DateSub(left, right)  # let the operation validate
+
+    return op.to_expr()
+
+
+_date_add = _binop_expr('__add__', _ops.DateAdd)
+
 _date_value_methods = dict(
     strftime=_timestamp_strftime,
     year=_extract_field('year', _ops.ExtractYear),
     month=_extract_field('month', _ops.ExtractMonth),
     day=_extract_field('day', _ops.ExtractDay),
-)
+    day_of_week=property(lambda self: _ops.DayOfWeekNode([self]).to_expr()),
 
+    truncate=_date_truncate,
+
+    __sub__=_date_sub,
+    sub=_date_sub,
+
+    __add__=_date_add,
+    add=_date_add,
+
+    __radd__=_date_add,
+    radd=_date_add
+)
 
 _add_methods(TimestampValue, _timestamp_value_methods)
 _add_methods(DateValue, _date_value_methods)
 
 
+def _convert_unit(value, unit, to):
+    units = ('W', 'D', 'h', 'm', 's', 'ms', 'us', 'ns')
+    factors = (7, 24, 60, 60, 1000, 1000, 1000)
+
+    monthly_units = ('Y', 'Q', 'M')
+    monthly_factors = (4, 3)
+
+    try:
+        i, j = units.index(unit), units.index(to)
+    except ValueError:
+        try:
+            i, j = monthly_units.index(unit), monthly_units.index(to)
+            factors = monthly_factors
+        except ValueError:
+            raise ValueError('Cannot convert to or from '
+                             'non-fixed-length interval')
+
+    factor = functools.reduce(operator.mul, factors[i:j], 1)
+
+    if i < j:
+        return value * factor
+    elif i > j:
+        return value // factor
+    else:
+        return value
+
+
+def _to_unit(arg, target_unit):
+    if arg.meta.unit != target_unit:
+        arg = _convert_unit(arg, arg.meta.unit, target_unit)
+        arg.unit = target_unit
+    return arg
+
+
+def _interval_property(target_unit):
+    return property(functools.partial(_to_unit, target_unit=target_unit))
+
+
+_interval_add = _binop_expr('__add__', _ops.IntervalAdd)
+_interval_radd = _binop_expr('__radd__', _ops.IntervalAdd)
+_interval_mul = _binop_expr('__mul__', _ops.IntervalMultiply)
+_interval_rmul = _binop_expr('__rmul__', _ops.IntervalMultiply)
+_interval_floordiv = _binop_expr('__floordiv__', _ops.IntervalFloorDivide)
+
+_interval_value_methods = dict(
+    to_unit=_to_unit,
+    years=_interval_property('Y'),
+    quarters=_interval_property('Q'),
+    months=_interval_property('M'),
+    weeks=_interval_property('W'),
+    days=_interval_property('D'),
+    hours=_interval_property('h'),
+    minutes=_interval_property('m'),
+    seconds=_interval_property('s'),
+    milliseconds=_interval_property('ms'),
+    microseconds=_interval_property('us'),
+    nanoseconds=_interval_property('ns'),
+
+    __add__=_interval_add,
+    add=_interval_add,
+
+    __radd__=_interval_radd,
+    radd=_interval_radd,
+
+    __mul__=_interval_mul,
+    mul=_interval_mul,
+
+    __rmul__=_interval_rmul,
+    rmul=_interval_rmul,
+
+    __floordiv__=_interval_floordiv,
+    floordiv=_interval_floordiv
+)
+
+_add_methods(IntervalValue, _interval_value_methods)
+
+
 # ---------------------------------------------------------------------
 # Time API
 
@@ -1961,8 +2343,54 @@ def between_time(arg, lower, upper, timezone=None):
     return op.to_expr()
 
 
+def _time_truncate(arg, unit):
+    """
+    Zero out smaller-size units beyond indicated unit. Commonly used for time
+    series resampling.
+
+    Parameters
+    ----------
+    unit : string, one of below table
+      'h': hour
+      'm': minute
+      's': second
+      'ms': millisecond
+      'us': microsecond
+      'ns': nanosecond
+
+    Returns
+    -------
+    truncated : time
+    """
+    return _ops.TimeTruncate(arg, unit).to_expr()
+
+
+def _time_sub(left, right):
+    right = as_value_expr(right)
+
+    if isinstance(right, ir.TimeValue):
+        op = _ops.TimeDiff(left, right)
+    else:
+        op = _ops.TimeSub(left, right)  # let the operation validate
+
+    return op.to_expr()
+
+
+_time_add = _binop_expr('__add__', _ops.TimeAdd)
+
+
 _time_value_methods = dict(
     between=between_time,
+    truncate=_time_truncate,
+
+    __sub__=_time_sub,
+    sub=_time_sub,
+
+    __add__=_time_add,
+    add=_time_add,
+
+    __radd__=_time_add,
+    radd=_time_add
 )
 
 _add_methods(TimeValue, _time_value_methods)
@@ -2386,7 +2814,31 @@ def _table_materialize(table):
         return TableExpr(op)
 
 
-def mutate(table, exprs=None, **kwds):
+def add_column(table, expr, name=None):
+    """
+    Add indicated column expression to table, producing a new table. Note:
+    this is a shortcut for performing a projection having the same effect.
+
+    Returns
+    -------
+    modified_table : TableExpr
+    """
+    warnings.warn('add_column is deprecated, use mutate(name=expr, ...)',
+                  DeprecationWarning)
+    if name is not None:
+        return table.mutate(**{name: expr})
+    else:
+        return table.mutate(expr)
+
+
+def _safe_get_name(expr):
+    try:
+        return expr.get_name()
+    except _com.ExpressionError:
+        return None
+
+
+def mutate(table, exprs=None, **mutations):
     """
     Convenience function for table projections involving adding columns
 
@@ -2394,27 +2846,64 @@ def mutate(table, exprs=None, **kwds):
     ----------
     exprs : list, default None
       List of named expressions to add as columns
-    kwds : keywords for new columns
-
-    Examples
-    --------
-    >>> expr = table.mutate(qux=table.foo + table.bar, baz=5)  # doctest: +SKIP
+    mutations : keywords for new columns
 
     Returns
     -------
     mutated : TableExpr
+
+    Examples
+    --------
+    Using keywords arguments to name the new columns
+
+    >>> import ibis
+    >>> table = ibis.table([('foo', 'double'), ('bar', 'double')], name='t')
+    >>> expr = table.mutate(qux=table.foo + table.bar, baz=5)
+    >>> expr  # doctest: +NORMALIZE_WHITESPACE
+    ref_0
+    UnboundTable[table]
+      name: t
+      schema:
+        foo : double
+        bar : double
+    <BLANKLINE>
+    Selection[table]
+      table:
+        Table: ref_0
+      selections:
+        Table: ref_0
+        baz = Literal[int8]
+          5
+        qux = Add[double*]
+          left:
+            foo = Column[double*] 'foo' from table
+              ref_0
+          right:
+            bar = Column[double*] 'bar' from table
+              ref_0
+
+    Using the :meth:`ibis.expr.types.Expr.name` method to name the new columns
+
+    >>> new_columns = [ibis.literal(5).name('baz',),
+    ...                (table.foo + table.bar).name('qux')]
+    >>> expr2 = table.mutate(new_columns)
+    >>> expr.equals(expr2)
+    True
     """
     if exprs is None:
         exprs = []
     else:
         exprs = util.promote_list(exprs)
 
-    for k, v in sorted(kwds.items()):
+    for k, v in sorted(mutations.items()):
         if util.is_function(v):
             v = v(table)
         else:
             v = as_value_expr(v)
-        exprs.append(v.name(k))
+
+        # TODO(phillipc): Fix this by making expressions hashable
+        named_v = v.name(k) if _safe_get_name(v) != k else v
+        exprs.append(named_v)
 
     has_replacement = False
     for expr in exprs:
@@ -2454,6 +2943,84 @@ def projection(table, exprs):
     Returns
     -------
     projection : TableExpr
+
+    Notes
+    -----
+    Passing an aggregate function to this method will broadcast the aggregate's
+    value over the number of rows in the table. See the examples section for
+    more details.
+
+    Examples
+    --------
+    Simple projection
+
+    >>> import ibis
+    >>> fields = [('a', 'int64'), ('b', 'double')]
+    >>> t = ibis.table(fields, name='t')
+    >>> proj = t.projection([t.a, (t.b + 1).name('b_plus_1')])
+    >>> proj  # doctest: +NORMALIZE_WHITESPACE
+    ref_0
+    UnboundTable[table]
+      name: t
+      schema:
+        a : int64
+        b : double
+    <BLANKLINE>
+    Selection[table]
+      table:
+        Table: ref_0
+      selections:
+        a = Column[int64*] 'a' from table
+          ref_0
+        b_plus_1 = Add[double*]
+          left:
+            b = Column[double*] 'b' from table
+              ref_0
+          right:
+            Literal[int8]
+              1
+    >>> proj2 = t[t.a, (t.b + 1).name('b_plus_1')]
+    >>> proj.equals(proj2)
+    True
+
+    Aggregate projection
+
+    >>> agg_proj = t[t.a.sum().name('sum_a'), t.b.mean().name('mean_b')]
+    >>> agg_proj  # doctest: +NORMALIZE_WHITESPACE, +ELLIPSIS
+    ref_0
+    UnboundTable[table]
+      name: t
+      schema:
+        a : int64
+        b : double
+    <BLANKLINE>
+    Selection[table]
+      table:
+        Table: ref_0
+      selections:
+        sum_a = WindowOp[int64*]
+          sum_a = Sum[int64]
+            a = Column[int64*] 'a' from table
+              ref_0
+            where:
+              None
+          <ibis.expr.window.Window object at 0x...>
+        mean_b = WindowOp[double*]
+          mean_b = Mean[double]
+            b = Column[double*] 'b' from table
+              ref_0
+            where:
+              None
+          <ibis.expr.window.Window object at 0x...>
+
+    Note the ``<ibis.expr.window.Window>`` objects here, their existence means
+    that the result of the aggregation will be broadcast across the number of
+    rows in the input column. The purpose of this expression rewrite is to make
+    it easy to write column/scalar-aggregate operations like
+
+    .. code-block:: python
+
+       t[(t.a - t.a.mean()).name('demeaned_a')]
     """
     import ibis.expr.analysis as L
 
@@ -2545,6 +3112,7 @@ _table_methods = dict(
     set_column=_table_set_column,
     filter=filter,
     materialize=_table_materialize,
+    add_column=add_column,
     mutate=mutate,
     projection=projection,
     select=projection,
