@@ -20,16 +20,17 @@ import toolz
 
 from ibis.expr.types import TableColumn  # noqa
 
-from ibis.expr.datatypes import HasSchema, Schema
+from ibis.expr.schema import HasSchema, Schema
 from ibis.expr.rules import value, string, number, integer, boolean, list_of
 from ibis.expr.types import (Node, as_value_expr, Expr,
                              ValueExpr, ColumnExpr, TableExpr,
-                             ValueOp, _safe_repr)
+                             ValueOp, _safe_repr, distinct_roots)
 import ibis.common as com
 import ibis.expr.datatypes as dt
 import ibis.expr.rules as rules
 import ibis.expr.types as ir
 import ibis.util as util
+import ibis.compat as compat
 
 
 _table_names = ('t{:d}'.format(i) for i in itertools.count())
@@ -249,19 +250,16 @@ class NullIfZero(ValueOp):
     output_type = rules.type_of_arg(0)
 
 
-def _coalesce_upcast(self):
-    # TODO: how much validation is necessary that the call is valid and can
-    # succeed?
-    first_value = self.args[0]
+class IsNan(ValueOp):
 
-    if isinstance(first_value, ir.IntegerValue):
-        out_type = 'int64'
-    elif isinstance(first_value, ir.FloatingValue):
-        out_type = 'double'
-    else:
-        out_type = first_value.type()
+    input_type = [rules.floating]
+    output_type = rules.shape_like_arg(0, 'boolean')
 
-    return rules.shape_like_args(self.args, out_type)
+
+class IsInf(ValueOp):
+
+    input_type = [rules.floating]
+    output_type = rules.shape_like_arg(0, 'boolean')
 
 
 class CoalesceLike(ValueOp):
@@ -272,7 +270,20 @@ class CoalesceLike(ValueOp):
     # DOUBLE; use CAST() when inserting into a smaller numeric column
 
     input_type = rules.varargs(rules.value)
-    output_type = _coalesce_upcast
+
+    def output_type(self):
+        # TODO: how much validation is necessary that the call is valid and can
+        # succeed?
+        first_value = self.args[0]
+
+        if isinstance(first_value, ir.IntegerValue):
+            out_type = 'int64'
+        elif isinstance(first_value, ir.FloatingValue):
+            out_type = 'double'
+        else:
+            out_type = first_value.type()
+
+        return rules.shape_like_args(self.args, out_type)
 
 
 class Coalesce(CoalesceLike):
@@ -542,7 +553,14 @@ class FuzzySearch(BooleanValueOp):
 
 
 class StringSQLLike(FuzzySearch):
-    pass
+
+    input_type = [
+        string,
+        string(name='pattern'),
+        rules.instance_of(
+            six.string_types, name='escape', default=None, optional=True
+        ),
+    ]
 
 
 class RegexSearch(FuzzySearch):
@@ -567,6 +585,20 @@ class StringReplace(ValueOp):
     output_type = rules.shape_like_arg(0, 'string')
 
 
+class StringSplit(ValueOp):
+
+    input_type = [string, string(name='delimiter')]
+    output_type = rules.shape_like_arg(0, 'array<string>')
+
+
+class StringConcat(ValueOp):
+
+    input_type = rules.varargs(rules.string)
+
+    def output_type(self):
+        return rules.shape_like_args(self.args, 'string')
+
+
 class ParseURL(ValueOp):
 
     input_type = [string, rules.string_options(['PROTOCOL', 'HOST', 'PATH',
@@ -670,6 +702,18 @@ class Count(Reduction):
         return ir.Int64Scalar
 
 
+class Arbitrary(Reduction):
+
+    input_type = [rules.column,
+                  rules.string_options(['first', 'last', 'heavy'],
+                                       name='how', default='first'),
+                  boolean(name='where', optional=True)]
+
+    def output_type(self):
+        # Scalar but type of caller
+        return self.args[0].type().scalar_type()
+
+
 def _sum_output_type(self):
     arg = self.args[0]
     if isinstance(arg, (ir.IntegerValue, ir.BooleanValue)):
@@ -747,9 +791,10 @@ class MultiQuantile(Quantile):
 
 class VarianceBase(Reduction):
 
-    input_type = [rules.column, boolean(name='where', optional=True),
+    input_type = [rules.column,
                   rules.string_options(['sample', 'pop'],
-                                       name='how', optional=True)]
+                                       name='how', optional=True),
+                  boolean(name='where', optional=True)]
     output_type = rules.scalar_output(_mean_output_type)
 
 
@@ -795,13 +840,17 @@ class HLLCardinality(Reduction):
 
     def output_type(self):
         # Impala 2.0 and higher returns a DOUBLE
-        return ir.DoubleScalar
+        # return ir.DoubleScalar
+        return ir.Int64Scalar
 
 
 class GroupConcat(Reduction):
 
-    input_type = [rules.column, string(name='sep', default=',')]
-    # boolean(name='where', optional=True)]
+    input_type = [
+        rules.column,
+        string(name='sep', default=','),
+        boolean(name='where', optional=True),
+    ]
 
     def output_type(self):
         return ir.StringScalar
@@ -829,27 +878,41 @@ class AnalyticOp(ValueOp):
 
 class WindowOp(ValueOp):
 
-    output_type = rules.type_of_arg(0)
+    def output_type(self):
+        return self.args[0].type().array_type()
 
     def __init__(self, expr, window):
         from ibis.expr.window import propagate_down_window
         if not is_analytic(expr):
-            raise com.IbisInputError('Expression does not contain a valid '
-                                     'window operation')
+            raise com.IbisInputError(
+                'Expression does not contain a valid window operation'
+            )
 
         table = ir.find_base_table(expr)
         if table is not None:
             window = window.bind(table)
 
         expr = propagate_down_window(expr, window)
-
-        ValueOp.__init__(self, expr, window)
+        super(WindowOp, self).__init__(expr, window)
 
     def over(self, window):
         existing_window = self.args[1]
         new_window = existing_window.combine(window)
         return WindowOp(self.args[0], new_window)
 
+    def root_tables(self):
+        window = self.args[1]
+        result = list(toolz.unique(
+            itertools.chain(
+                self.args[0]._root_tables(),
+                distinct_roots(
+                    *itertools.chain(window._order_by, window._group_by)
+                )
+            ),
+            key=id
+        ))
+        return result
+
 
 def is_analytic(expr, exclude_windows=False):
     def _is_analytic(op):
@@ -1086,7 +1149,7 @@ class DistinctColumn(ValueOp):
 
 class CountDistinct(Reduction):
 
-    input_type = [rules.column]
+    input_type = [rules.column, boolean(name='where', optional=True)]
 
     def output_type(self):
         return ir.Int64Scalar
@@ -1186,7 +1249,7 @@ class SimpleCaseBuilder(object):
         case_expr = as_value_expr(case_expr)
         result_expr = as_value_expr(result_expr)
 
-        if not self.base._can_compare(case_expr):
+        if not rules.comparable(self.base, case_expr):
             raise TypeError('Base expression and passed case are not '
                             'comparable')
 
@@ -1298,18 +1361,21 @@ class SimpleCase(ValueOp):
 
     def __init__(self, base, cases, results, default):
         assert len(cases) == len(results)
-        ValueOp.__init__(self, base, cases, results, default)
+        super(SimpleCase, self).__init__(base, cases, results, default)
 
     def root_tables(self):
         base, cases, results, default = self.args
-        all_exprs = [base] + cases + results
-        if default is not None:
-            all_exprs.append(default)
+        all_exprs = [base] + cases + results + (
+            [] if default is None else [default]
+        )
         return ir.distinct_roots(*all_exprs)
 
     def output_type(self):
         base, cases, results, default = self.args
-        out_exprs = results + [default]
+        out_exprs = list(filter(
+            lambda expr: expr is not None,
+            results + [default]
+        ))
         typename = rules.highest_precedence_type(out_exprs)
         return rules.shape_like(base, typename)
 
@@ -1322,13 +1388,11 @@ class SearchedCase(ValueOp):
 
     def __init__(self, cases, results, default):
         assert len(cases) == len(results)
-        ValueOp.__init__(self, cases, results, default)
+        super(SearchedCase, self).__init__(cases, results, default)
 
     def root_tables(self):
         cases, results, default = self.args
-        all_exprs = cases + results
-        if default is not None:
-            all_exprs.append(default)
+        all_exprs = cases + results + ([] if default is None else [default])
         return ir.distinct_roots(*all_exprs)
 
     def output_type(self):
@@ -1412,11 +1476,15 @@ def _clean_join_predicates(left, right, predicates):
 
 
 def _validate_join_predicates(left, right, predicates):
+    from ibis.expr.analysis import fully_originate_from
+
     # Validate join predicates. Each predicate must be valid jointly when
     # considering the roots of each input table
-    from ibis.expr.analysis import CommonSubexpr
-    validator = CommonSubexpr([left, right])
-    validator.validate_all(predicates)
+    for predicate in predicates:
+        if not fully_originate_from(predicate, [left, right]):
+            raise com.RelationError('The expression {!r} does not fully '
+                                    'originate from dependencies of the table '
+                                    'expression.'.format(predicate))
 
 
 class Join(TableNode):
@@ -1781,12 +1849,19 @@ class Selection(TableNode, HasSchema):
         pass
 
     def is_ancestor(self, other):
+        import ibis.expr.lineage as lin
+
         if isinstance(other, ir.Expr):
             other = other.op()
 
         if self.equals(other):
             return True
 
+        expr = self.to_expr()
+        fn = lambda e: (lin.proceed, e.op())  # noqa: E731
+        for child in lin.traverse(fn, expr):
+            if child.equals(other):
+                return True
         return False
 
     # Operator combination / fusion logic
@@ -2070,22 +2145,22 @@ class Xor(LogicalBinaryOp):
 class Comparison(BinaryOp, BooleanValueOp):
 
     def _maybe_cast_args(self, left, right):
-        if left._can_implicit_cast(right):
-            return left, left._implicit_cast(right)
+        # it might not be necessary?
+        with compat.suppress(com.IbisTypeError):
+            return left, ir.cast(right, left)
 
-        if right._can_implicit_cast(left):
-            return right._implicit_cast(left), right
+        with compat.suppress(com.IbisTypeError):
+            return ir.cast(left, right), right
 
         return left, right
 
     def output_type(self):
-        self._assert_can_compare()
+        if not rules.comparable(self.left, self.right):
+            raise TypeError('Arguments with datatype {} and {} are '
+                            'not comparable'.format(self.left.type(),
+                                                    self.right.type()))
         return rules.shape_like_args(self.args, 'boolean')
 
-    def _assert_can_compare(self):
-        if not self.left._can_compare(self.right):
-            raise TypeError('Cannot compare argument types')
-
 
 class Equals(Comparison):
     pass
@@ -2118,20 +2193,19 @@ class IdenticalTo(Comparison):
 class Between(BooleanValueOp):
 
     input_type = [
-        rules.value,
+        rules.value(),
         rules.value(name='lower_bound'),
         rules.value(name='upper_bound')
     ]
 
     def output_type(self):
-        self._assert_can_compare()
-        return rules.shape_like_args(self.args, 'boolean')
+        arg, lower, upper = self.args
 
-    def _assert_can_compare(self):
-        expr, lower, upper = self.args
-        if not expr._can_compare(lower) or not expr._can_compare(upper):
+        if not (rules.comparable(arg, lower) and rules.comparable(arg, upper)):
             raise TypeError('Arguments are not comparable')
 
+        return rules.shape_like_args(self.args, 'boolean')
+
 
 class BetweenTime(Between):
 
@@ -2278,74 +2352,114 @@ class TimestampUnaryOp(UnaryOp):
     input_type = [rules.timestamp]
 
 
-_truncate_units = [
-    'Y', 'Q', 'M', 'D', 'J', 'W', 'H', 'MI'
-]
+_truncate_units = dict(
+    Y='Y',
+    y='Y',
+    year='Y',
+    YEAR='Y',
+    YYYY='Y',
+    SYYYY='Y',
+    YYY='Y',
+    YY='Y',
 
-_truncate_unit_aliases = {
-    # year
-    'YYYY': 'Y',
-    'SYYYY': 'Y',
-    'YEAR': 'Y',
-    'YYY': 'Y',
-    'YY': 'Y',
+    Q='Q',
+    q='Q',
+    quarter='Q',
+    QUARTER='Q',
 
-    # month
-    'MONTH': 'M',
-    'MON': 'M',
+    M='M',
+    month='M',
+    MONTH='M',
 
-    # week
-    'WW': 'W',
+    w='W',
+    W='W',
+    week='W',
+    WEEK='W',
 
-    # day of month
+    d='D',
+    D='D',
+    J='D',
+    day='D',
+    DAY='D',
 
-    # starting day of week
+    h='h',
+    H='h',
+    HH24='h',
+    hour='h',
+    HOUR='h',
 
-    # hour
-    'HOUR': 'H',
-    'HH24': 'H',
+    m='m',
+    MI='m',
+    minute='m',
+    MINUTE='m',
 
-    # minute
-    'MINUTE': 'MI',
+    s='s',
+    second='s',
+    SECOND='s',
 
-    # second
+    ms='ms',
+    millisecond='ms',
+    MILLISECOND='ms',
 
-    # millisecond
+    us='us',
+    microsecond='ms',
+    MICROSECOND='ms',
 
-    # microsecond
-}
+    ns='ns',
+    nanosecond='ns',
+    NANOSECOND='ns',
+)
 
 
-def _truncate_unit_validate(unit):
-    orig_unit = unit
-    unit = unit.upper()
-
+def _truncate_unit_validate(orig_unit):
     # TODO: truncate autocompleter
 
-    unit = _truncate_unit_aliases.get(unit, unit)
-    valid_units = set(_truncate_units)
+    if orig_unit not in _truncate_units:
+        valid_units = set(_truncate_units.keys())
+        raise com.IbisInputError(
+            'Passed unit {} was not one of {}'.format(
+                orig_unit, repr(valid_units)
+            )
+        )
 
-    if unit not in valid_units:
-        raise com.IbisInputError('Passed unit {0} was not one of'
-                                 ' {1}'.format(orig_unit,
-                                               repr(valid_units)))
-
-    return unit
+    return _truncate_units[orig_unit]
 
 
-class Truncate(ValueOp):
+class TimestampTruncate(ValueOp):
 
     input_type = [
         rules.timestamp,
-        rules.string_options(_truncate_units, name='unit',
-                             validator=_truncate_unit_validate)]
-    output_type = rules.shape_like_arg(0, 'timestamp')
+        rules.string_options(['Y', 'Q', 'M', 'W', 'D',
+                              'h', 'm', 's', 'ms', 'us', 'ns'], name='unit',
+                             validator=_truncate_unit_validate)
+    ]
+    output_type = rules.shape_like_arg(0, dt.timestamp)
+
+
+class DateTruncate(ValueOp):
+
+    input_type = [
+        rules.date,
+        rules.string_options(['Y', 'Q', 'M', 'W', 'D'], name='unit',
+                             validator=_truncate_unit_validate)
+    ]
+    output_type = rules.shape_like_arg(0, dt.date)
+
+
+class TimeTruncate(ValueOp):
+
+    input_type = [
+        rules.time,
+        rules.string_options(['h', 'm', 's', 'ms', 'us', 'ns'], name='unit',
+                             validator=_truncate_unit_validate)
+    ]
+    output_type = rules.shape_like_arg(0, dt.time)
 
 
 class Strftime(ValueOp):
 
     input_type = [rules.temporal, rules.string(name='format_str')]
-    output_type = rules.shape_like_arg(0, 'string')
+    output_type = rules.shape_like_arg(0, dt.string)
 
 
 class ExtractTemporalField(TemporalUnaryOp):
@@ -2366,6 +2480,37 @@ class ExtractMonth(ExtractTemporalField):
     pass
 
 
+class DayOfWeekIndex(UnaryOp):
+
+    input_type = [rules.one_of((dt.date, dt.timestamp))]
+    output_type = rules.shape_like_arg(0, 'int32')
+
+
+class DayOfWeekName(UnaryOp):
+
+    input_type = [rules.one_of((dt.date, dt.timestamp))]
+    output_type = rules.shape_like_arg(0, 'string')
+
+
+class DayOfWeek(ir.Expr):
+
+    def index(self):
+        arg, = self.op().args
+        return DayOfWeekIndex(arg).to_expr()
+
+    def full_name(self):
+        arg, = self.op().args
+        return DayOfWeekName(arg).to_expr()
+
+
+class DayOfWeekNode(ir.Node):
+
+    input_type = [rules.one_of((dt.date, dt.timestamp))]
+
+    def output_type(self):
+        return DayOfWeek
+
+
 class ExtractDay(ExtractTemporalField):
     pass
 
@@ -2391,6 +2536,11 @@ class Time(UnaryOp):
     output_type = rules.shape_like_arg(0, 'time')
 
 
+class Date(UnaryOp):
+
+    output_type = rules.shape_like_arg(0, 'date')
+
+
 class TimestampFromUNIX(ValueOp):
 
     input_type = [value, rules.string_options(['s', 'ms', 'us'], name='unit')]
@@ -2418,12 +2568,107 @@ class Hash(ValueOp):
     output_type = rules.shape_like_arg(0, 'int64')
 
 
-class TimestampDelta(ValueOp):
+class DateAdd(Add):
 
-    input_type = [rules.timestamp, rules.timedelta(name='offset')]
+    input_type = [rules.date, rules.interval(units=['Y', 'Q', 'M', 'W', 'D'])]
+    output_type = rules.shape_like_arg(0, 'date')
+
+
+class DateSub(Subtract):
+
+    input_type = [rules.date, rules.interval(units=['Y', 'Q', 'M', 'W', 'D'])]
+    output_type = rules.shape_like_arg(0, 'date')
+
+
+class DateDiff(BinaryOp):
+
+    input_type = [rules.date, rules.date]
+    output_type = rules.shape_like_arg(0, dt.Interval('D'))
+
+
+class TimeAdd(Add):
+
+    input_type = [rules.time, rules.interval(units=['h', 'm', 's'])]
+    output_type = rules.shape_like_arg(0, 'time')
+
+
+class TimeSub(Subtract):
+
+    input_type = [rules.time, rules.interval(units=['h', 'm', 's'])]
+    output_type = rules.shape_like_arg(0, 'time')
+
+
+class TimeDiff(BinaryOp):
+
+    input_type = [rules.time, rules.time]
+    output_type = rules.shape_like_arg(0, dt.Interval('s'))
+
+
+class TimestampAdd(Add):
+
+    input_type = [rules.timestamp, rules.interval]
     output_type = rules.shape_like_arg(0, 'timestamp')
 
 
+class TimestampSub(Subtract):
+
+    input_type = [rules.timestamp, rules.interval]
+    output_type = rules.shape_like_arg(0, 'timestamp')
+
+
+class TimestampDiff(BinaryOp):
+
+    input_type = [rules.timestamp, rules.timestamp]
+    output_type = rules.shape_like_arg(0, dt.Interval('s'))
+
+
+class IntervalAdd(Add):
+
+    input_type = [rules.interval, rules.interval]
+
+    def output_type(self):
+        left, right = self.args
+
+        # should upconvert to the smaller unit?
+        # value_type = left.to_unit(right.type().unit).type()
+        return rules.shape_like(left, left.type())
+
+
+class IntervalMultiply(Multiply):
+
+    def output_type(self):
+        helper = rules.IntervalPromoter(self.left, self.right, operator.mul)
+        return helper.get_result()
+
+
+class IntervalFloorDivide(FloorDivide):
+
+    input_type = [rules.interval, number]
+
+    def output_type(self):
+        return rules.shape_like(self.args[0],  self.args[0].type())
+
+
+class IntervalFromInteger(ValueOp):
+
+    input_type = [
+        rules.integer,
+        rules.string_options([
+            'Y', 'Q', 'M', 'W', 'D',
+            'h', 'm', 's', 'ms', 'us', 'ns'
+        ], name='unit')
+    ]
+
+    @property
+    def resolution(self):
+        return dt.Interval(self.unit).resolution
+
+    def output_type(self):
+        arg, unit = self.args
+        type = dt.Interval(unit, arg.type())
+        return rules.shape_like(arg, type)
+
+
 class ArrayLength(UnaryOp):
 
     input_type = [rules.array(dt.any)]
@@ -2508,6 +2753,26 @@ class MapValueForKey(ValueOp):
         return rules.shape_like(self.args[0], map_type.value_type)
 
 
+class MapValueOrDefaultForKey(ValueOp):
+
+    input_type = [
+        rules.map(dt.any, dt.any),
+        rules.one_of((dt.string, dt.int_), name='key'),
+        rules.value(name='default')
+    ]
+
+    def output_type(self):
+        map_type = self.args[0].type()
+        value_type = map_type.value_type
+        default_type = self.default.type()
+
+        if default_type is not dt.null and value_type != default_type:
+            raise ValueError("default type: {}  must be the same "
+                             "as the map value_type {}".format(
+                                 default_type, value_type))
+        return rules.shape_like(self.args[0], map_type.value_type)
+
+
 class MapKeys(ValueOp):
 
     input_type = [rules.map(dt.any, dt.any)]
