@@ -12,16 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import abc
+import six
 import sys
-import functools
 import operator
 
 from collections import Counter
 
-import six
-
-from toolz import first
-
 from ibis.common import IbisTypeError
 import ibis.expr.datatypes as dt
 import ibis.expr.types as ir
@@ -88,6 +85,28 @@ class BinaryPromoter(object):
             raise TypeError('String and non-string incompatible')
 
 
+class IntervalPromoter(BinaryPromoter):
+    """Infers the output type of the binary interval operation
+
+    This is a slightly modified version of BinaryPromoter, it converts
+    back and forth between the interval and its inner value.
+
+    This trick reuses the numeric type promotion logics.
+    Any non-integer output type raises a TypeError.
+    """
+
+    def __init__(self, left, right, op):
+        left_type = left.type()
+        value_type = shape_like(left, left_type.value_type)
+        self.unit = left_type.unit
+        super(IntervalPromoter, self).__init__(value_type(left), right, op)
+
+    def get_result(self):
+        promoted_value_type = self._get_type()
+        promoted_type = dt.Interval(self.unit, promoted_value_type)
+        return shape_like_args(self.args, promoted_type)
+
+
 def _decimal_promoted_type(args):
     max_precision = max_scale = ~sys.maxsize
     for arg in args:
@@ -103,8 +122,6 @@ class PowerPromoter(BinaryPromoter):
         super(PowerPromoter, self).__init__(left, right, operator.pow)
 
     def _get_type(self):
-        rval = self.args[1].op()
-
         if util.any_of(self.args, ir.FloatingValue):
             if util.any_of(self.args, ir.DoubleValue):
                 return 'double'
@@ -112,10 +129,8 @@ class PowerPromoter(BinaryPromoter):
                 return 'float'
         elif util.any_of(self.args, ir.DecimalValue):
             return _decimal_promoted_type(self.args)
-        elif isinstance(rval, ir.Literal) and rval.value < 0:
-            return 'double'
         elif util.all_of(self.args, ir.IntegerValue):
-            return self._get_int_type()
+            return 'double'
         else:
             raise NotImplementedError(
                 'Operands {}, {} not supported for binary operation {}'.format(
@@ -125,73 +140,6 @@ class PowerPromoter(BinaryPromoter):
             )
 
 
-# Impala type precedence (more complex in other database implementations)
-# timestamp
-# double
-# float
-# decimal
-# bigint
-# int
-# smallint
-# tinyint
-# boolean
-# string
-# binary
-
-
-_SCALAR_TYPE_PRECEDENCE = {
-    'timestamp': 11,
-    'double': 10,
-    'float': 9,
-    'decimal': 8,
-    'int64': 7,
-    'int32': 6,
-    'int16': 5,
-    'int8': 4,
-    'boolean': 3,
-    'string': 2,
-    'binary': 1,
-    'null': 0,
-}
-
-
-def higher_precedence(left, right):
-    left_name = left.name.lower()
-    right_name = right.name.lower()
-
-    if (left_name in _SCALAR_TYPE_PRECEDENCE and
-            right_name in _SCALAR_TYPE_PRECEDENCE):
-        left_prec = _SCALAR_TYPE_PRECEDENCE[left_name]
-        right_prec = _SCALAR_TYPE_PRECEDENCE[right_name]
-        _, highest_type = max(
-            ((left_prec, left), (right_prec, right)),
-            key=first
-        )
-        return highest_type
-
-    # TODO(phillipc): Ensure that left and right are API compatible
-
-    if isinstance(left, dt.Array):
-        return dt.Array(higher_precedence(left.value_type, right.value_type))
-
-    if isinstance(left, dt.Map):
-        return dt.Map(
-            higher_precedence(left.key_type, right.key_type),
-            higher_precedence(left.value_type, right.value_type)
-        )
-
-    if isinstance(left, dt.Struct):
-        if left.names != right.names:
-            raise TypeError('Struct names are not equal')
-        return dt.Struct(
-            left.names,
-            list(map(higher_precedence, left.types, right.types))
-        )
-    raise TypeError(
-        'Cannot compute precedence for {} and {} types'.format(left, right)
-    )
-
-
 def highest_precedence_type(exprs):
     # Return the highest precedence type from the passed expressions. Also
     # verifies that there are valid implicit casts between any of the types and
@@ -199,17 +147,8 @@ def highest_precedence_type(exprs):
     if not exprs:
         raise ValueError('Must pass at least one expression')
 
-    expr_types = {expr.type() for expr in exprs}
-    highest_type = functools.reduce(higher_precedence, expr_types)
-
-    for expr in exprs:
-        if not expr._can_cast_implicit(highest_type):
-            raise TypeError(
-                'Expression with type {0} cannot be implicitly casted to {1}'
-                .format(expr.type(), highest_type)
-            )
-
-    return highest_type
+    expr_dtypes = {expr.type() for expr in exprs}
+    return dt.highest_precedence(expr_dtypes)
 
 
 def _int_bounds_promotion(ltype, rtype, op):
@@ -234,43 +173,16 @@ def _int_one_literal_promotion(atype, lit_val, op):
 
 
 def _smallest_int_containing(values, allow_overflow=False):
-    containing_types = [int_literal_class(x, allow_overflow=allow_overflow)
+    containing_types = [dt.infer(x, allow_overflow=allow_overflow)
                         for x in values]
     return _largest_int(containing_types)
 
 
-def int_literal_class(value, allow_overflow=False):
-    if -128 <= value <= 127:
-        t = 'int8'
-    elif -32768 <= value <= 32767:
-        t = 'int16'
-    elif -2147483648 <= value <= 2147483647:
-        t = 'int32'
-    else:
-        if value < -9223372036854775808 or value > 9223372036854775807:
-            if not allow_overflow:
-                raise OverflowError(value)
-        t = 'int64'
-    return dt.validate_type(t)
-
-
 def _largest_int(int_types):
     nbytes = max(t._nbytes for t in int_types)
     return dt.validate_type('int%d' % (8 * nbytes))
 
 
-class ImplicitCast(object):
-
-    def __init__(self, value_type, implicit_targets):
-        self.value_type = value_type
-        self.implicit_targets = implicit_targets
-
-    def can_cast(self, target):
-        base_type = target.name.lower()
-        return (base_type in self.implicit_targets or
-                target == self.value_type)
-
-
 # ----------------------------------------------------------------------
 # Input / output type rules and validation
 
@@ -320,7 +232,9 @@ class Argument(object):
         self.optional = optional
         self.validator = validator
         self.doc = doc
-        self.as_value_expr = as_value_expr or ir.literal
+        self.as_value_expr = (
+            ir.literal if as_value_expr is None else as_value_expr
+        )
 
     def validate(self, args, i):
         arg = args[i]
@@ -339,6 +253,11 @@ class Argument(object):
     def _validate(self, args, i):
         raise NotImplementedError
 
+    def __str__(self):
+        fields = {'name', 'doc', 'optional'}
+        return str({k: v for k, v in self.__dict__.items()
+                    if k in fields and v is not None})
+
 
 def _to_argument(val):
     if isinstance(val, dt.DataType):
@@ -437,12 +356,7 @@ def array_output(rule):
 def shape_like_flatargs(out_type):
 
     def output_type(self):
-        flattened = []
-        for arg in self.args:
-            if isinstance(arg, (list, tuple)):
-                flattened.extend(arg)
-            else:
-                flattened.append(arg)
+        flattened = list(self.flat_args())
         return shape_like_args(flattened, out_type)
 
     return output_type
@@ -456,24 +370,6 @@ def shape_like_arg(i, out_type):
     return output_type
 
 
-def numeric_highest_promote(i):
-
-    def output_type(self):
-        arg = self.args[i]
-
-        if isinstance(arg, ir.DecimalValue):
-            return arg._factory
-        elif isinstance(arg, ir.FloatingValue):
-            # Impala upcasts float to double in this op
-            return shape_like(arg, 'double')
-        elif isinstance(arg, ir.IntegerValue):
-            return shape_like(arg, 'int64')
-        else:
-            raise NotImplementedError
-
-    return output_type
-
-
 def type_of_arg(i):
 
     def output_type(self):
@@ -519,14 +415,15 @@ class AnyTyped(Argument):
         return arg
 
     def _type_matches(self, arg):
-        for t in self.types:
-            if (isinstance(t, dt.DataType) or
-                    isinstance(t, type) and issubclass(t, dt.DataType)):
-                if t.can_implicit_cast(arg.type()):
-                    return True
-            else:
-                if isinstance(arg, t):
+        for expected in self.types:
+            if isinstance(expected, dt.DataType):
+                dtype = arg.type()
+                if dtype.issubtype(expected):
                     return True
+                return dtype.castable(expected)
+            elif isinstance(arg, expected):
+                return True
+
         return False
 
 
@@ -547,15 +444,6 @@ class ArrayValueTyped(ValueTyped):
             dt.Array(value_type), *args, **kwargs
         )
 
-    def _validate(self, args, i):
-        arg = super(ArrayValueTyped, self)._validate(args, i)
-        type, = self.types
-        arg_type = arg.type()
-        if (arg_type.equals(dt.Array(dt.any)) or
-                arg_type.equals(dt.Array(dt.null))):
-            return arg.cast(type)
-        return arg
-
 
 class MapValueTyped(ValueTyped):
 
@@ -701,6 +589,10 @@ def decimal(**arg_kwds):
     return ValueTyped(ir.DecimalValue, 'not decimal', **arg_kwds)
 
 
+def floating(**arg_kwds):
+    return ValueTyped(ir.FloatingValue, 'not floating', **arg_kwds)
+
+
 def timestamp(**arg_kwds):
     return ValueTyped(ir.TimestampValue, 'not timestamp', **arg_kwds)
 
@@ -713,9 +605,29 @@ def time(**arg_kwds):
     return ValueTyped(ir.TimeValue, 'not time', **arg_kwds)
 
 
-def timedelta(**arg_kwds):
-    from ibis.expr.temporal import Timedelta
-    return AnyTyped(Timedelta, 'not a timedelta', **arg_kwds)
+class Interval(ValueTyped):
+
+    def __init__(self, units=None, **arg_kwds):
+        super(Interval, self).__init__(
+            ir.IntervalValue, 'not an interval', **arg_kwds
+        )
+        if units is None:
+            self.allowed_units = None
+        else:
+            self.allowed_units = frozenset(units)
+
+    def _validate(self, args, i):
+        arg = super(Interval, self)._validate(args, i)
+
+        unit = arg.type().unit
+        if self.allowed_units is not None and unit not in self.allowed_units:
+            msg = 'Interval unit `{}` is not among the allowed ones {}'
+            raise IbisTypeError(msg.format(unit, self.allowed_units))
+
+        return arg
+
+
+interval = Interval
 
 
 def string(**arg_kwds):
@@ -742,8 +654,7 @@ def array(value_type, **arg_kwds):
 
 
 def boolean(**arg_kwds):
-    """Require that an expression has type boolean.
-    """
+    """Require that an expression has type boolean."""
     return ValueTyped(dt.boolean, 'not string', **arg_kwds)
 
 
@@ -842,22 +753,27 @@ enum = Enum
 
 class ListOf(Argument):
 
-    def __init__(self, value_type, min_length=0, **arg_kwds):
-        super(ListOf, self).__init__(**arg_kwds)
+    def __init__(
+        self, value_type, min_length=0, as_value_expr=None, **arg_kwds
+    ):
+        super(ListOf, self).__init__(as_value_expr=as_value_expr, **arg_kwds)
         self.value_type = _to_argument(value_type)
         self.min_length = min_length
+        self.as_value_expr = (
+            ir.as_value_expr if as_value_expr is None else as_value_expr
+        )
 
     def _validate(self, args, i):
         arg = args[i]
-        if isinstance(arg, tuple):
+        if not isinstance(arg, list):
             arg = args[i] = list(arg)
 
-        if not isinstance(arg, list):
-            raise IbisTypeError('not a list')
+        assert isinstance(arg, list), 'not a list in ListOf validation'
 
         if len(arg) < self.min_length:
-            raise IbisTypeError('list must have at least {} elements'
-                                .format(self.min_length))
+            raise IbisTypeError(
+                'list must have at least {:d} elements'.format(self.min_length)
+            )
 
         checked_args = []
         for j in range(len(arg)):
@@ -865,14 +781,13 @@ class ListOf(Argument):
                 checked_arg = self.value_type.validate(arg, j)
             except IbisTypeError as e:
                 exc = e.args[0]
-                msg = ('List element {0} had a type error: {1}'
-                       .format(j, exc))
+                msg = 'List element {} had a type error: {}'.format(j, exc)
                 raise IbisTypeError(msg)
             checked_args.append(checked_arg)
 
         args[i] = checked_args
 
-        return checked_args
+        return self.as_value_expr(checked_args)
 
 
 list_of = ListOf
@@ -888,3 +803,126 @@ class DataTypeArgument(Argument):
 
 
 data_type = DataTypeArgument
+
+
+def comparable(left, right):
+    return ir.castable(left, right) or ir.castable(right, left)
+
+
+@six.add_metaclass(abc.ABCMeta)
+class TableColumnValidator(object):
+    @abc.abstractmethod
+    def validate(self):
+        pass
+
+
+class SubsetValidator(TableColumnValidator):
+    def __init__(self, *rules):
+        self.rules = self._validate_rules(rules)
+
+    def _validate_rules(self, rules):
+        for column_rule in rules:
+            # Members of a schema are arguments with a name
+            if not isinstance(column_rule, Argument):
+                raise ValueError(
+                    'Arguments of subset schema must be instances of the '
+                    'Argument class (rules).')
+            if column_rule.name is None:
+                raise ValueError(
+                    'Column rules must be named inside a table.')
+        return rules
+
+    def validate(self, arg):
+        if isinstance(arg, ir.TableExpr):
+            # Check that columns match the schema first
+            for column_rule in self.rules:
+                if column_rule.name not in arg:
+                    if column_rule.optional:
+                        continue
+                    else:
+                        raise IbisTypeError(
+                            'No column with name {}.'.format(column_rule.name))
+
+                column = arg[column_rule.name]
+                try:
+                    # Arguments must validate the column
+                    column_rule.validate([column], 0)
+                except IbisTypeError as e:
+                    six.raise_from(
+                        IbisTypeError('Could not satisfy rule: {}.'.format(
+                            str(column_rule))), e)
+
+
+class Table(Argument):
+    """A table argument.
+
+    Parameters
+    ----------
+    name : str
+        The name of the table argument.
+    optional : bool
+        Whether this table argument is optional or not.
+    schema : TableColumnValidator
+        A validator for the table's columns. Only column subset validators are
+        currently supported. One can be created through the class method
+        ``Table.with_column_subset``. See the example for usage.
+    doc : str
+        A docstring to document this argument.
+    validator : Argument
+        Allows adding custom validation logic to this argument.
+
+    Examples
+    --------
+    The following op will accept an argument named ``'table'``. Note that the
+    ``schema`` argument specifies rules for columns that are required to be in
+    the table: ``time``, ``group`` and ``value1``. These must match the types
+    specified in the column rules. Column ``value2`` is optional, but if
+    present it must be of the specified type. The table may have extra columns
+    not specified in the schema.
+
+    >>> import ibis.expr.datatypes as dt
+    >>> import ibis.expr.rules as rules
+    >>> import ibis.expr.operations as ops
+    >>> class MyOp(ops.ValueOp):
+    ...    input_type = [
+    ...        rules.table(
+    ...            name='table',
+    ...            schema=rules.table.with_column_subset(
+    ...                rules.column(name='time', value_type=rules.number),
+    ...                rules.column(name='group', value_type=rules.number),
+    ...                rules.column(name='value1', value_type=rules.number),
+    ...                rules.column(name='value2', value_type=rules.number,
+    ...                             optional=True)))]
+    ...    output_type = rules.type_of_arg(0)
+    """
+    def __init__(self, name=None, optional=False, schema=None, doc=None,
+                 validator=None, **arg_kwds):
+        self.name = name
+        self.optional = optional
+
+        if not ((schema is None) or isinstance(schema, TableColumnValidator)):
+            raise ValueError(
+                'schema argument must be an instance of TableColumnValidator')
+
+        self.schema = schema
+
+        self.doc = doc
+        self.validator = validator
+
+    @classmethod
+    def with_column_subset(cls, *col_rules):
+        return SubsetValidator(*col_rules)
+
+    def _validate(self, args, i):
+        arg = args[i]
+
+        if not isinstance(arg, ir.TableExpr):
+            raise IbisTypeError('Argument must be a table.')
+
+        if self.schema is not None:
+            self.schema.validate(arg)
+
+        return arg
+
+
+table = Table
