@@ -16,7 +16,6 @@ import pytest
 
 import ibis
 
-from ibis.compat import unittest
 import ibis.expr.analysis as L
 import ibis.expr.operations as ops
 import ibis.common as com
@@ -143,7 +142,7 @@ def test_filter_on_projected_field(con):
     assert result.op().table is tpch
 
 
-def test_bad_join_predicate_raises():
+def test_join_predicate_from_derived_raises():
     # Join predicate references a derived table, but we can salvage and
     # rewrite it to get the join semantics out
     # see ibis #74
@@ -164,8 +163,26 @@ def test_bad_join_predicate_raises():
     with pytest.raises(com.ExpressionError):
         table.inner_join(table2, [table3['g'] == table2['key']])
 
-    # expected = table.inner_join(table2, [table['g'] == table2['key']])
-    # assert_equal(result, expected)
+
+def test_bad_join_predicate_raises():
+    table = ibis.table([
+        ('c', 'int32'),
+        ('f', 'double'),
+        ('g', 'string')
+    ], 'foo_table')
+
+    table2 = ibis.table([
+        ('key', 'string'),
+        ('value', 'double')
+    ], 'bar_table')
+
+    table3 = ibis.table([
+        ('key', 'string'),
+        ('value', 'double')
+    ], 'baz_table')
+
+    with pytest.raises(com.ExpressionError):
+        table.inner_join(table2, [table['g'] == table3['key']])
 
 
 def test_filter_self_join():
@@ -301,3 +318,15 @@ def test_join_table_choice():
     t = x.aggregate(cnt=x.n.count())
     predicate = t.cnt > 0
     assert L.sub_for(predicate, [(t, t.op().table)]).equals(predicate)
+
+
+def test_is_ancestor_analytic():
+    x = ibis.table(ibis.schema([('col', 'int32')]), 'x')
+    with_filter_col = x[x.columns + [ibis.null().name('filter')]]
+    filtered = with_filter_col[with_filter_col['filter'].isnull()]
+    subquery = filtered[filtered.columns]
+
+    with_analytic = subquery[subquery.columns +
+                             [subquery.count().name('analytic')]]
+
+    assert not subquery.op().is_ancestor(with_analytic)
