@@ -14,6 +14,7 @@
 
 import pytest
 
+from ibis.compat import pickle
 from ibis.expr.types import ColumnExpr, TableExpr, RelationError
 from ibis.common import ExpressionError
 import ibis.expr.api as api
@@ -36,7 +37,7 @@ def test_empty_schema():
 def test_columns(con):
     t = con.table('alltypes')
     result = t.columns
-    expected = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i']
+    expected = ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k']
     assert result == expected
 
 
@@ -274,9 +275,9 @@ def test_mutate_alter_existing_columns(table):
     expr = table.mutate(f=new_f, foo=foo)
 
     expected = table['a', 'b', 'c', 'd', 'e',
-                          new_f.name('f'), 'g', 'h',
-                          'i',
-                          foo.name('foo')]
+                     new_f.name('f'), 'g', 'h',
+                     'i', 'j', 'k',
+                     foo.name('foo')]
 
     assert_equal(expr, expected)
 
@@ -312,8 +313,9 @@ def test_add_predicate(table):
 def test_invalid_predicate(table, schema):
     # a lookalike
     table2 = api.table(schema, name='bar')
+    predicate = table2.a > 5
     with pytest.raises(RelationError):
-        table[table2.a > 5]
+        table.filter(predicate)
 
 
 def test_add_predicate_coalesce(table):
@@ -1239,6 +1241,17 @@ def test_add_column2(table):
     assert_equal(result, expected)
 
 
+def test_add_column_proxies_to_mutate(table):
+    result = table.add_column(ibis.now().cast('date'), name='date')
+    expected = table.mutate(date=ibis.now().cast('date'))
+    assert_equal(result, expected)
+
+
+def test_add_column_depricated(table):
+    with pytest.deprecated_call():
+        table.add_column(ibis.now().cast('date'), name='date')
+
+
 def test_groupby_mutate(table):
     t = table
 
@@ -1270,3 +1283,17 @@ def test_set_column(table):
     result = table.set_column('f', g)
     expected = table.set_column('f', table.f * 2)
     assert_equal(result, expected)
+
+
+def test_pickle_table_expr():
+    schema = [('time', 'timestamp'), ('key', 'string'), ('value', 'double')]
+    t0 = ibis.table(schema, name='t0')
+    raw = pickle.dumps(t0)
+    t1 = pickle.loads(raw)
+    assert t1.equals(t0)
+
+
+def test_group_by_key_function():
+    t = ibis.table([('a', 'timestamp'), ('b', 'string'), ('c', 'double')])
+    expr = t.groupby(new_key=lambda t: t.b.length()).aggregate(foo=t.c.mean())
+    assert expr.columns == ['new_key', 'foo']
