@@ -12,13 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import pytest
+import toolz
 import operator
+import numpy as np
+
 from collections import OrderedDict
 from operator import methodcaller
 from datetime import date, datetime, time
 
-import pytest
-
 from ibis.common import IbisTypeError
 from ibis.expr import (
     rules, api, datatypes as dt, types as ir, operations as ops
@@ -29,8 +31,6 @@ from ibis.compat import PY2
 from ibis import literal
 from ibis.tests.util import assert_equal
 
-import toolz
-
 
 def test_null():
     expr = ibis.literal(None)
@@ -150,13 +150,23 @@ def test_simple_map_operations():
     expr = ibis.literal(value)
     expr2 = ibis.literal(value2)
     assert isinstance(expr, ir.MapValue)
-    assert isinstance(expr['b'].op(), ops.MapValueForKey)
     assert isinstance(expr.length().op(), ops.MapLength)
-    assert isinstance(expr.keys().op(), ops.MapKeys)
-    assert isinstance(expr.values().op(), ops.MapValues)
     assert isinstance((expr + expr2).op(), ops.MapConcat)
     assert isinstance((expr2 + expr).op(), ops.MapConcat)
 
+    default = ibis.literal([0.0])
+    assert isinstance(expr.get('d', default).op(), ops.MapValueOrDefaultForKey)
+
+    # test for an invalid default type, nulls are ok
+    with pytest.raises(ValueError):
+        expr.get('d', ibis.literal('foo'))
+    assert isinstance(expr.get('d', ibis.literal(None)).op(),
+                      ops.MapValueOrDefaultForKey)
+
+    assert isinstance(expr['b'].op(), ops.MapValueForKey)
+    assert isinstance(expr.keys().op(), ops.MapKeys)
+    assert isinstance(expr.values().op(), ops.MapValues)
+
 
 @pytest.mark.parametrize(
     ['value', 'expected_type'],
@@ -288,12 +298,18 @@ def test_distinct_array_interactions(functional_alltypes):
         a + b
 
 
-def test_distinct_count(functional_alltypes):
-    result = functional_alltypes.string_col.distinct().count()
-    expected = functional_alltypes.string_col.nunique().name('count')
-    assert_equal(result, expected)
+@pytest.mark.parametrize('where', [lambda t: None, lambda t: t.int_col != 0])
+def test_distinct_count(functional_alltypes, where):
+    result = functional_alltypes.string_col.distinct().count(
+        where=where(functional_alltypes)
+    )
     assert isinstance(result.op(), ops.CountDistinct)
 
+    expected = functional_alltypes.string_col.nunique(
+        where=where(functional_alltypes)
+    ).name('count')
+    assert result.equals(expected)
+
 
 def test_distinct_unnamed_array_expr():
     table = ibis.table([('year', 'int32'),
@@ -346,6 +362,28 @@ def test_notnull(table):
     assert isinstance(expr.op(), ops.NotNull)
 
 
+@pytest.mark.parametrize('column', ['e', 'f'], ids=['float', 'double'])
+def test_isnan_isinf_column(table, column):
+    expr = table[column].isnan()
+    assert isinstance(expr, api.BooleanColumn)
+    assert isinstance(expr.op(), ops.IsNan)
+
+    expr = table[column].isinf()
+    assert isinstance(expr, api.BooleanColumn)
+    assert isinstance(expr.op(), ops.IsInf)
+
+
+@pytest.mark.parametrize('value', [1.3, np.nan, np.inf, -np.inf])
+def test_isnan_isinf_scalar(value):
+    expr = ibis.literal(value).isnan()
+    assert isinstance(expr, api.BooleanScalar)
+    assert isinstance(expr.op(), ops.IsNan)
+
+    expr = ibis.literal(value).isinf()
+    assert isinstance(expr, api.BooleanScalar)
+    assert isinstance(expr.op(), ops.IsInf)
+
+
 @pytest.mark.xfail(raises=AssertionError, reason='NYT')
 def test_null_literal():
     assert False
@@ -487,6 +525,24 @@ def test_isnull_notnull():
     assert False
 
 
+@pytest.mark.parametrize('column', [
+    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'
+])
+@pytest.mark.parametrize('how', [
+    None, 'first', 'last', 'heavy'
+])
+@pytest.mark.parametrize('condition_fn', [
+    lambda t: None,
+    lambda t: t.a > 8
+])
+def test_arbitrary(table, column, how, condition_fn):
+    col = table[column]
+    where = condition_fn(table)
+    expr = col.arbitrary(how=how, where=where)
+    assert isinstance(expr, col.type().scalar_type())
+    assert ops.is_reduction(expr)
+
+
 @pytest.mark.parametrize(
     ['column', 'operation'],
     [
@@ -723,31 +779,32 @@ def test_binop_string_type_error(table, operation):
         (operator.truediv, 'b', -5, 'double'),
         (operator.truediv, 'c', 5, 'double'),
 
-        (operator.pow, 'a', 0, 'int8'),
-        (operator.pow, 'b', 0, 'int16'),
-        (operator.pow, 'c', 0, 'int32'),
-        (operator.pow, 'd', 0, 'int64'),
+        (operator.pow, 'a', 0, 'double'),
+        (operator.pow, 'b', 0, 'double'),
+        (operator.pow, 'c', 0, 'double'),
+        (operator.pow, 'd', 0, 'double'),
         (operator.pow, 'e', 0, 'float'),
         (operator.pow, 'f', 0, 'double'),
 
-        (operator.pow, 'a', 2, 'int16'),
-        (operator.pow, 'b', 2, 'int32'),
-        (operator.pow, 'c', 2, 'int64'),
-        (operator.pow, 'd', 2, 'int64'),
+        (operator.pow, 'a', 2, 'double'),
+        (operator.pow, 'b', 2, 'double'),
+        (operator.pow, 'c', 2, 'double'),
+        (operator.pow, 'd', 2, 'double'),
 
         (operator.pow, 'a', 1.5, 'double'),
         (operator.pow, 'b', 1.5, 'double'),
         (operator.pow, 'c', 1.5, 'double'),
         (operator.pow, 'd', 1.5, 'double'),
 
+        (operator.pow, 'e', 2, 'float'),
+        (operator.pow, 'f', 2, 'double'),
+
         (operator.pow, 'a', -2, 'double'),
         (operator.pow, 'b', -2, 'double'),
         (operator.pow, 'c', -2, 'double'),
         (operator.pow, 'd', -2, 'double'),
-
-        (operator.pow, 'e', 2, 'float'),
-        (operator.pow, 'f', 2, 'double'),
-    ]
+    ],
+    ids=lambda arg: str(getattr(arg, '__name__', arg))
 )
 def test_literal_promotions(table, op, name, case, ex_type):
     col = table[name]
@@ -1030,11 +1087,11 @@ def test_time_invalid_compare_on_py2():
 
 
 def test_scalar_parameter_repr():
-    value = ibis.param(dt.timestamp, name='value')
+    value = ibis.param(dt.timestamp).name('value')
     assert repr(value) == 'value = ScalarParameter[timestamp]'
 
     value_op = value.op()
-    assert repr(value_op) == "ScalarParameter(name='value', type=timestamp)"
+    assert repr(value_op) == "ScalarParameter(type=timestamp)"
 
 
 @pytest.mark.parametrize(
@@ -1042,38 +1099,38 @@ def test_scalar_parameter_repr():
     [
         (
             # same value type, same name
-            ibis.param(dt.timestamp, name='value1'),
-            ibis.param(dt.timestamp, name='value1'),
-            True,
+            ibis.param(dt.timestamp),
+            ibis.param(dt.timestamp),
+            False,
         ),
         (
             # different value type, same name
-            ibis.param(dt.date, name='value1'),
-            ibis.param(dt.timestamp, name='value1'),
+            ibis.param(dt.date),
+            ibis.param(dt.timestamp),
             False,
         ),
         (
             # same value type, different name
-            ibis.param(dt.timestamp, name='value1'),
-            ibis.param(dt.timestamp, name='value2'),
+            ibis.param(dt.timestamp),
+            ibis.param(dt.timestamp),
             False,
         ),
         (
             # different value type, different name
-            ibis.param(dt.date, name='value1'),
-            ibis.param(dt.timestamp, name='value2'),
+            ibis.param(dt.date),
+            ibis.param(dt.timestamp),
             False,
         ),
         (
             # different Python class, left side is param
-            ibis.param(dt.timestamp, 'value'),
+            ibis.param(dt.timestamp),
             dt.date,
             False
         ),
         (
             # different Python class, right side is param
             dt.date,
-            ibis.param(dt.timestamp, 'value'),
+            ibis.param(dt.timestamp),
             False
         ),
     ]
@@ -1150,9 +1207,9 @@ def test_empty_array_as_argument():
     node = FooNode([])
     value = node.value
     expected = literal([]).cast(dt.Array(dt.int64))
-    assert not value.type().equals(dt.Array(dt.null))
-    assert value.type().equals(dt.Array(dt.int64))
-    assert value.equals(expected)
+
+    assert value.type().equals(dt.Array(dt.null))
+    assert value.cast(dt.Array(dt.int64)).equals(expected)
 
 
 def test_struct_field_dir():
