@@ -12,29 +12,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import collections
-import datetime
-import itertools
 import os
-import sys
-import warnings
-import webbrowser
-
 import six
+import sys
 import toolz
+import itertools
+import webbrowser
+import collections
 
 from ibis.common import IbisError, RelationError
-import ibis.common as com
-import ibis.compat as compat
-import ibis.config as config
 import ibis.util as util
+
+import ibis
+import ibis.common as com
+import ibis.config as config
+import ibis.expr.schema as sch
 import ibis.expr.datatypes as dt
 
 
 class Expr(object):
 
     """
-
+    Base expression class
     """
 
     def _type_display(self):
@@ -70,6 +69,8 @@ class Expr(object):
         return ExprFormatter(self, memo=memo).get_result()
 
     def _repr_png_(self):
+        if not ibis.options.graphviz_repr:
+            return None
         try:
             import ibis.expr.visualize as viz
         except ImportError:
@@ -171,10 +172,7 @@ class Expr(object):
             return type(self)(arg, name=name)
         return factory
 
-    def _can_implicit_cast(self, arg):
-        return False
-
-    def execute(self, limit='default', async=False, params=None):
+    def execute(self, limit='default', async=False, params=None, **kwargs):
         """
         If this expression is based on physical tables in a database backend,
         execute it against that backend.
@@ -191,7 +189,7 @@ class Expr(object):
           Result of compiling expression and executing in backend
         """
         from ibis.client import execute
-        return execute(self, limit=limit, async=async, params=params)
+        return execute(self, limit=limit, async=async, params=params, **kwargs)
 
     def compile(self, limit=None, params=None):
         """
@@ -221,9 +219,6 @@ class Expr(object):
             return False
         return self._arg.equals(other._arg, cache=cache)
 
-    def _can_compare(self, other):
-        return False
-
     def _root_tables(self):
         return self.op().root_tables()
 
@@ -317,7 +312,9 @@ class Node(six.with_metaclass(OperationMeta, object)):
 
     def flat_args(self):
         for arg in self.args:
-            if isinstance(arg, (tuple, list)):
+            if not isinstance(arg, six.string_types) and isinstance(
+                arg, collections.Iterable
+            ):
                 for x in arg:
                     yield x
             else:
@@ -483,6 +480,12 @@ class ExprList(Expr):
     def names(self):
         return [x.get_name() for x in self.exprs()]
 
+    def types(self):
+        return [x.type() for x in self.exprs()]
+
+    def schema(self):
+        return sch.schema(self.names(), self.types())
+
     def rename(self, f):
         new_exprs = [x.name(f(x.get_name())) for x in self.exprs()]
         return ExpressionList(new_exprs).to_expr()
@@ -510,46 +513,11 @@ class ExprList(Expr):
 
 
 def infer_literal_type(value):
-    import ibis.expr.rules as rules
-
-    if value is None or value is null:
+    # TODO: depricate?
+    if value is null:
         return dt.null
-    elif isinstance(value, bool):
-        return dt.boolean
-    elif isinstance(value, compat.integer_types):
-        return rules.int_literal_class(value)
-    elif isinstance(value, float):
-        return dt.double
-    elif isinstance(value, six.string_types):
-        return dt.string
-    elif isinstance(value, datetime.datetime):
-        return dt.timestamp
-    elif isinstance(value, datetime.date):
-        return dt.date
-    elif isinstance(value, datetime.time):
-        return dt.time
-    elif isinstance(value, list):
-        if not value:
-            return dt.Array(dt.null)
-        return dt.Array(rules.highest_precedence_type(
-            list(map(literal, value))
-        ))
-    elif isinstance(value, collections.OrderedDict):
-        if not value:
-            raise TypeError('Empty struct type not supported')
-        return dt.Struct(
-            list(value.keys()),
-            [literal(element).type() for element in value.values()],
-        )
-    elif isinstance(value, dict):
-        if not value:
-            return dt.Map(dt.null, dt.null)
-        return dt.Map(
-            rules.highest_precedence_type(list(map(literal, value.keys()))),
-            rules.highest_precedence_type(list(map(literal, value.values()))),
-        )
 
-    raise com.InputTypeError(value)
+    return dt.infer(value)
 
 
 class Literal(ValueOp):
@@ -579,24 +547,23 @@ class Literal(ValueOp):
         return []
 
 
-_parameter_counter = itertools.count()
-
-
-def _parameter_name():
-    return 'param[{:d}]'.format(next(_parameter_counter))
-
-
 class ScalarParameter(ValueOp):
 
-    def __init__(self, type, name=None):
+    parameter_counter = itertools.count()
+
+    def __init__(self, type):
         super(ScalarParameter, self).__init__(type)
-        self.name = name
         self.output_type = type.scalar_type
+        self.counter = next(self.__class__.parameter_counter)
+
+    def resolve_name(self):
+        return 'param_{:d}'.format(self.counter)
 
     def __repr__(self):
-        return '{}(name={!r}, type={})'.format(
-            type(self).__name__, self.name, self.type
-        )
+        return '{}(type={})'.format(type(self).__name__, self.type)
+
+    def __hash__(self):
+        return hash((self.type, self.counter))
 
     @property
     def type(self):
@@ -605,18 +572,15 @@ class ScalarParameter(ValueOp):
     def equals(self, other, cache=None):
         return (
             isinstance(other, ScalarParameter) and
-            self.name == other.name and
+            self.counter == other.counter and
             self.type.equals(other.type, cache=cache)
         )
 
     def root_tables(self):
         return []
 
-    def resolve_name(self):
-        return self.name
 
-
-def param(type, name=None):
+def param(type):
     """Create a parameter of a particular type to be defined just before
     execution.
 
@@ -624,8 +588,6 @@ def param(type, name=None):
     ----------
     type : dt.DataType
         The type of the unbound parameter, e.g., double, int64, date, etc.
-    name : str, optional
-        The name of the parameter
 
     Returns
     -------
@@ -642,17 +604,15 @@ def param(type, name=None):
     >>> predicates = [t.timestamp_col >= start, t.timestamp_col <= end]
     >>> expr = t.filter(predicates).value.sum()
     """
-    if name is None:
-        name = _parameter_name()
-    expr = ScalarParameter(dt.validate_type(type), name=name).to_expr()
-    return expr.name(name)
+    expr = ScalarParameter(dt.dtype(type)).to_expr()
+    return expr
 
 
-def distinct_roots(*args):
-    all_roots = []
-    for arg in args:
-        all_roots.extend(arg._root_tables())
-    return list(toolz.unique(all_roots, key=id))
+def distinct_roots(*expressions):
+    roots = toolz.concat(
+        expression._root_tables() for expression in expressions
+    )
+    return list(toolz.unique(roots, key=id))
 
 
 # ---------------------------------------------------------------------
@@ -666,8 +626,6 @@ class ValueExpr(Expr):
     either a single value (scalar)
     """
 
-    _implicit_casts = frozenset()
-
     def __init__(self, arg, name=None):
         super(ValueExpr, self).__init__(arg)
         self._name = name
@@ -686,11 +644,6 @@ class ValueExpr(Expr):
             )
         )
 
-    def _can_cast_implicit(self, typename):
-        from ibis.expr.rules import ImplicitCast
-        rule = ImplicitCast(self.type(), self._implicit_casts)
-        return rule.can_cast(typename)
-
     def has_name(self):
         if self._name is not None:
             return True
@@ -820,20 +773,28 @@ class TableExpr(Expr):
     def __len__(self):
         raise com.ExpressionError('Use .count() instead')
 
+    def __setstate__(self, instance_dictionary):
+        self.__dict__ = instance_dictionary
+
     def __getattr__(self, key):
         try:
-            return object.__getattribute__(self, key)
-        except AttributeError:
-            if not self._is_materialized() or key not in self.schema():
-                raise
+            schema = self.schema()
+        except com.IbisError:
+            raise AttributeError(key)
 
+        if key not in schema:
+            raise AttributeError(key)
+
+        try:
             return self.get_column(key)
+        except com.IbisTypeError:
+            raise AttributeError(key)
 
     def __dir__(self):
         attrs = dir(type(self))
         if self._is_materialized():
-            attrs = list(frozenset(attrs + self.schema().names))
-        return attrs
+            attrs = frozenset(attrs + self.schema().names)
+        return sorted(attrs)
 
     def _resolve(self, exprs):
         exprs = util.promote_list(exprs)
@@ -913,25 +874,6 @@ class TableExpr(Expr):
         # The operation produces a known schema
         return self.op().has_schema()
 
-    def add_column(self, expr, name=None):
-        """
-        Add indicated column expression to table, producing a new table. Note:
-        this is a shortcut for performing a projection having the same effect.
-
-        Returns
-        -------
-        modified_table : TableExpr
-        """
-        expr = self._ensure_expr(expr)
-
-        if not isinstance(expr, ColumnExpr):
-            raise com.InputTypeError('Must pass array expression')
-
-        if name is not None:
-            expr = expr.name(name)
-
-        return self.projection([self, expr])
-
     def group_by(self, by=None, **additional_grouping_expressions):
         """
         Create an intermediate grouped table expression, pending some group
@@ -977,14 +919,9 @@ class NullValue(AnyValue):
     def type(self):
         return dt.null
 
-    def _can_cast_implicit(self, typename):
-        return True
-
 
 class NumericValue(AnyValue):
-
-    def _can_compare(self, other):
-        return isinstance(other, NumericValue)
+    pass
 
 
 class IntegerValue(NumericValue):
@@ -999,54 +936,70 @@ class BooleanValue(NumericValue):
 
 class Int8Value(IntegerValue):
 
-    _implicit_casts = set([
-        'int16', 'int32', 'int64', 'float', 'double', 'decimal'
-    ])
-
     def type(self):
         return dt.int8
 
 
 class Int16Value(IntegerValue):
 
-    _implicit_casts = set(['int32', 'int64', 'float', 'double', 'decimal'])
-
     def type(self):
         return dt.int16
 
 
 class Int32Value(IntegerValue):
 
-    _implicit_casts = set(['int64', 'float', 'double', 'decimal'])
-
     def type(self):
         return dt.int32
 
 
 class Int64Value(IntegerValue):
 
-    _implicit_casts = set(['float', 'double', 'decimal'])
-
     def type(self):
         return dt.int64
 
 
+class UInt8Value(IntegerValue):
+
+    def type(self):
+        return dt.uint8
+
+
+class UInt16Value(IntegerValue):
+
+    def type(self):
+        return dt.uint16
+
+
+class UInt32Value(IntegerValue):
+
+    def type(self):
+        return dt.uint32
+
+
+class UInt64Value(IntegerValue):
+
+    def type(self):
+        return dt.uint64
+
+
+class HalffloatValue(NumericValue):
+
+    def type(self):
+        return dt.halffloat
+
+
 class FloatingValue(NumericValue):
     pass
 
 
 class FloatValue(FloatingValue):
 
-    _implicit_casts = set(['double', 'decimal'])
-
     def type(self):
         return dt.float
 
 
 class DoubleValue(FloatingValue):
 
-    _implicit_casts = set(['decimal'])
-
     def type(self):
         return dt.double
 
@@ -1056,18 +1009,12 @@ class StringValue(AnyValue):
     def type(self):
         return dt.string
 
-    def _can_compare(self, other):
-        return isinstance(other, (StringValue, TemporalValue))
-
 
 class BinaryValue(AnyValue):
 
     def type(self):
         return dt.binary
 
-    def _can_compare(self, other):
-        return isinstance(other, BinaryValue)
-
 
 class ParameterizedValue(AnyValue):
 
@@ -1086,19 +1033,11 @@ class ParameterizedValue(AnyValue):
 
 
 class DecimalValue(ParameterizedValue, NumericValue):
-
-    _implicit_casts = set(['float', 'double'])
-
-    @classmethod
-    def _make_constructor(cls, meta):
-        def constructor(arg, name=None):
-            return cls(arg, meta, name=name)
-        return constructor
+    pass
 
 
 class TemporalValue(AnyValue):
-    def _can_compare(self, other):
-        return isinstance(other, (TemporalValue, StringValue))
+    pass
 
 
 class DateValue(TemporalValue):
@@ -1106,47 +1045,12 @@ class DateValue(TemporalValue):
     def type(self):
         return dt.date
 
-    def _can_implicit_cast(self, arg):
-        op = arg.op()
-        if isinstance(op, Literal):
-            try:
-                import pandas as pd
-                pd.Timestamp(op.value)
-                return True
-            except ValueError:
-                return False
-        return False
-
-    def _implicit_cast(self, arg):
-        # assume we've checked this is OK at this point...
-        op = arg.op()
-        return DateScalar(op)
-
 
 class TimeValue(TemporalValue):
 
     def type(self):
         return dt.time
 
-    def _can_implicit_cast(self, arg):
-        op = arg.op()
-        if isinstance(op, Literal):
-            try:
-                from ibis.compat import to_time
-                to_time(op.value)
-                return True
-            except ValueError:
-                return False
-        return False
-
-    def _can_compare(self, other):
-        return isinstance(other, (TimeValue, StringValue))
-
-    def _implicit_cast(self, arg):
-        # assume we've checked this is OK at this point...
-        op = arg.op()
-        return TimeScalar(op)
-
 
 class TimestampValue(TemporalValue):
 
@@ -1157,79 +1061,40 @@ class TimestampValue(TemporalValue):
     def type(self):
         return dt.Timestamp(timezone=self._timezone)
 
-    @classmethod
-    def _make_constructor(cls, meta):
-        def constructor(arg, name=None):
-            return cls(arg, meta, name=name)
-        return constructor
 
-    def _can_implicit_cast(self, arg):
-        op = arg.op()
-        if isinstance(op, Literal):
-            if isinstance(op.value, six.integer_types):
-                warnings.warn(
-                    'Integer values for timestamp literals are deprecated in '
-                    '0.11.0 and will be removed in 0.12.0. To pass integers '
-                    'as timestamp literals, use '
-                    'pd.Timestamp({:d}, unit=...)'.format(op.value)
-                )
-            try:
-                import pandas as pd
-                pd.Timestamp(op.value)
-                return True
-            except ValueError:
-                return False
-        return False
+class IntervalValue(ParameterizedValue):
 
-    def _implicit_cast(self, arg):
-        # assume we've checked this is OK at this point...
-        op = arg.op()
-        return TimestampScalar(op)
+    def __init__(self, meta=None):
+        self.meta = meta
+
+    @property
+    def unit(self):
+        return self.meta.unit
+
+    @unit.setter
+    def unit(self, unit):
+        self.meta.unit = unit
+
+    @property
+    def resolution(self):
+        """Unit's name"""
+        return self.meta.resolution
 
 
 class ArrayValue(ParameterizedValue):
-
-    def _can_compare(self, other):
-        return isinstance(other, ArrayValue)
-
-    def _can_cast_implicit(self, typename):
-        if not isinstance(typename, dt.Array):
-            return False
-
-        self_type = self.type()
-        return (
-            super(ArrayValue, self)._can_cast_implicit(typename) or
-            self_type.equals(dt.Array(dt.null)) or
-            self_type.equals(dt.Array(dt.any))
-        )
+    pass
 
 
 class MapValue(ParameterizedValue):
-
-    def _can_compare(self, other):
-        return isinstance(other, MapValue)
-
-    def _can_cast_implicit(self, typename):
-        if not isinstance(typename, dt.Map):
-            return False
-
-        self_type = self.type()
-        return (
-            super(MapValue, self)._can_cast_implicit(typename) or
-            self_type.equals(dt.Map(dt.null, dt.null)) or
-            self_type.equals(dt.Map(dt.any, dt.any))
-        )
+    pass
 
 
 class StructValue(ParameterizedValue):
 
-    def _can_compare(self, other):
-        return isinstance(other, StructValue)
-
     def __dir__(self):
-        parent_attributes = dir(type(self))
-        field_names = self.type().names
-        return list(frozenset(itertools.chain(parent_attributes, field_names)))
+        return sorted(frozenset(
+            itertools.chain(dir(type(self)), self.type().names)
+        ))
 
 
 class NumericColumn(ColumnExpr, NumericValue):
@@ -1287,6 +1152,46 @@ class Int64Column(NumericColumn, Int64Value):
     pass
 
 
+class UInt8Scalar(ScalarExpr, UInt8Value):
+    pass
+
+
+class UInt8Column(NumericColumn, UInt8Value):
+    pass
+
+
+class UInt16Scalar(ScalarExpr, UInt16Value):
+    pass
+
+
+class UInt16Column(NumericColumn, UInt16Value):
+    pass
+
+
+class UInt32Scalar(ScalarExpr, UInt32Value):
+    pass
+
+
+class UInt32Column(NumericColumn, UInt32Value):
+    pass
+
+
+class UInt64Scalar(ScalarExpr, UInt64Value):
+    pass
+
+
+class UInt64Column(NumericColumn, UInt64Value):
+    pass
+
+
+class HalffloatScalar(ScalarExpr, HalffloatValue):
+    pass
+
+
+class HalffloatColumn(NumericColumn, HalffloatValue):
+    pass
+
+
 class FloatScalar(ScalarExpr, FloatValue):
     pass
 
@@ -1349,6 +1254,20 @@ class TimestampColumn(ColumnExpr, TimestampValue):
         TimestampValue.__init__(self, meta=meta)
 
 
+class IntervalScalar(ScalarExpr, IntervalValue):
+
+    def __init__(self, arg, meta=None, name=None):
+        ScalarExpr.__init__(self, arg, name=name)
+        IntervalValue.__init__(self, meta=meta)
+
+
+class IntervalColumn(ColumnExpr, IntervalValue):
+
+    def __init__(self, arg, meta=None, name=None):
+        ColumnExpr.__init__(self, arg, name=name)
+        IntervalValue.__init__(self, meta=meta)
+
+
 class DecimalScalar(DecimalValue, ScalarExpr):
 
     def __init__(self, arg, meta, name=None):
@@ -1364,17 +1283,11 @@ class DecimalColumn(DecimalValue, NumericColumn):
 
 
 class CategoryValue(ParameterizedValue):
-
     """
     Represents some ordered data categorization; tracked as an int32 value
     until explicitly
     """
 
-    _implicit_casts = Int16Value._implicit_casts
-
-    def _can_compare(self, other):
-        return isinstance(other, IntegerValue)
-
 
 class CategoryScalar(CategoryValue, ScalarExpr):
 
@@ -1452,6 +1365,32 @@ def as_value_expr(val):
     return val
 
 
+def castable(source, target):
+    """Return whether source ir type is castable to target
+
+    Based on the underlying datatypes and the value in case of Literals
+    """
+    op = source.op()
+    value = op.value if isinstance(op, Literal) else None
+    return dt.castable(source.type(), target.type(), value=value)
+
+
+def cast(source, target):
+    """Currently Literal to *Scalar implicit casts are allowed"""
+
+    if not castable(source, target):
+        raise com.IbisTypeError('Source is not castable to target type!')
+
+    # currently it prevents column -> scalar implicit castings
+    # however the datatypes are matching
+    op = source.op()
+    if not isinstance(op, Literal):
+        raise com.IbisTypeError('Only able to implicitly cast literals!')
+
+    out_type = target.type().scalar_type()
+    return out_type(op)
+
+
 def literal(value, type=None):
     """Create a scalar expression from a Python value.
 
@@ -1487,21 +1426,20 @@ def literal(value, type=None):
     if hasattr(value, 'op') and isinstance(value.op(), Literal):
         return value
 
-    if type is None:
-        type = infer_literal_type(value)
-    else:
-        type = dt.validate_type(type)
+    dtype = infer_literal_type(value)
 
-    if not type.valid_literal(value):
-        raise TypeError(
-            'Value {!r} cannot be safely coerced to {}'.format(value, type)
-        )
+    if type is not None:
+        try:
+            # check that dtype is implicitly castable to explicitly given dtype
+            dtype = dtype.cast(type, value=value)
+        except com.IbisTypeError:
+            raise TypeError('Value {!r} cannot be safely coerced '
+                            'to {}'.format(value, type))
 
     if value is None or value is _NULL or value is null:
-        result = null().cast(type)
+        return null().cast(dtype)
     else:
-        result = Literal(value, type=type).to_expr()
-    return result
+        return Literal(value, type=dtype).to_expr()
 
 
 _NULL = None
@@ -1558,7 +1496,34 @@ class NullLiteral(ValueOp):
 
 
 class ListExpr(ColumnExpr, AnyValue):
-    pass
+
+    @property
+    def values(self):
+        return self.op().values
+
+    def __iter__(self):
+        return iter(self.values)
+
+    def __getitem__(self, key):
+        return self.values[key]
+
+    def __add__(self, other):
+        other_values = getattr(other, 'values', other)
+        return type(self.op())(self.values + other_values).to_expr()
+
+    def __radd__(self, other):
+        other_values = getattr(other, 'values', other)
+        return type(self.op())(other_values + self.values).to_expr()
+
+    def __bool__(self):
+        return bool(self.values)
+
+    def __len__(self):
+        return len(self.values)
+
+    def type(self):
+        from ibis.expr import rules
+        return rules.highest_precedence_type(self.values)
 
 
 class SortExpr(Expr):
@@ -1574,8 +1539,8 @@ class ValueList(ValueOp):
     """
 
     def __init__(self, args):
-        self.values = [as_value_expr(x) for x in args]
-        ValueOp.__init__(self, self.values)
+        self.values = list(map(as_value_expr, args))
+        super(ValueList, self).__init__(self.values)
 
     def root_tables(self):
         return distinct_roots(*self.values)
