@@ -30,16 +30,17 @@ import ibis.common as com
 from ibis.config import options
 from ibis.client import (Query, AsyncQuery, Database,
                          DatabaseEntity, SQLClient)
-from ibis.compat import lzip
+from ibis.compat import lzip, parse_version
 from ibis.filesystems import HDFS, WebHDFS
 from ibis.impala import udf, ddl
 from ibis.impala.compat import impyla, ImpylaError, HS2Error
-from ibis.impala.compiler import build_ast
+from ibis.impala.compiler import build_ast, ImpalaDialect
 from ibis.util import log
 from ibis.sql.compiler import DDL
-import ibis.expr.datatypes as dt
-import ibis.expr.operations as ops
 import ibis.expr.types as ir
+import ibis.expr.schema as sch
+import ibis.expr.operations as ops
+import ibis.expr.datatypes as dt
 import ibis.util as util
 
 
@@ -159,13 +160,12 @@ class ImpalaConnection(object):
         self._connections.add(con)
 
         # make sure the connection works
-        cursor = con.cursor(convert_types=True)
+        cursor = con.cursor(user=params.get('user'), convert_types=True)
         cursor.ping()
 
         wrapper = ImpalaCursor(cursor, self, con, self.database,
                                self.options.copy())
         wrapper.set_options()
-
         return wrapper
 
     def ping(self):
@@ -188,7 +188,11 @@ class ImpalaCursor(object):
         self.con.connection_pool_size += 1
 
     def __del__(self):
-        self._close_cursor()
+        try:
+            self._close_cursor()
+        except Exception:
+            pass
+
         with self.con.lock:
             self.con.connection_pool_size -= 1
 
@@ -215,7 +219,7 @@ class ImpalaCursor(object):
 
     def set_options(self):
         for k, v in self.options.items():
-            query = 'SET {0}={1}'.format(k, v)
+            query = 'SET {} = {!r}'.format(k, v)
             self._cursor.execute(query)
 
     @property
@@ -289,10 +293,15 @@ class ImpalaQuery(Query):
     def _fetch(self, cursor):
         batches = cursor.fetchall(columnar=True)
         names = [x[0] for x in cursor.description]
-        return _column_batches_to_dataframe(names, batches)
+        df = _column_batches_to_dataframe(names, batches)
 
-    def _db_type_to_dtype(self, db_type, column):
-        return _HS2_TTypeId_to_dtype[db_type]
+        # Ugly Hack for PY2 to ensure unicode values for string columns
+        if self.expr is not None:
+            # in case of metadata queries there is no expr and
+            # self.schema() would raise an exception
+            return self.schema().apply_to(df)
+
+        return df
 
 
 def _column_batches_to_dataframe(names, batches):
@@ -365,8 +374,10 @@ class ImpalaAsyncQuery(ImpalaQuery, AsyncQuery):
         self._operation_active = False
 
     def __del__(self):
-        if self._cursor is not None:
+        try:
             self._cursor.release()
+        except AttributeError:
+            pass
 
     def execute(self):
         if self._operation_active:
@@ -462,6 +473,7 @@ class ImpalaClient(SQLClient):
     database_class = ImpalaDatabase
     sync_query = ImpalaQuery
     async_query = ImpalaAsyncQuery
+    dialect = ImpalaDialect
 
     def __init__(self, con, hdfs_client=None, **params):
         import hdfs
@@ -479,8 +491,8 @@ class ImpalaClient(SQLClient):
 
         self._ensure_temp_db_exists()
 
-    def _build_ast(self, expr, params=None):
-        return build_ast(expr, params=params)
+    def _build_ast(self, expr, context):
+        return build_ast(expr, context)
 
     def _get_hdfs(self):
         if self._hdfs is None:
@@ -713,12 +725,20 @@ class ImpalaClient(SQLClient):
 
         names = [x.lower() for x in names]
 
-        return dt.Schema(names, ibis_types)
+        return sch.Schema(names, ibis_types)
 
     @property
     def client_options(self):
         return self.con.options
 
+    @property
+    def version(self):
+        with self._execute('select version()', results=True) as cur:
+            raw = self._get_list(cur)[0]
+
+        vstring = raw.split()[2]
+        return parse_version(vstring)
+
     def get_options(self):
         """
         Return current query options for the Impala session
@@ -773,7 +793,7 @@ class ImpalaClient(SQLClient):
         expr : ibis TableExpr
         database : string, default None
         """
-        ast = self._build_ast(expr)
+        ast = self._build_ast(expr, ImpalaDialect.make_context())
         select = ast.queries[0]
         statement = ddl.CreateView(name, select, database=database)
         return self._execute(statement)
@@ -840,7 +860,7 @@ class ImpalaClient(SQLClient):
                 writer, to_insert = write_temp_dataframe(self, obj)
             else:
                 to_insert = obj
-            ast = self._build_ast(to_insert)
+            ast = self._build_ast(to_insert, ImpalaDialect.make_context())
             select = ast.queries[0]
 
             statement = ddl.CTAS(table_name, select,
@@ -1180,7 +1200,7 @@ class ImpalaClient(SQLClient):
         # all lowercase fields from Impala.
         names = [x.lower() for x in names]
 
-        return dt.Schema(names, ibis_types)
+        return sch.Schema(names, ibis_types)
 
     def create_function(self, func, name=None, database=None):
         """
@@ -1694,7 +1714,7 @@ class ImpalaTable(ir.TableExpr, DatabaseEntity):
         else:
             partition_schema = None
 
-        ast = build_ast(expr)
+        ast = build_ast(expr, ImpalaDialect.make_context())
         select = ast.queries[0]
         statement = ddl.InsertSelect(self._qualified_name,
                                      select,
@@ -1791,7 +1811,7 @@ class ImpalaTable(ir.TableExpr, DatabaseEntity):
             partition_fields.append((x, name_to_type[x]))
 
         pnames, ptypes = zip(*partition_fields)
-        return dt.Schema(pnames, ptypes)
+        return sch.Schema(pnames, ptypes)
 
     def add_partition(self, spec, location=None):
         """
@@ -1946,7 +1966,7 @@ def _validate_compatible(from_schema, to_schema):
     for name in from_schema:
         lt = from_schema[name]
         rt = to_schema[name]
-        if not rt.can_implicit_cast(lt):
+        if not lt.castable(rt):
             raise com.IbisInputError('Cannot safely cast {0!r} to {1!r}'
                                      .format(lt, rt))
 
