@@ -20,7 +20,6 @@ import ibis.expr.analysis as L
 import ibis.expr.datatypes as dt
 import ibis.expr.types as ir
 import ibis.expr.operations as ops
-import ibis.expr.temporal as tempo
 
 import ibis.sql.compiler as comp
 import ibis.sql.transforms as transforms
@@ -31,20 +30,25 @@ import ibis.common as com
 import ibis.util as util
 
 
-def build_ast(expr, context=None, params=None):
-    builder = ImpalaQueryBuilder(expr, context=context, params=params)
+def build_ast(expr, context):
+    assert context is not None, 'context is None'
+    builder = ImpalaQueryBuilder(expr, context=context)
     return builder.get_result()
 
 
-def _get_query(expr, context, params=None):
-    ast = build_ast(expr, context, params=params)
+def _get_query(expr, context):
+    assert context is not None, 'context is None'
+    ast = build_ast(expr, context)
     query = ast.queries[0]
 
     return query
 
 
-def to_sql(expr, context=None, params=None):
-    query = _get_query(expr, context, params=params)
+def to_sql(expr, context=None):
+    if context is None:
+        context = ImpalaDialect.make_context()
+    assert context is not None, 'context is None'
+    query = _get_query(expr, context)
     return query.compile()
 
 
@@ -62,15 +66,11 @@ class ImpalaQueryBuilder(comp.QueryBuilder):
 
     select_builder = ImpalaSelectBuilder
 
-    @property
-    def _make_context(self):
-        return ImpalaContext
-
 
 class ImpalaContext(comp.QueryContext):
 
     def _to_sql(self, expr, ctx):
-        return to_sql(expr, context=ctx)
+        return to_sql(expr, ctx)
 
 
 class ImpalaSelect(comp.Select):
@@ -106,7 +106,7 @@ class ImpalaTableSetFormatter(comp.TableSetFormatter):
         jname = self._join_names[type(op)]
 
         # Impala requires this
-        if len(op.predicates) == 0:
+        if not op.predicates:
             jname = self._join_names[ops.CrossJoin]
 
         return jname
@@ -217,9 +217,9 @@ def _window(translator, expr):
     )
 
     if isinstance(window_op, _unsupported_reductions):
-        raise com.TranslationError('{} is not supported in '
-                                   'window functions'
-                                   .format(type(window_op)))
+        raise com.UnsupportedOperationError(
+            '{} is not supported in window functions'.format(type(window_op))
+        )
 
     if isinstance(window_op, ops.CumulativeOp):
         arg = _cumulative_to_window(translator, arg, window)
@@ -369,14 +369,13 @@ def unary(func_name):
     return fixed_arity(func_name, 1)
 
 
-def _reduction_format(translator, func_name, arg, where):
+def _reduction_format(translator, func_name, arg, args, where):
     if where is not None:
-        case = where.ifelse(arg, ibis.NA)
-        arg = translator.translate(case)
-    else:
-        arg = translator.translate(arg)
+        arg = where.ifelse(arg, ibis.NA)
 
-    return '{}({})'.format(func_name, arg)
+    return '{}({})'.format(
+        func_name, ', '.join(map(translator.translate, [arg] + list(args)))
+    )
 
 
 def _reduction(func_name):
@@ -384,21 +383,24 @@ def _reduction(func_name):
         op = expr.op()
 
         # HACK: support trailing arguments
-        arg, where = op.args[:2]
+        where = op.where
+        args = [arg for arg in op.args if arg is not where]
 
-        return _reduction_format(translator, func_name, arg, where)
+        return _reduction_format(
+            translator, func_name, args[0], args[1:], where
+        )
     return formatter
 
 
 def _variance_like(func_name):
     func_names = {
-        'sample': func_name,
+        'sample': '{}_samp'.format(func_name),
         'pop': '{}_pop'.format(func_name)
     }
 
     def formatter(translator, expr):
-        arg, where, how = expr.op().args
-        return _reduction_format(translator, func_names[how], arg, where)
+        arg, how, where = expr.op().args
+        return _reduction_format(translator, func_names[how], arg, [], where)
     return formatter
 
 
@@ -442,7 +444,6 @@ def _binary_infix_op(infix_sym):
 
         left_arg = translator.translate(left)
         right_arg = translator.translate(right)
-
         if _needs_parens(left):
             left_arg = _parenthesize(left_arg)
 
@@ -478,15 +479,7 @@ def _needs_parens(op):
     op_klass = type(op)
     # function calls don't need parens
     return (op_klass in _binary_infix_ops or
-            op_klass in [ops.Negate])
-
-
-def _need_parenthesize_args(op):
-    if isinstance(op, ir.Expr):
-        op = op.op()
-    op_klass = type(op)
-    return (op_klass in _binary_infix_ops or
-            op_klass in [ops.Negate])
+            op_klass in {ops.Negate, ops.IsNull, ops.NotNull})
 
 
 def _boolean_literal_format(expr):
@@ -494,24 +487,48 @@ def _boolean_literal_format(expr):
     return 'TRUE' if value else 'FALSE'
 
 
-def _number_literal_format(expr):
-    value = expr.op().value
-    return repr(value)
-
-
 def _string_literal_format(expr):
     value = expr.op().value
     return "'{}'".format(value.replace("'", "\\'"))
 
 
+def _number_literal_format(expr):
+    value = expr.op().value
+    formatted = repr(value)
+
+    if formatted in {'nan', 'inf', '-inf'}:
+        return "CAST({!r} AS DOUBLE)".format(formatted)
+
+    return formatted
+
+
+def _interval_literal_format(expr):
+    return 'INTERVAL {} {}'.format(expr.op().value, expr.resolution.upper())
+
+
+def _interval_from_integer(translator, expr):
+    # interval cannot be selected from impala
+    op = expr.op()
+    arg, unit = op.args
+    arg_formatted = translator.translate(arg)
+
+    return 'INTERVAL {} {}'.format(arg_formatted, expr.resolution.upper())
+
+
+def _date_literal_format(expr):
+    value = expr.op().value
+    if isinstance(value, datetime.date):
+        value = value.strftime('%Y-%m-%d')
+
+    return repr(value)
+
+
 def _timestamp_literal_format(expr):
     value = expr.op().value
     if isinstance(value, datetime.datetime):
-        if value.microsecond != 0:
-            raise ValueError(value)
         value = value.strftime('%Y-%m-%d %H:%M:%S')
 
-    return "'{}'".format(value)
+    return repr(value)
 
 
 def quote_identifier(name, quotechar='`', force=False):
@@ -595,30 +612,30 @@ def _table_array_view(translator, expr):
 # ---------------------------------------------------------------------
 # Timestamp arithmetic and other functions
 
-def _timestamp_delta(translator, expr):
+def _timestamp_op(func):
+    def _formatter(translator, expr):
+        op = expr.op()
+        left, right = op.args
+        formatted_left = translator.translate(left)
+        formatted_right = translator.translate(right)
+
+        if isinstance(left, (ir.TimestampScalar, ir.DateValue)):
+            formatted_left = 'cast({} as timestamp)'.format(formatted_left)
+
+        if isinstance(right, (ir.TimestampScalar, ir.DateValue)):
+            formatted_right = 'cast({} as timestamp)'.format(formatted_right)
+
+        return '{}({}, {})'.format(func, formatted_left, formatted_right)
+
+    return _formatter
+
+
+def _timestamp_diff(translator, expr):
     op = expr.op()
-    arg, offset = op.args
-    formatted_arg = translator.translate(arg)
-    return _timestamp_format_offset(offset, formatted_arg)
+    left, right = op.args
 
-
-_impala_delta_functions = {
-    tempo.Year: 'years_add',
-    tempo.Month: 'months_add',
-    tempo.Week: 'weeks_add',
-    tempo.Day: 'days_add',
-    tempo.Hour: 'hours_add',
-    tempo.Minute: 'minutes_add',
-    tempo.Second: 'seconds_add',
-    tempo.Millisecond: 'milliseconds_add',
-    tempo.Microsecond: 'microseconds_add',
-    tempo.Nanosecond: 'nanoseconds_add'
-}
-
-
-def _timestamp_format_offset(offset, arg):
-    f = _impala_delta_functions[type(offset)]
-    return '{}({}, {})'.format(f, arg, offset.n)
+    return 'unix_timestamp({}) - unix_timestamp({})'.format(
+        translator.translate(left), translator.translate(right))
 
 
 # ---------------------------------------------------------------------
@@ -679,22 +696,40 @@ def _extract_field(sql_attr):
     return extract_field_formatter
 
 
+def _day_of_week_index(t, expr):
+    arg, = expr.op().args
+    return 'dayofweek({})'.format(t.translate(arg))
+
+
+def _day_of_week_name(t, expr):
+    arg, = expr.op().args
+    return 'dayname({})'.format(t.translate(arg))
+
+
+_impala_unit_names = {
+    'Y': 'Y',
+    'Q': 'Q',
+    'M': 'MONTH',
+    'W': 'W',
+    'D': 'J',
+    'h': 'HH',
+    'm': 'MI'
+}
+
+
 def _truncate(translator, expr):
     op = expr.op()
+    arg, unit = op.args
 
-    arg = translator.translate(op.args[0])
+    arg_formatted = translator.translate(arg)
+    try:
+        unit = _impala_unit_names[unit]
+    except KeyError:
+        raise com.UnsupportedOperationError(
+            '{!r} unit is not supported in timestamp truncate'.format(unit)
+        )
 
-    _impala_unit_names = {
-        'M': 'MONTH',
-        'D': 'J',
-        'J': 'D',
-        'H': 'HH'
-    }
-
-    unit = op.args[1]
-    unit = _impala_unit_names.get(unit, unit)
-
-    return "trunc({}, '{}')".format(arg, unit)
+    return "trunc({}, '{}')".format(arg_formatted, unit)
 
 
 def _timestamp_from_unix(translator, expr):
@@ -826,13 +861,19 @@ def _log(translator, expr):
 
     if base is None:
         return 'ln({})'.format(arg_formatted)
-    return 'log({}, {})'.format(arg_formatted, translator.translate(base))
+
+    base_formatted = translator.translate(base)
+    return 'log({}, {})'.format(base_formatted, arg_formatted)
 
 
 def _count_distinct(translator, expr):
-    op = expr.op()
-    arg_formatted = translator.translate(op.args[0])
-    return 'COUNT(DISTINCT {})'.format(arg_formatted)
+    arg, where = expr.op().args
+
+    if where is not None:
+        arg_formatted = translator.translate(where.ifelse(arg, None))
+    else:
+        arg_formatted = translator.translate(arg)
+    return 'count(DISTINCT {})'.format(arg_formatted)
 
 
 def _literal(translator, expr):
@@ -842,8 +883,12 @@ def _literal(translator, expr):
         typeclass = 'string'
     elif isinstance(expr, ir.NumericValue):
         typeclass = 'number'
+    elif isinstance(expr, ir.DateValue):
+        typeclass = 'date'
     elif isinstance(expr, ir.TimestampValue):
         typeclass = 'timestamp'
+    elif isinstance(expr, ir.IntervalValue):
+        typeclass = 'interval'
     else:
         raise NotImplementedError
 
@@ -858,7 +903,9 @@ _literal_formatters = {
     'boolean': _boolean_literal_format,
     'number': _number_literal_format,
     'string': _string_literal_format,
-    'timestamp': _timestamp_literal_format
+    'interval': _interval_literal_format,
+    'timestamp': _timestamp_literal_format,
+    'date': _date_literal_format
 }
 
 
@@ -873,7 +920,15 @@ def _identical_to(translator, expr):
     if op.args[0].equals(op.args[1]):
         return 'TRUE'
 
-    left, right = map(translator.translate, op.args)
+    left_expr = op.left
+    right_expr = op.right
+    left = translator.translate(left_expr)
+    right = translator.translate(right_expr)
+
+    if _needs_parens(left_expr):
+        left = _parenthesize(left)
+    if _needs_parens(right_expr):
+        right = _parenthesize(right)
     return '{} IS NOT DISTINCT FROM {}'.format(left, right)
 
 
@@ -913,6 +968,14 @@ _binary_infix_ops = {
 }
 
 
+def _string_like(translator, expr):
+    arg, pattern, _ = expr.op().args
+    return '{} LIKE {}'.format(
+        translator.translate(arg),
+        translator.translate(pattern),
+    )
+
+
 _operation_registry = {
     # Unary operations
     ops.NotNull: _not_null,
@@ -920,6 +983,9 @@ _operation_registry = {
     ops.Negate: _negate,
     ops.Not: _not,
 
+    ops.IsNan: unary('is_nan'),
+    ops.IsInf: unary('is_inf'),
+
     ops.IfNull: _ifnull_workaround,
     ops.NullIf: fixed_arity('nullif', 2),
 
@@ -955,9 +1021,9 @@ _operation_registry = {
     ops.Min: _reduction('min'),
 
     ops.StandardDev: _variance_like('stddev'),
-    ops.Variance: _variance_like('variance'),
+    ops.Variance: _variance_like('var'),
 
-    ops.GroupConcat: fixed_arity('group_concat', 2),
+    ops.GroupConcat: _reduction('group_concat'),
 
     ops.Count: _reduction('count'),
     ops.CountDistinct: _count_distinct,
@@ -981,13 +1047,14 @@ _operation_registry = {
     ops.LPad: fixed_arity('lpad', 3),
     ops.RPad: fixed_arity('rpad', 3),
     ops.StringJoin: _string_join,
-    ops.StringSQLLike: _binary_infix_op('LIKE'),
-    ops.RegexSearch: _binary_infix_op('RLIKE'),
+    ops.StringSQLLike: _string_like,
+    ops.RegexSearch: fixed_arity('regexp_like', 2),
     ops.RegexExtract: fixed_arity('regexp_extract', 3),
     ops.RegexReplace: fixed_arity('regexp_replace', 3),
     ops.ParseURL: _parse_url,
 
     # Timestamp operations
+    ops.Date: unary('to_date'),
     ops.TimestampNow: lambda *args: 'now()',
     ops.ExtractYear: _extract_field('year'),
     ops.ExtractMonth: _extract_field('month'),
@@ -996,7 +1063,9 @@ _operation_registry = {
     ops.ExtractMinute: _extract_field('minute'),
     ops.ExtractSecond: _extract_field('second'),
     ops.ExtractMillisecond: _extract_field('millisecond'),
-    ops.Truncate: _truncate,
+    ops.TimestampTruncate: _truncate,
+    ops.DateTruncate: _truncate,
+    ops.IntervalFromInteger: _interval_from_integer,
 
     # Other operations
     ops.E: lambda *args: 'e()',
@@ -1025,7 +1094,12 @@ _operation_registry = {
 
     ops.TableArrayView: _table_array_view,
 
-    ops.TimestampDelta: _timestamp_delta,
+    ops.DateAdd: _timestamp_op('date_add'),
+    ops.DateSub: _timestamp_op('date_sub'),
+    ops.DateDiff: _timestamp_op('datediff'),
+    ops.TimestampAdd: _timestamp_op('date_add'),
+    ops.TimestampSub: _timestamp_op('date_sub'),
+    ops.TimestampDiff: _timestamp_diff,
     ops.TimestampFromUNIX: _timestamp_from_unix,
 
     transforms.ExistsSubquery: _exists_subquery,
@@ -1044,6 +1118,9 @@ _operation_registry = {
     ops.Lead: _shift_like('lead'),
     ops.WindowOp: _window,
     ops.NTile: _ntile,
+
+    ops.DayOfWeekIndex: _day_of_week_index,
+    ops.DayOfWeekName: _day_of_week_name,
 }
 
 _operation_registry.update(_binary_infix_ops)
@@ -1052,13 +1129,20 @@ _operation_registry.update(_binary_infix_ops)
 class ImpalaExprTranslator(comp.ExprTranslator):
 
     _registry = _operation_registry
-    _context_class = ImpalaContext
+    context_class = ImpalaContext
 
     def name(self, translated, name, force=True):
         return _name_expr(translated,
                           quote_identifier(name, force=force))
 
 
+class ImpalaDialect(comp.Dialect):
+    translator = ImpalaExprTranslator
+
+
+dialect = ImpalaDialect
+
+
 compiles = ImpalaExprTranslator.compiles
 rewrites = ImpalaExprTranslator.rewrites
 
