@@ -1,17 +1,3 @@
-# Copyright 2014 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import unittest
 
 import numpy as np
@@ -386,3 +372,9 @@ LIMIT 10"""
 
         assert cur1 == cur2
         assert cur2 == cur3
+
+    def test_set_option_with_dot(self):
+        con = self.con
+        con.set_options({'request_pool': 'baz.quux'})
+        result = dict(con.raw_sql('set', True).fetchall())
+        assert result['REQUEST_POOL'] == 'baz.quux'
