@@ -33,8 +33,8 @@ pytest.importorskip('hdfs')
 pytest.importorskip('sqlalchemy')
 pytest.importorskip('impala.dbapi')
 
-from ibis.impala.compiler import ImpalaExprTranslator, to_sql  # noqa: E402
-from ibis.impala.compiler import ImpalaContext  # noqa: E402
+from ibis.impala.compiler import (
+    ImpalaExprTranslator, to_sql, ImpalaDialect)  # noqa: E402
 from ibis.sql.tests.test_compiler import ExprTestCases  # noqa: E402
 from ibis.impala.tests.common import ImpalaE2E  # noqa: E402
 
@@ -45,13 +45,15 @@ def approx_equal(a, b, eps):
 
 class ExprSQLTest(object):
 
-    def _check_expr_cases(self, cases, context=None, named=False):
+    def _check_expr_cases(self, cases, named=False):
         for expr, expected in cases:
             repr(expr)
-            result = self._translate(expr, named=named, context=context)
+            result = self._translate(expr, named=named)
             assert result == expected
 
-    def _translate(self, expr, named=False, context=None):
+    def _translate(self, expr, context=None, named=False):
+        if context is None:
+            context = ImpalaDialect.make_context()
         translator = ImpalaExprTranslator(expr, context=context, named=named)
         return translator.get_result()
 
@@ -103,7 +105,7 @@ class TestValueExprs(unittest.TestCase, ExprSQLTest):
         self._check_literals(cases)
 
     def test_column_ref_table_aliases(self):
-        context = ImpalaContext()
+        context = ImpalaDialect.make_context()
 
         table1 = ibis.table([
             ('key1', 'string'),
@@ -127,13 +129,13 @@ class TestValueExprs(unittest.TestCase, ExprSQLTest):
     def test_column_ref_quoting(self):
         schema = [('has a space', 'double')]
         table = ibis.table(schema)
-        self._translate(table['has a space'], '`has a space`')
+        self._translate(table['has a space'], named='`has a space`')
 
     def test_identifier_quoting(self):
         schema = [('date', 'double'), ('table', 'string')]
         table = ibis.table(schema)
-        self._translate(table['date'], '`date`')
-        self._translate(table['table'], '`table`')
+        self._translate(table['date'], named='`date`')
+        self._translate(table['table'], named='`table`')
 
     def test_named_expressions(self):
         a, b, g = self.table.get_columns(['a', 'b', 'g'])
@@ -266,8 +268,7 @@ FROM alltypes"""
 
     def test_timestamp_deltas(self):
         units = ['year', 'month', 'week', 'day',
-                 'hour', 'minute', 'second',
-                 'millisecond', 'microsecond']
+                 'hour', 'minute', 'second']
 
         t = self.table.i
         f = '`i`'
@@ -276,10 +277,11 @@ FROM alltypes"""
         for unit in units:
             K = 5
             offset = getattr(ibis, unit)(K)
-            template = '{0}s_add({1}, {2})'
+            add_template = 'date_add({1}, INTERVAL {2} {0})'
+            sub_template = 'date_sub({1}, INTERVAL {2} {0})'
 
-            cases.append((t + offset, template.format(unit, f, K)))
-            cases.append((t - offset, template.format(unit, f, -K)))
+            cases.append((t + offset, add_template.format(unit.upper(), f, K)))
+            cases.append((t - offset, sub_template.format(unit.upper(), f, K)))
 
         self._check_expr_cases(cases)
 
@@ -296,6 +298,20 @@ FROM alltypes"""
         ]
         self._check_expr_cases(cases)
 
+    def test_timestamp_day_of_week(self):
+        timestamp_value = L('2015-09-01 01:00:23', type='timestamp')
+        cases = [
+            (
+                timestamp_value.day_of_week.index(),
+                "dayofweek('2015-09-01 01:00:23')"
+            ),
+            (
+                timestamp_value.day_of_week.full_name(),
+                "dayname('2015-09-01 01:00:23')"
+            ),
+        ]
+        self._check_expr_cases(cases)
+
     def test_timestamp_from_integer(self):
         col = self.table.c
 
@@ -320,7 +336,7 @@ FROM alltypes"""
 
         expr = t0.g == t1.g
 
-        ctx = ImpalaContext()
+        ctx = ImpalaDialect.make_context()
         ctx.make_alias(t0)
 
         # Grab alias from parent context
@@ -376,7 +392,7 @@ class TestUnaryBuiltins(unittest.TestCase, ExprSQLTest):
 
     def test_log_other_bases(self):
         cases = [
-            (self.table.double_col.log(5), 'log(`double_col`, 5)')
+            (self.table.double_col.log(5), 'log(5, `double_col`)')
         ]
         self._check_expr_cases(cases)
 
@@ -412,10 +428,10 @@ class TestUnaryBuiltins(unittest.TestCase, ExprSQLTest):
             (c.mean(where=cond), tmp.format('avg')),
             (c.max(where=cond), tmp.format('max')),
             (c.min(where=cond), tmp.format('min')),
-            (c.std(where=cond), tmp.format('stddev')),
+            (c.std(where=cond), tmp.format('stddev_samp')),
             (c.std(where=cond, how='pop'), tmp.format('stddev_pop')),
-            (c.var(where=cond), tmp.format('variance')),
-            (c.var(where=cond, how='pop'), tmp.format('variance_pop')),
+            (c.var(where=cond), tmp.format('var_samp')),
+            (c.var(where=cond, how='pop'), tmp.format('var_pop')),
         ]
         self._check_expr_cases(cases)
 
@@ -862,7 +878,7 @@ class TestStringBuiltins(unittest.TestCase, ExprSQLTest):
         self._check_expr_cases(cases)
 
     def test_rlike(self):
-        ex = "`string_col` RLIKE '[\d]+'"
+        ex = "regexp_like(`string_col`, '[\d]+')"
         cases = [
             (self.table.string_col.rlike('[\d]+'), ex),
             (self.table.string_col.re_search('[\d]+'), ex),
@@ -1207,9 +1223,11 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
             (i64.to_timestamp(), pd.to_datetime(5 * 10 ** 8, unit='s')),
 
             (stamp.truncate('y'), pd.Timestamp('2009-01-01')),
-            (stamp.truncate('m'), pd.Timestamp('2009-05-01')),
+            (stamp.truncate('M'), pd.Timestamp('2009-05-01')),
+            (stamp.truncate('month'), pd.Timestamp('2009-05-01')),
             (stamp.truncate('d'), pd.Timestamp('2009-05-17')),
             (stamp.truncate('h'), pd.Timestamp('2009-05-17 12:00')),
+            (stamp.truncate('m'), pd.Timestamp('2009-05-17 12:34')),
             (stamp.truncate('minute'), pd.Timestamp('2009-05-17 12:34'))
         ]
 
@@ -1234,25 +1252,23 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
         d = L('5.245')
         dc = d.cast('decimal(12, 5)')
         cases = [
-            (dc % 5, Decimal('0.245')),
+            (dc % 5, 0.245),
 
-            (dc.fillna(0), Decimal('5.245')),
+            (dc.fillna(0), 5.245),
 
             (dc.exp(), 189.6158),
             (dc.log(), 1.65728),
             (dc.log2(), 2.39094),
             (dc.log10(), 0.71975),
             (dc.sqrt(), 2.29019),
-            (dc.zeroifnull(), Decimal('5.245')),
-            (-dc, Decimal('-5.245'))
+            (dc.zeroifnull(), 5.245),
+            (-dc, -5.245)
         ]
 
         for expr, expected in cases:
             result = self.con.execute(expr)
-            if isinstance(expected, Decimal):
-                tol = Decimal('0.0001')
-            else:
-                tol = 0.0001
+            tol = 0.0001
+
             approx_equal(result, expected, tol)
 
     def test_string_functions(self):
@@ -1387,8 +1403,7 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
         ]
 
         timestamp_fields = ['year', 'month', 'day', 'hour', 'minute',
-                            'second', 'millisecond', 'microsecond',
-                            'week']
+                            'second', 'week']
         for field in timestamp_fields:
             if hasattr(ts, field):
                 exprs.append(getattr(ts, field)())
@@ -1407,11 +1422,11 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
         # #310
         table = self.alltypes
 
-        expr = (table.filter([table.timestamp_col <
-                             (ibis.timestamp('2010-01-01') + ibis.month(3)),
-                             table.timestamp_col < (ibis.now() + ibis.day(10))
-                              ])
-                .count())
+        expr = (table.filter([
+            table.timestamp_col <
+            (ibis.timestamp('2010-01-01') + ibis.month(3)),
+            table.timestamp_col < (ibis.now() + ibis.day(10))
+        ]).count())
         expr.execute()
 
     def test_aggregations(self):
@@ -1698,3 +1713,13 @@ FROM t0
 WHERE `value` = 42
 GROUP BY 1"""
     assert ibis.impala.compile(expr) == expected
+
+
+def test_nunique_where():
+    t = ibis.table([('key', 'string'), ('value', 'double')], name='t0')
+    expr = t.key.nunique(where=t.value >= 1.0)
+    expected = """\
+SELECT count(DISTINCT CASE WHEN `value` >= 1.0 THEN `key` ELSE NULL END) AS `nunique`
+FROM t0"""  # noqa: E501
+    result = ibis.impala.compile(expr)
+    assert result == expected
