@@ -22,7 +22,7 @@ import ibis.expr.types as ir
 
 from ibis.common import IbisTypeError
 from ibis.compat import Decimal
-from ibis.expr.datatypes import validate_type
+from ibis.expr import datatypes as dt
 from ibis.expr.tests.mocks import MockConnection
 
 import ibis.expr.rules as rules
@@ -89,7 +89,7 @@ class TestWrapping(unittest.TestCase):
         for t, sv, av in types:
             func = self._register_udf([t], t, 'test')
 
-            ibis_type = validate_type(t)
+            ibis_type = dt.validate_type(t)
 
             expr = func(sv)
             assert type(expr) == type(ibis_type.scalar_type()(expr.op()))  # noqa: E501, E721
@@ -111,7 +111,7 @@ class TestWrapping(unittest.TestCase):
         for t, sv, av in types:
             func = self._register_uda([t], t, 'test')
 
-            ibis_type = validate_type(t)
+            ibis_type = dt.validate_type(t)
 
             expr1 = func(sv)
             expr2 = func(sv)
@@ -136,12 +136,12 @@ class TestWrapping(unittest.TestCase):
             ('boolean', [], self.all_cols[:8] + self.all_cols[9:]),
 
             # allowing double here for now
-            ('float', self.all_cols[:4], [self.s, self.b, self.t, self.dec]),
+            ('float', self.all_cols[:6], [self.s, self.b, self.t]),
 
-            ('double', self.all_cols[:4], [self.s, self.b, self.t, self.dec]),
+            ('double', self.all_cols[:6], [self.s, self.b, self.t]),
             ('string', [], self.all_cols[:7] + self.all_cols[8:]),
             ('timestamp', [], self.all_cols[:-1]),
-            ('decimal', [], self.all_cols[:4] + self.all_cols[7:])
+            ('decimal', self.all_cols[:7], self.all_cols[7:])
         ]
 
         for t, valid_casts, invalid_casts in cases:
