@@ -2,7 +2,6 @@ from __future__ import absolute_import
 
 import datetime
 import decimal
-import functools
 import numbers
 import operator
 
@@ -17,12 +16,21 @@ import toolz
 
 from ibis import compat
 
+import ibis.common as com
 import ibis.expr.types as ir
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 
+from ibis.compat import functools
+
 from ibis.pandas.core import (
-    integer_types, simple_types, numeric_types, fixed_width_types
+    boolean_types,
+    integer_types,
+    floating_types,
+    simple_types,
+    numeric_types,
+    fixed_width_types,
+    scalar_types
 )
 from ibis.pandas.dispatch import execute, execute_node
 from ibis.pandas.execution import constants
@@ -40,6 +48,12 @@ def execute_limit_frame(op, data, limit, offset, **kwargs):
     return data.iloc[offset:offset + limit]
 
 
+@execute_node.register(ops.Cast, SeriesGroupBy, dt.DataType)
+def execute_cast_series_group_by(op, data, type, **kwargs):
+    result = execute_node(op, data.obj, type, **kwargs)
+    return result.groupby(data.grouper.groupings)
+
+
 @execute_node.register(ops.Cast, pd.Series, dt.DataType)
 def execute_cast_series_generic(op, data, type, **kwargs):
     return data.astype(constants.IBIS_TYPE_TO_PANDAS_TYPE[type])
@@ -169,17 +183,14 @@ def execute_series_natural_log(op, data, **kwargs):
 
 @execute_node.register(
     ops.Clip, pd.Series,
-    (pd.Series, float, integer_types, type(None)),
-    (pd.Series, float, integer_types, type(None))
+    (pd.Series, type(None)) + numeric_types,
+    (pd.Series, type(None)) + numeric_types
 )
 def execute_series_clip(op, data, lower, upper, **kwargs):
     return data.clip(lower=lower, upper=upper)
 
 
-@execute_node.register(
-    ops.Quantile,
-    (pd.Series, SeriesGroupBy), (float,) + six.integer_types
-)
+@execute_node.register(ops.Quantile, (pd.Series, SeriesGroupBy), numeric_types)
 def execute_series_quantile(op, data, quantile, context=None, **kwargs):
     return context.agg(
         data, 'quantile', q=quantile, interpolation=op.interpolation
@@ -194,6 +205,17 @@ def execute_series_quantile_list(op, data, quantile, context=None, **kwargs):
     return list(result)
 
 
+@execute_node.register(ops.MultiQuantile, SeriesGroupBy, list)
+def execute_series_quantile_groupby(
+        op, data, quantile, context=None, **kwargs):
+
+    def q(x, quantile, interpolation):
+        return [x.quantile(quantile, interpolation=interpolation).tolist()]
+
+    result = context.agg(data, q, quantile, op.interpolation)
+    return result
+
+
 @execute_node.register(ops.Cast, datetime.datetime, dt.String)
 def execute_cast_datetime_or_timestamp_to_string(op, data, type, **kwargs):
     """Cast timestamps to strings"""
@@ -272,6 +294,18 @@ def execute_cast_string_literal(op, data, type, **kwargs):
         return cast_function(data)
 
 
+@execute_node.register(
+    ops.Round,
+    scalar_types,
+    (six.integer_types, type(None))
+)
+def execute_round_scalars(op, data, places, **kwargs):
+    if places is None:
+        return np.around(data)
+    else:
+        return np.around(data, places)
+
+
 @execute_node.register(
     ops.Round,
     pd.Series,
@@ -358,43 +392,76 @@ def execute_reduction_series_groupby(op, data, mask, context=None, **kwargs):
     return context.agg(data, type(op).__name__.lower())
 
 
+variance_ddof = {
+    'pop': 0,
+    'sample': 1,
+}
+
+
 @execute_node.register(ops.Variance, SeriesGroupBy, type(None))
 def execute_reduction_series_groupby_var(op, data, _, context=None, **kwargs):
-    return context.agg(data, 'var')
+    return context.agg(data, 'var', ddof=variance_ddof[op.how])
 
 
 @execute_node.register(ops.StandardDev, SeriesGroupBy, type(None))
 def execute_reduction_series_groupby_std(op, data, _, context=None, **kwargs):
-    return context.agg(data, 'std')
+    return context.agg(data, 'std', ddof=variance_ddof[op.how])
 
 
-@execute_node.register(ops.CountDistinct, SeriesGroupBy)
-def execute_count_distinct_series_groupby(op, data, context=None, **kwargs):
+@execute_node.register(ops.CountDistinct, SeriesGroupBy, type(None))
+def execute_count_distinct_series_groupby(op, data, _, context=None, **kwargs):
     return context.agg(data, 'nunique')
 
 
+@execute_node.register(ops.Arbitrary, SeriesGroupBy, type(None))
+def execute_arbitrary_series_groupby(op, data, _, context=None, **kwargs):
+    if op.how not in {'first', 'last'}:
+        raise com.OperationNotDefinedError(
+            'Arbitrary {!r} is not supported'.format(op.how))
+    return context.agg(data, op.how)
+
+
+def _filtered_reduction(mask, method, data):
+    return method(data[mask[data.index]])
+
+
 @execute_node.register(ops.Reduction, SeriesGroupBy, SeriesGroupBy)
 def execute_reduction_series_gb_mask(op, data, mask, context=None, **kwargs):
     method = operator.methodcaller(type(op).__name__.lower())
     return context.agg(
         data,
-        lambda x, mask=mask.obj, method=method: method(x[mask[x.index]])
+        functools.partial(_filtered_reduction, mask.obj, method)
+    )
+
+
+@execute_node.register(ops.CountDistinct, SeriesGroupBy, SeriesGroupBy)
+def execute_count_distinct_series_groupby_mask(
+    op, data, mask, context=None, **kwargs
+):
+    return context.agg(
+        data,
+        functools.partial(_filtered_reduction, mask.obj, pd.Series.nunique)
     )
 
 
 @execute_node.register(ops.Variance, SeriesGroupBy, SeriesGroupBy)
 def execute_var_series_groupby_mask(op, data, mask, context=None, **kwargs):
-    return context.agg(data, lambda x, mask=mask.obj: x[mask[x.index]].var())
+    return context.agg(
+        data,
+        lambda x, mask=mask.obj, ddof=variance_ddof[op.how]: (
+            x[mask[x.index]].var(ddof=ddof)
+        )
+    )
 
 
 @execute_node.register(ops.StandardDev, SeriesGroupBy, SeriesGroupBy)
 def execute_std_series_groupby_mask(op, data, mask, context=None, **kwargs):
-    return context.agg(data, lambda x, mask=mask.obj: x[mask[x.index]].std())
-
-
-@execute_node.register(ops.GroupConcat, SeriesGroupBy, six.string_types)
-def execute_group_concat_series_gb(op, data, sep, context=None, **kwargs):
-    return context.agg(data, lambda x, sep=sep: sep.join(x.astype(str)))
+    return context.agg(
+        data,
+        lambda x, mask=mask.obj, ddof=variance_ddof[op.how]: (
+            x[mask[x.index]].std(ddof=ddof)
+        )
+    )
 
 
 @execute_node.register(ops.Count, DataFrameGroupBy, type(None))
@@ -411,27 +478,41 @@ def execute_reduction_series_mask(op, data, mask, context=None, **kwargs):
     return context.agg(operand, type(op).__name__.lower())
 
 
+@execute_node.register(ops.CountDistinct, pd.Series, (pd.Series, type(None)))
+def execute_count_distinct_series_mask(op, data, mask, context=None, **kwargs):
+    return context.agg(data[mask] if mask is not None else data, 'nunique')
+
+
+@execute_node.register(ops.Arbitrary, pd.Series, (pd.Series, type(None)))
+def execute_arbitrary_series_mask(op, data, mask, context=None, **kwargs):
+    if op.how == 'first':
+        index = 0
+    elif op.how == 'last':
+        index = -1
+    else:
+        raise com.OperationNotDefinedError(
+            'Arbitrary {!r} is not supported'.format(op.how))
+
+    data = data[mask] if mask is not None else data
+    return data.iloc[index]
+
+
 @execute_node.register(ops.StandardDev, pd.Series, (pd.Series, type(None)))
 def execute_standard_dev_series(op, data, mask, context=None, **kwargs):
-    return context.agg(data[mask] if mask is not None else data, 'std')
+    return context.agg(
+        data[mask] if mask is not None else data,
+        'std',
+        ddof=variance_ddof[op.how]
+    )
 
 
 @execute_node.register(ops.Variance, pd.Series, (pd.Series, type(None)))
 def execute_variance_series(op, data, mask, context=None, **kwargs):
-    return context.agg(data[mask] if mask is not None else data, 'var')
-
-
-@execute_node.register(
-    ops.GroupConcat,
-    pd.Series, six.string_types, (pd.Series, type(None))
-)
-def execute_group_concat_series_mask(op, data, sep, mask, **kwargs):
-    return sep.join(data[mask] if mask is not None else data)
-
-
-@execute_node.register(ops.GroupConcat, pd.Series, six.string_types)
-def execute_group_concat_series(op, data, sep, **kwargs):
-    return sep.join(data.astype(str))
+    return context.agg(
+        data[mask] if mask is not None else data,
+        'var',
+        ddof=variance_ddof[op.how]
+    )
 
 
 @execute_node.register((ops.Any, ops.All), pd.Series)
@@ -439,10 +520,14 @@ def execute_any_all_series(op, data, context=None, **kwargs):
     return context.agg(data, type(op).__name__.lower())
 
 
-@execute_node.register(ops.CountDistinct, pd.Series)
-def execute_count_distinct_series(op, data, **kwargs):
-    # TODO(phillipc): Does count distinct have a mask?
-    return data.nunique()
+@execute_node.register(ops.NotAny, pd.Series)
+def execute_notany_series(op, data, context=None, **kwargs):
+    return ~context.agg(data, 'any')
+
+
+@execute_node.register(ops.NotAll, pd.Series)
+def execute_notall_series(op, data, context=None, **kwargs):
+    return ~context.agg(data, 'all')
 
 
 @execute_node.register(ops.Count, pd.DataFrame, type(None))
@@ -455,9 +540,23 @@ def execute_not_bool(op, data, **kwargs):
     return not data
 
 
-@execute_node.register(ops.BinaryOp, pd.Series, (pd.Series,) + simple_types)
-@execute_node.register(ops.BinaryOp, numeric_types, numeric_types)
-@execute_node.register(ops.BinaryOp, six.string_types, six.string_types)
+@execute_node.register(
+    ops.BinaryOp, (pd.Series, numeric_types), (pd.Series, numeric_types)
+)
+@execute_node.register(ops.StringConcat, pd.Series, pd.Series)
+@execute_node.register(
+    (ops.Comparison, ops.StringConcat), six.string_types, six.string_types
+)
+@execute_node.register(
+    (ops.Comparison, ops.StringConcat, ops.Multiply),
+    pd.Series, six.string_types
+)
+@execute_node.register(
+    (ops.Comparison, ops.Multiply, ops.StringConcat),
+    six.string_types, pd.Series
+)
+@execute_node.register(ops.Multiply, integer_types, six.string_types)
+@execute_node.register(ops.Multiply, six.string_types, integer_types)
 def execute_binary_op(op, left, right, **kwargs):
     op_type = type(op)
     try:
@@ -479,16 +578,22 @@ def execute_binary_op_series_group_by(op, left, right, **kwargs):
             'Cannot perform {} operation on two series with '
             'different groupings'.format(type(op).__name__)
         )
-    result = execute_binary_op(op, left.obj, right.obj)
+    result = execute_node(op, left.obj, right.obj, **kwargs)
     return result.groupby(left_groupings)
 
 
 @execute_node.register(ops.BinaryOp, SeriesGroupBy, simple_types)
-def execute_binary_op_series_gb(op, left, right, **kwargs):
+def execute_binary_op_series_gb_simple(op, left, right, **kwargs):
     result = execute_node(op, left.obj, right, **kwargs)
     return result.groupby(left.grouper.groupings)
 
 
+@execute_node.register(ops.BinaryOp, simple_types, SeriesGroupBy)
+def execute_binary_op_simple_series_gb(op, left, right, **kwargs):
+    result = execute_node(op, left, right.obj, **kwargs)
+    return result.groupby(right.grouper.groupings)
+
+
 @execute_node.register(ops.UnaryOp, SeriesGroupBy)
 def execute_unary_op_series_gb(op, operand, **kwargs):
     result = execute_node(op, operand.obj, **kwargs)
@@ -516,137 +621,16 @@ def execute_not_series(op, data, **kwargs):
     return ~data
 
 
-@execute_node.register(ops.Strftime, pd.Timestamp, six.string_types)
-def execute_strftime_timestamp_str(op, data, format_string, **kwargs):
-    return data.strftime(format_string)
-
-
-@execute_node.register(ops.Strftime, pd.Series, six.string_types)
-def execute_strftime_series_str(op, data, format_string, **kwargs):
-    return data.dt.strftime(format_string)
-
-
-@execute_node.register(
-    (ops.ExtractTimestampField, ops.ExtractTemporalField),
-    pd.Timestamp
-)
-def execute_extract_timestamp_field_timestamp(op, data, **kwargs):
-    field_name = type(op).__name__.lower().replace('extract', '')
-    return getattr(data, field_name)
-
-
-@execute_node.register(ops.ExtractMillisecond, pd.Timestamp)
-def execute_extract_millisecond_timestamp(op, data, **kwargs):
-    return int(data.microsecond // 1000.0)
-
-
-@execute_node.register(
-    (ops.ExtractTimestampField, ops.ExtractTemporalField),
-    pd.Series
-)
-def execute_extract_timestamp_field_series(op, data, **kwargs):
-    field_name = type(op).__name__.lower().replace('extract', '')
-    return getattr(data.dt, field_name)
-
-
 @execute_node.register(ops.NullIfZero, pd.Series)
 def execute_null_if_zero_series(op, data, **kwargs):
     return data.where(data != 0, np.nan)
 
 
-@execute_node.register(ops.StringLength, pd.Series)
-def execute_string_length_series(op, data, **kwargs):
-    return data.str.len()
-
-
 @execute_node.register(
-    ops.Substring,
-    pd.Series,
-    (pd.Series,) + integer_types,
-    (pd.Series,) + integer_types
+    ops.StringSplit, pd.Series, (pd.Series,) + six.string_types
 )
-def execute_string_substring(op, data, start, length, **kwargs):
-    return data.str[start:start + length]
-
-
-@execute_node.register(ops.Strip, pd.Series)
-def execute_string_strip(op, data, **kwargs):
-    return data.str.strip()
-
-
-@execute_node.register(ops.LStrip, pd.Series)
-def execute_string_lstrip(op, data, **kwargs):
-    return data.str.lstrip()
-
-
-@execute_node.register(ops.RStrip, pd.Series)
-def execute_string_rstrip(op, data, **kwargs):
-    return data.str.rstrip()
-
-
-@execute_node.register(
-    ops.LPad,
-    pd.Series,
-    (pd.Series,) + integer_types,
-    (pd.Series,) + six.string_types
-)
-def execute_string_lpad(op, data, length, pad, **kwargs):
-    return data.str.pad(length, side='left', fillchar=pad)
-
-
-@execute_node.register(
-    ops.RPad,
-    pd.Series,
-    (pd.Series,) + integer_types,
-    (pd.Series,) + six.string_types
-)
-def execute_string_rpad(op, data, length, pad, **kwargs):
-    return data.str.pad(length, side='right', fillchar=pad)
-
-
-@execute_node.register(ops.Reverse, pd.Series)
-def execute_string_reverse(op, data, **kwargs):
-    return data.str[::-1]
-
-
-@execute_node.register(ops.Lowercase, pd.Series)
-def execute_string_lower(op, data, **kwargs):
-    return data.str.lower()
-
-
-@execute_node.register(ops.Uppercase, pd.Series)
-def execute_string_upper(op, data, **kwargs):
-    return data.str.upper()
-
-
-@execute_node.register(ops.Capitalize, pd.Series)
-def execute_string_capitalize(op, data, **kwargs):
-    return data.str.capitalize()
-
-
-@execute_node.register(ops.Repeat, pd.Series, (pd.Series,) + integer_types)
-def execute_string_repeat(op, data, times, **kwargs):
-    return data.str.repeat(times)
-
-
-@execute_node.register(
-    ops.StringFind,
-    pd.Series,
-    (pd.Series,) + six.string_types,
-    (pd.Series, type(None)) + integer_types,
-    (pd.Series, type(None)) + integer_types,
-)
-def execute_string_contains(op, data, needle, start, end, **kwargs):
-    return data.str.find(needle, start, end)
-
-
-@execute_node.register(
-    ops.StringSQLLike,
-    pd.Series,
-    (pd.Series,) + six.string_types,
-)
-def execute_string_like(op, data, pattern, **kwargs):
-    return data.str.contains(pattern, regex=True)
+def execute_string_split(op, data, delimiter, **kwargs):
+    return data.str.split(delimiter)
 
 
 @execute_node.register(
@@ -659,20 +643,6 @@ def execute_between(op, data, lower, upper, **kwargs):
     return data.between(lower, upper)
 
 
-@execute_node.register(
-    ops.BetweenTime,
-    pd.Series,
-    (pd.Series, str, datetime.time),
-    (pd.Series, str, datetime.time),
-)
-def execute_between_time(op, data, lower, upper, **kwargs):
-    indexer = pd.DatetimeIndex(data).indexer_between_time(
-        lower, upper)
-    result = np.zeros(len(data), dtype=np.bool_)
-    result[indexer] = True
-    return result
-
-
 @execute_node.register(ops.DistinctColumn, pd.Series)
 def execute_series_distinct(op, data, **kwargs):
     return pd.Series(data.unique(), name=data.name)
@@ -693,73 +663,82 @@ def execute_series_notnnull(op, data, **kwargs):
     return data.notnull()
 
 
-@execute_node.register(ops.ArrayLength, pd.Series)
-def execute_array_length(op, data, **kwargs):
-    return data.apply(len)
+@execute_node.register(ops.IsNan, (pd.Series, floating_types))
+def execute_isnan(op, data, **kwargs):
+    return np.isnan(data)
 
 
-@execute_node.register(ops.ArrayLength, list)
-def execute_array_length_scalar(op, data, **kwargs):
-    return len(data)
-
-
-@execute_node.register(
-    ops.ArraySlice,
-    pd.Series, six.integer_types, (six.integer_types, type(None))
-)
-def execute_array_slice(op, data, start, stop, **kwargs):
-    return data.apply(operator.itemgetter(slice(start, stop)))
-
-
-@execute_node.register(
-    ops.ArraySlice,
-    list, six.integer_types, (six.integer_types, type(None))
-)
-def execute_array_slice_scalar(op, data, start, stop, **kwargs):
-    return data[start:stop]
-
-
-@execute_node.register(ops.ArrayIndex, pd.Series, six.integer_types)
-def execute_array_index(op, data, index, **kwargs):
-    return data.apply(
-        lambda array, index=index: (
-            array[index] if -len(array) <= index < len(array) else None
-        )
-    )
-
-
-@execute_node.register(ops.ArrayIndex, list, six.integer_types)
-def execute_array_index_scalar(op, data, index, **kwargs):
-    try:
-        return data[index]
-    except IndexError:
-        return None
-
-
-@execute_node.register(ops.ArrayConcat, pd.Series, (pd.Series, list))
-@execute_node.register(ops.ArrayConcat, list, pd.Series)
-@execute_node.register(ops.ArrayConcat, list, list)
-def execute_array_concat(op, left, right, **kwargs):
-    return left + right
-
-
-@execute_node.register(ops.ArrayRepeat, pd.Series, pd.Series)
-@execute_node.register(ops.ArrayRepeat, six.integer_types, (pd.Series, list))
-@execute_node.register(ops.ArrayRepeat, (pd.Series, list), six.integer_types)
-def execute_array_repeat(op, left, right, **kwargs):
-    return left * right
-
-
-@execute_node.register(ops.ArrayCollect, pd.Series)
-def execute_array_collect(op, data, **kwargs):
-    return list(data)
-
-
-@execute_node.register(ops.ArrayCollect, SeriesGroupBy)
-def execute_array_collect_group_by(op, data, **kwargs):
-    return data.apply(list)
+@execute_node.register(ops.IsInf, (pd.Series, floating_types))
+def execute_isinf(op, data, **kwargs):
+    return np.isinf(data)
 
 
 @execute_node.register(ops.SelfReference, pd.DataFrame)
 def execute_node_self_reference_dataframe(op, data, **kwargs):
     return data
+
+
+@execute_node.register(ir.ValueList)
+def execute_node_value_list(op, **kwargs):
+    return [execute(arg, **kwargs) for arg in op.values]
+
+
+@execute_node.register(ops.Contains, pd.Series, list)
+def execute_node_contains_series_list(op, data, elements, **kwargs):
+    return data.isin(elements)
+
+
+@execute_node.register(ops.NotContains, pd.Series, list)
+def execute_node_not_contains_series_list(op, data, elements, **kwargs):
+    return ~data.isin(elements)
+
+
+# Series, Series, Series
+# Series, Series, scalar
+@execute_node.register(ops.Where, pd.Series, pd.Series, pd.Series)
+@execute_node.register(ops.Where, pd.Series, pd.Series, scalar_types)
+def execute_node_where_series_series_series(op, cond, true, false, **kwargs):
+    # No need to turn false into a series, pandas will broadcast it
+    return true.where(cond, other=false)
+
+
+# Series, scalar, Series
+def execute_node_where_series_scalar_scalar(op, cond, true, false, **kwargs):
+    return pd.Series(np.repeat(true, len(cond))).where(cond, other=false)
+
+
+# Series, scalar, scalar
+for scalar_type in scalar_types:
+    execute_node_where_series_scalar_scalar = execute_node.register(
+        ops.Where, pd.Series, scalar_type, scalar_type
+    )(execute_node_where_series_scalar_scalar)
+
+
+# scalar, Series, Series
+@execute_node.register(ops.Where, boolean_types, pd.Series, pd.Series)
+def execute_node_where_scalar_scalar_scalar(op, cond, true, false, **kwargs):
+    # Note that it is not necessary to check that true and false are also
+    # scalars. This allows users to do things like:
+    # ibis.where(even_or_odd_bool, [2, 4, 6], [1, 3, 5])
+    return true if cond else false
+
+
+# scalar, scalar, scalar
+for scalar_type in scalar_types:
+    execute_node_where_scalar_scalar_scalar = execute_node.register(
+        ops.Where, boolean_types, scalar_type, scalar_type
+    )(execute_node_where_scalar_scalar_scalar)
+
+
+# scalar, Series, scalar
+@execute_node.register(ops.Where, boolean_types, pd.Series, scalar_types)
+def execute_node_where_scalar_series_scalar(op, cond, true, false, **kwargs):
+    return true if cond else pd.Series(
+        np.repeat(false, len(true)), index=true.index
+    )
+
+
+# scalar, scalar, Series
+@execute_node.register(ops.Where, boolean_types, scalar_types, pd.Series)
+def execute_node_where_scalar_scalar_series(op, cond, true, false, **kwargs):
+    return pd.Series(np.repeat(true, len(false))) if cond else false
