@@ -5,6 +5,7 @@ from operator import methodcaller
 import pytest
 
 import numpy as np
+import numpy.testing as npt
 
 import pandas as pd
 import pandas.util.testing as tm
@@ -634,3 +635,158 @@ def test_scalar_parameter(t, df, raw_value):
     result = expr.execute(params={value: raw_value})
     expected = df.float64_with_zeros == raw_value
     tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    'elements',
+    [
+        [1],
+        (1,),
+        pytest.mark.xfail({1}, raises=TypeError, reason='Not yet implemented'),
+        pytest.mark.xfail(
+            frozenset({1}), raises=TypeError, reason='Not yet implemented'
+        ),
+    ]
+)
+def test_isin(t, df, elements):
+    expr = t.plain_float64.isin(elements)
+    expected = df.plain_float64.isin(elements)
+    result = expr.execute()
+    tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    'elements',
+    [
+        [1],
+        (1,),
+        pytest.mark.xfail({1}, raises=TypeError, reason='Not yet implemented'),
+        pytest.mark.xfail(
+            frozenset({1}), raises=TypeError, reason='Not yet implemented'
+        ),
+    ]
+)
+def test_notin(t, df, elements):
+    expr = t.plain_float64.notin(elements)
+    expected = ~df.plain_float64.isin(elements)
+    result = expr.execute()
+    tm.assert_series_equal(result, expected)
+
+
+def test_cast_on_group_by(t, df):
+    expr = t.groupby(t.dup_strings).aggregate(
+        casted=(t.float64_with_zeros == 0).cast('int64').sum()
+    )
+    result = expr.execute()
+    expected = df.groupby('dup_strings').float64_with_zeros.apply(
+        lambda s: (s == 0).astype('int64').sum()
+    ).reset_index().rename(columns={'float64_with_zeros': 'casted'})
+    tm.assert_frame_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    'op',
+    [
+        operator.add,
+        operator.mul,
+        operator.sub,
+        operator.truediv,
+        operator.floordiv,
+        operator.mod,
+        operator.pow,
+    ],
+    ids=operator.attrgetter('__name__'),
+)
+@pytest.mark.parametrize('args', [lambda c: (1.0, c), lambda c: (c, 1.0)])
+def test_left_binary_op(t, df, op, args):
+    expr = op(*args(t.float64_with_zeros))
+    result = expr.execute()
+    expected = op(*args(df.float64_with_zeros))
+    tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    'op',
+    [
+        operator.add,
+        operator.mul,
+        operator.sub,
+        operator.truediv,
+        operator.floordiv,
+        operator.mod,
+        operator.pow,
+    ],
+    ids=operator.attrgetter('__name__'),
+)
+@pytest.mark.parametrize('args', [lambda c: (1.0, c), lambda c: (c, 1.0)])
+def test_left_binary_op_gb(t, df, op, args):
+    expr = t.groupby('dup_strings').aggregate(
+        foo=op(*args(t.float64_with_zeros)).sum()
+    )
+    result = expr.execute()
+    expected = df.groupby('dup_strings').float64_with_zeros.apply(
+        lambda s: op(*args(s)).sum()
+    ).reset_index().rename(columns={'float64_with_zeros': 'foo'})
+    tm.assert_frame_equal(result, expected)
+
+
+def test_where_series(t, df):
+    col_expr = t['plain_int64']
+    result = ibis.where(col_expr > col_expr.mean(), col_expr, 0.0).execute()
+
+    ser = df['plain_int64']
+    expected = ser.where(ser > ser.mean(), other=0.0)
+
+    tm.assert_series_equal(result, expected)
+
+
+@pytest.mark.parametrize(
+    ('cond', 'expected_func'),
+    [
+        (True, lambda df: df['plain_int64']),
+        (False, lambda df: pd.Series(np.repeat(3.0, len(df))))
+    ]
+)
+def test_where_scalar(t, df, cond, expected_func):
+    expr = ibis.where(cond, t['plain_int64'], 3.0)
+    result = expr.execute()
+    expected = expected_func(df)
+    tm.assert_series_equal(result, expected)
+
+
+def test_where_long(batting, batting_df):
+    col_expr = batting['AB']
+    result = ibis.where(col_expr > col_expr.mean(), col_expr, 0.0).execute()
+
+    ser = batting_df['AB']
+    expected = ser.where(ser > ser.mean(), other=0.0)
+
+    tm.assert_series_equal(result, expected)
+
+
+def test_round(t, df):
+    precision = 2
+    mult = 3.33333
+    result = (t.count() * mult).round(precision).execute()
+    expected = np.around(len(df) * mult, precision)
+    npt.assert_almost_equal(result, expected, decimal=precision)
+
+
+def test_quantile_groupby(batting, batting_df):
+    def q_fun(x, quantile, interpolation):
+        return [x.quantile(quantile, interpolation=interpolation).tolist()]
+
+    frac = 0.2
+    intp = 'linear'
+    result = (batting
+              .groupby('teamID')
+              .mutate(res=lambda x: x.RBI.quantile([frac, 1 - frac], intp))
+              .res
+              .execute())
+    expected = (batting_df
+                .groupby('teamID')
+                .RBI
+                .transform(q_fun, quantile=[frac, 1 - frac],
+                           interpolation=intp)
+                .rename('res'))
+    tm.assert_series_equal(result, expected)
