@@ -1,21 +1,9 @@
-# Copyright 2014 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+"""Core expression to SQL infrastructure.
+"""
 
 from six import StringIO
 from collections import defaultdict
 
-from ibis.compat import lzip
 import ibis.common as com
 import ibis.expr.analysis as L
 import ibis.expr.analytics as analytics
@@ -31,6 +19,8 @@ import ibis
 
 class QueryAST(object):
 
+    __slots__ = 'context', 'queries'
+
     def __init__(self, context, queries):
         self.context = context
         self.queries = queries
@@ -48,7 +38,7 @@ class SelectBuilder(object):
     relevant query unit aliases to be used when actually generating SQL.
     """
 
-    def __init__(self, expr, context, params):
+    def __init__(self, expr, context):
         self.expr = expr
 
         self.query_expr, self.result_handler = _adapt_expr(self.expr)
@@ -69,7 +59,6 @@ class SelectBuilder(object):
         self.distinct = False
 
         self.op_memo = util.IbisSet()
-        self.params = params
 
     def get_result(self):
         # make idempotent
@@ -118,8 +107,7 @@ class SelectBuilder(object):
                      distinct=self.distinct,
                      result_handler=self.result_handler,
                      parent_expr=self.query_expr,
-                     context=self.context,
-                     params=self.params)
+                     context=self.context)
 
     def _populate_context(self):
         # Populate aliases for the distinct relations used to output this
@@ -322,14 +310,13 @@ class SelectBuilder(object):
             backup_metric_name='__tmp__',
             parent_table=self.table_set)
 
-        # GH #667; this may reference a filtered version of self.table_set
-        arg = L.substitute_parents(op.arg)
-
-        pred = (arg == getattr(rank_set, op.arg.get_name()))
+        # GH 1393: previously because of GH667 we were substituting parents,
+        # but that introduced a bug when comparing reductions to columns on the
+        # same relation, so we leave this alone.
+        arg = op.arg
+        pred = arg == getattr(rank_set, arg.get_name())
         self.table_set = self.table_set.semi_join(rank_set, [pred])
 
-        return None
-
     # ---------------------------------------------------------------------
     # Analysis of table set
 
@@ -604,7 +591,7 @@ class _ExtractSubqueries(object):
         # Ordered set that uses object .equals to find keys
         self.observed_exprs = util.IbisMap()
 
-        self.expr_counts = defaultdict(lambda: 0)
+        self.expr_counts = defaultdict(int)
 
     def get_result(self):
         if self.query.table_set is not None:
@@ -616,8 +603,8 @@ class _ExtractSubqueries(object):
         to_extract = []
 
         # Read them inside-out, to avoid nested dependency issues
-        for expr, key in reversed(lzip(self.observed_exprs.keys,
-                                       self.observed_exprs.values)):
+        for expr, key in zip(self.observed_exprs.keys,
+                             self.observed_exprs.values):
             v = self.expr_counts[key]
 
             if self.greedy or v > 1:
@@ -674,15 +661,15 @@ class _ExtractSubqueries(object):
     _visit_NotExistsSubquery = _visit_Exists
 
     def _visit_Aggregation(self, expr):
-        self.observe(expr)
         self.visit(expr.op().table)
+        self.observe(expr)
 
     def _visit_Distinct(self, expr):
         self.observe(expr)
 
     def _visit_Limit(self, expr):
-        self.observe(expr)
         self.visit(expr.op().table)
+        self.observe(expr)
 
     def _visit_Union(self, expr):
         op = expr.op()
@@ -690,12 +677,12 @@ class _ExtractSubqueries(object):
         self.visit(op.right)
 
     def _visit_MaterializedJoin(self, expr):
-        self.observe(expr)
         self.visit(expr.op().join)
+        self.observe(expr)
 
     def _visit_Selection(self, expr):
-        self.observe(expr)
         self.visit(expr.op().table)
+        self.observe(expr)
 
     def _visit_SQLQueryResult(self, expr):
         self.observe(expr)
@@ -757,8 +744,6 @@ class _CorrelatedRefCheck(object):
                 self._visit(arg, in_subquery=in_subquery,
                             visit_cache=visit_cache,
                             visit_table_cache=visit_table_cache)
-            else:
-                continue
 
     def _is_subquery(self, node):
         # XXX
@@ -840,10 +825,16 @@ def _adapt_expr(expr):
         else:
             base_table = ir.find_base_table(expr)
             if base_table is None:
-                # expr with no table refs
+                # exprs with no table refs
+                # TODO(phillipc): remove ScalarParameter hack
+                if isinstance(expr.op(), ir.ScalarParameter):
+                    name = expr.get_name()
+                    assert name is not None, \
+                        'scalar parameter {} has no name'.format(expr)
+                    return expr, _get_scalar(name)
                 return expr.name('tmp'), _get_scalar('tmp')
-            else:
-                raise NotImplementedError(expr._repr())
+
+            raise NotImplementedError(repr(expr))
 
     elif isinstance(expr, ir.AnalyticExpr):
         return expr.to_aggregation(), as_is
@@ -907,18 +898,9 @@ class QueryBuilder(object):
 
     select_builder = SelectBuilder
 
-    def __init__(self, expr, context=None, params=None):
+    def __init__(self, expr, context):
         self.expr = expr
-
-        if context is None:
-            context = self._make_context()
-
         self.context = context
-        self.params = params if params is not None else {}
-
-    @property
-    def _make_context(self):
-        raise NotImplementedError
 
     @property
     def _union_class(self):
@@ -940,21 +922,24 @@ class QueryBuilder(object):
         op = self.expr.op()
         return self._union_class(op.left, op.right, self.expr,
                                  distinct=op.distinct,
-                                 context=self.context,
-                                 params=self.params)
+                                 context=self.context)
 
     def _make_select(self):
-        builder = self.select_builder(self.expr, self.context, self.params)
+        builder = self.select_builder(self.expr, self.context)
         return builder.get_result()
 
 
 class QueryContext(object):
 
-    """
-    Records bits of information used during ibis AST to SQL translation
+    """Records bits of information used during ibis AST to SQL translation.
+
+    Notably, table aliases (for subquery construction) and scalar query
+    parameters are tracked here.
     """
 
-    def __init__(self, indent=2, parent=None, memo=None):
+    def __init__(
+        self, indent=2, parent=None, memo=None, dialect=None, params=None
+    ):
         self._table_refs = {}
         self.extracted_subexprs = set()
         self.subquery_memo = {}
@@ -967,6 +952,8 @@ class QueryContext(object):
 
         self._table_key_memo = {}
         self.memo = memo or format.FormatMemo()
+        self.dialect = dialect
+        self.params = params if params is not None else {}
 
     def _compile_subquery(self, expr):
         sub_ctx = self.subcontext()
@@ -1023,7 +1010,7 @@ class QueryContext(object):
         alias = 't%d' % i
         self.set_ref(expr, alias)
 
-    def need_aliases(self):
+    def need_aliases(self, expr=None):
         return self.always_alias or len(self._table_refs) > 1
 
     def has_ref(self, expr, parent_contexts=False):
@@ -1052,7 +1039,7 @@ class QueryContext(object):
         self.make_alias(expr)
 
     def subcontext(self):
-        return type(self)(indent=self.indent, parent=self)
+        return type(self)(indent=self.indent, parent=self, params=self.params)
 
     # Maybe temporary hacks for correlated / uncorrelated subqueries
 
@@ -1107,23 +1094,24 @@ class QueryContext(object):
 
 class ExprTranslator(object):
 
+    """Class that performs translation of ibis expressions into executable
+    SQL.
+    """
+
     _rewrites = {}
 
-    def __init__(
-        self,
-        expr, context=None, named=False, permit_subquery=False, params=None
-    ):
+    context_class = QueryContext
+
+    def __init__(self, expr, context, named=False, permit_subquery=False):
         self.expr = expr
         self.permit_subquery = permit_subquery
-        self.context = self._context_class() if context is None else context
+
+        assert context is not None, \
+            'context is None in {}'.format(type(self).__name__)
+        self.context = context
 
         # For now, governing whether the result will have a name
         self.named = named
-        self.params = params
-
-    @property
-    def _context_class(self):
-        raise NotImplementedError
 
     def get_result(self):
         """
@@ -1156,7 +1144,7 @@ class ExprTranslator(object):
         # The operation node type the typed expression wraps
         op = expr.op()
 
-        if type(op) in self._rewrites and type(op) not in self._registry:
+        if type(op) in self._rewrites:  # even if type(op) is in self._registry
             expr = self._rewrites[type(op)](expr)
             op = expr.op()
 
@@ -1170,11 +1158,14 @@ class ExprTranslator(object):
             formatter = self._registry[type(op)]
             return formatter(self, expr)
         else:
-            raise com.TranslationError('No translator rule for {}'
-                                       .format(type(op)))
+            raise com.OperationNotDefinedError(
+                'No translation rule for {}'.format(type(op))
+            )
 
     def _trans_param(self, expr):
-        return self.params[expr]
+        raw_value = self.context.params[expr.op()]
+        literal = ibis.literal(raw_value, type=expr.type())
+        return self.translate(literal)
 
     @classmethod
     def rewrites(cls, klass, f=None):
@@ -1290,6 +1281,24 @@ def _notall_expand(expr):
     return arg.sum() < t.count()
 
 
+class Dialect(object):
+
+    """Dialects encode the properties of a particular flavor of SQL.
+
+    For example, quoting behavior is a property that should be encoded by
+    ``Dialect``. Each backend has its own dialect.
+    """
+
+    translator = ExprTranslator
+
+    @classmethod
+    def make_context(cls, params=None):
+        if params is None:
+            params = {}
+        params = {expr.op(): value for expr, value in params.items()}
+        return cls.translator.context_class(dialect=cls(), params=params)
+
+
 class DDL(object):
     pass
 
@@ -1302,12 +1311,11 @@ class Select(DDL):
     generated it
     """
 
-    def __init__(self, table_set, select_set,
+    def __init__(self, table_set, select_set, context,
                  subqueries=None, where=None, group_by=None, having=None,
                  order_by=None, limit=None,
                  distinct=False, indent=2,
-                 result_handler=None, parent_expr=None,
-                 context=None, params=None):
+                 result_handler=None, parent_expr=None):
         self.context = context
 
         self.select_set = select_set
@@ -1329,19 +1337,16 @@ class Select(DDL):
         self.indent = indent
 
         self.result_handler = result_handler
-        self.params = params
 
-    translator = None
-
-    def _translate(self, expr, context=None, named=False,
-                   permit_subquery=False):
-        if context is None:
-            context = self.context
+    @property
+    def translator(self):
+        return self.context.dialect.translator
 
+    def _translate(self, expr, named=False, permit_subquery=False):
+        context = self.context
         translator = self.translator(expr, context=context,
                                      named=named,
-                                     permit_subquery=permit_subquery,
-                                     params=self.params)
+                                     permit_subquery=permit_subquery)
         return translator.get_result()
 
     def equals(self, other, cache=None):
@@ -1412,8 +1417,11 @@ class Select(DDL):
         # GROUP BY and HAVING
         groupby_frag = self.format_group_by()
 
-        # ORDER BY and LIMIT
-        order_frag = self.format_postamble()
+        # ORDER BY
+        order_frag = self.format_order_by()
+
+        # LIMIT
+        limit_frag = self.format_limit()
 
         # Glue together the query fragments and return
         query = '\n'.join(filter(
@@ -1425,6 +1433,7 @@ class Select(DDL):
                 where_frag,
                 groupby_frag,
                 order_frag,
+                limit_frag
             ]
         ))
         return query
@@ -1453,7 +1462,7 @@ class Select(DDL):
                 expr_str = self._translate(expr, named=True)
             elif isinstance(expr, ir.TableExpr):
                 # A * selection, possibly prefixed
-                if context.need_aliases():
+                if context.need_aliases(expr):
                     alias = context.get_ref(expr)
 
                     # materialized join will not have an alias. see #491
@@ -1545,7 +1554,7 @@ class Select(DDL):
         fmt_preds = []
         for pred in self.where:
             new_pred = self._translate(pred, permit_subquery=True)
-            if isinstance(pred.op(), ops.Or):
+            if isinstance(pred.op(), (ops.Or, ops.Xor)):
                 # parens for OR exprs because it binds looser than AND
                 new_pred = '({})'.format(new_pred)
             fmt_preds.append(new_pred)
@@ -1554,34 +1563,35 @@ class Select(DDL):
         buf.write(conj.join(fmt_preds))
         return buf.getvalue()
 
-    def format_postamble(self):
-        buf = StringIO()
-        lines = 0
-
-        if len(self.order_by) > 0:
-            buf.write('ORDER BY ')
-            formatted = []
-            for expr in self.order_by:
-                key = expr.op()
-                translated = self._translate(key.expr)
-                if not key.ascending:
-                    translated += ' DESC'
-                formatted.append(translated)
-            buf.write(', '.join(formatted))
-            lines += 1
-
-        if self.limit is not None:
-            if lines:
-                buf.write('\n')
-            n, offset = self.limit['n'], self.limit['offset']
-            buf.write('LIMIT {}'.format(n))
-            if offset is not None and offset != 0:
-                buf.write(' OFFSET {}'.format(offset))
-            lines += 1
-
-        if not lines:
+    def format_order_by(self):
+        if not self.order_by:
             return None
 
+        buf = StringIO()
+        buf.write('ORDER BY ')
+
+        formatted = []
+        for expr in self.order_by:
+            key = expr.op()
+            translated = self._translate(key.expr)
+            if not key.ascending:
+                translated += ' DESC'
+            formatted.append(translated)
+
+        buf.write(', '.join(formatted))
+        return buf.getvalue()
+
+    def format_limit(self):
+        if not self.limit:
+            return None
+
+        buf = StringIO()
+
+        n, offset = self.limit['n'], self.limit['offset']
+        buf.write('LIMIT {}'.format(n))
+        if offset is not None and offset != 0:
+            buf.write(' OFFSET {}'.format(offset))
+
         return buf.getvalue()
 
 
@@ -1608,7 +1618,7 @@ class TableSetFormatter(object):
         self.join_predicates = []
 
     def _translate(self, expr):
-        return self.parent._translate(expr, context=self.context)
+        return self.parent._translate(expr)
 
     def _walk_join_tree(self, op):
         left = op.left.op()
@@ -1695,7 +1705,7 @@ class TableSetFormatter(object):
             result = '(\n{}\n)'.format(util.indent(subquery, self.indent))
             is_subquery = True
 
-        if is_subquery or ctx.need_aliases():
+        if is_subquery or ctx.need_aliases(expr):
             result += ' {}'.format(ctx.get_ref(expr))
 
         return result
@@ -1719,9 +1729,17 @@ class TableSetFormatter(object):
             buf.write('\n')
             buf.write(util.indent('{} {}'.format(jtype, table), self.indent))
 
-            if len(preds):
+            fmt_preds = []
+            for pred in preds:
+                new_pred = self._translate(pred)
+                if isinstance(pred.op(), (ops.Or, ops.Xor)):
+                    # parens for OR exprs because it binds looser than AND
+                    new_pred = '({})'.format(new_pred)
+                fmt_preds.append(new_pred)
+
+            if len(fmt_preds):
                 buf.write('\n')
-                fmt_preds = [self._translate(pred) for pred in preds]
+
                 conj = ' AND\n{}'.format(' ' * 3)
                 fmt_preds = util.indent('ON ' + conj.join(fmt_preds),
                                         self.indent * 2)
@@ -1732,15 +1750,13 @@ class TableSetFormatter(object):
 
 class Union(DDL):
 
-    def __init__(self, left_table, right_table, expr, distinct=False,
-                 context=None, params=None):
+    def __init__(self, left_table, right_table, expr, context, distinct=False):
         self.context = context
         self.left = left_table
         self.right = right_table
         self.distinct = distinct
         self.table_set = expr
         self.filters = []
-        self.params = params
 
     def _extract_subqueries(self):
         self.subqueries = _extract_subqueries(self)
