@@ -151,6 +151,8 @@ def test_timestamp_cast_noop(alltypes, at, translate):
         (methodcaller('minute'), 48),
         (methodcaller('second'), 5),
         (methodcaller('millisecond'), 359),
+        (lambda x: x.day_of_week.index(), 3),
+        (lambda x: x.day_of_week.full_name(), 'Tuesday'),
     ]
 )
 def test_simple_datetime_operations(con, func, expected, translate):
@@ -350,7 +352,7 @@ def test_translate(con):
     [
         (L('a'), 0),
         (L('b'), 1),
-        (L('d'), None),
+        (L('d'), -1),
         (L(None).cast(dt.string), 3),
     ]
 )
@@ -510,14 +512,14 @@ def test_numeric_builtins_work(alltypes, df):
             lambda t: (t.double_col > 20).ifelse(10, -20),
             lambda df: pd.Series(
                 np.where(df.double_col > 20, 10, -20),
-                dtype='int64'
+                dtype='int8'
             ),
         ),
         (
             lambda t: (t.double_col > 20).ifelse(10, -20).abs(),
             lambda df: pd.Series(
                 np.where(df.double_col > 20, 10, -20),
-                dtype='int64'
+                dtype='int8'
             ).abs(),
         ),
     ]
@@ -572,7 +574,7 @@ def test_ifelse(alltypes, df, op, pandas_op):
 def test_bucket(alltypes, df, func, pandas_func):
     expr = func(alltypes.double_col)
     result = expr.execute()
-    expected = pandas_func(df.double_col)
+    expected = pandas_func(df.double_col).astype('category')
     tm.assert_series_equal(result, expected, check_names=False)
 
 
@@ -714,6 +716,15 @@ def test_aggregations(alltypes, df, func, pandas_func):
     np.testing.assert_allclose(result, expected)
 
 
+def test_not_contains(alltypes, df):
+    n = 100
+    table = alltypes.limit(n)
+    expr = table.string_col.notin(['1', '7'])
+    result = expr.execute()
+    expected = ~df.head(n).string_col.isin(['1', '7'])
+    tm.assert_series_equal(result, expected, check_names=False)
+
+
 def test_group_concat(alltypes, df):
     expr = alltypes.string_col.group_concat()
     result = expr.execute()
@@ -753,7 +764,7 @@ def test_interactive_repr_shows_error(alltypes):
     with config.option_context('interactive', True):
         result = repr(expr)
 
-    assert 'no translator rule' in result.lower()
+    assert 'no translation rule' in result.lower()
 
 
 def test_subquery(alltypes, df):
@@ -1180,9 +1191,9 @@ def test_rank(con):
     expr = t.double_col.rank()
     sqla_expr = expr.compile()
     result = str(sqla_expr.compile(compile_kwargs=dict(literal_binds=True)))
-    expected = """\
-SELECT rank() OVER (ORDER BY t0.double_col ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) - 1 AS tmp 
-FROM functional_alltypes AS t0"""  # noqa: E501,W291
+    expected = ("SELECT rank() OVER (ORDER BY t0.double_col ROWS BETWEEN "
+                "UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) - 1 AS tmp \n"
+                "FROM functional_alltypes AS t0")
     assert result == expected
 
 
@@ -1191,9 +1202,9 @@ def test_percent_rank(con):
     expr = t.double_col.percent_rank()
     sqla_expr = expr.compile()
     result = str(sqla_expr.compile(compile_kwargs=dict(literal_binds=True)))
-    expected = """\
-SELECT percent_rank() OVER (ORDER BY t0.double_col ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS tmp 
-FROM functional_alltypes AS t0"""  # noqa: E501,W291
+    expected = ("SELECT percent_rank() OVER (ORDER BY t0.double_col ROWS "
+                "BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS "
+                "tmp \nFROM functional_alltypes AS t0")
     assert result == expected
 
 
@@ -1202,9 +1213,9 @@ def test_ntile(con):
     expr = t.double_col.ntile(7)
     sqla_expr = expr.compile()
     result = str(sqla_expr.compile(compile_kwargs=dict(literal_binds=True)))
-    expected = """\
-SELECT ntile(7) OVER (ORDER BY t0.double_col ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) - 1 AS tmp 
-FROM functional_alltypes AS t0"""  # noqa: E501,W291
+    expected = ("SELECT ntile(7) OVER (ORDER BY t0.double_col ROWS BETWEEN "
+                "UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) - 1 AS tmp \n"
+                "FROM functional_alltypes AS t0")
     assert result == expected
 
 
@@ -1287,6 +1298,18 @@ def test_boolean_summary(alltypes):
             'approx_nunique',
         ]
     )
+
+    type_conversions = {
+        'count': 'int64',
+        'nulls': 'int64',
+        'min': 'bool',
+        'max': 'bool',
+        'sum': 'int64',
+        'approx_nunique': 'int64'
+    }
+    for k, v in type_conversions.items():
+        expected[k] = expected[k].astype(v)
+
     tm.assert_frame_equal(result, expected)
 
 
