@@ -40,14 +40,14 @@ def _cast(t, expr):
             return sa.func.datetime(sa_arg, 'unixepoch')
         elif isinstance(arg, ir.StringValue):
             return sa.func.strftime('%Y-%m-%d %H:%M:%f', sa_arg)
-        raise com.TranslationError(type(arg))
+        raise com.UnsupportedOperationError(type(arg))
 
     if isinstance(target_type, dt.Date):
         if isinstance(arg, ir.IntegerValue):
             return sa.func.date(sa.func.datetime(sa_arg, 'unixepoch'))
         elif isinstance(arg, ir.StringValue):
             return sa.func.date(sa_arg)
-        raise com.TranslationError(type(arg))
+        raise com.UnsupportedOperationError(type(arg))
 
     if isinstance(arg, ir.CategoryValue) and target_type == 'int32':
         return sa_arg
@@ -121,6 +121,29 @@ def _strftime_int(fmt):
     return translator
 
 
+_truncate_modifiers = {
+    'Y': 'start of year',
+    'M': 'start of month',
+    'D': 'start of day',
+    'W': 'weekday 1',
+}
+
+
+def _truncate(func):
+    def translator(t, expr):
+        arg, unit = expr.op().args
+        sa_arg = t.translate(arg)
+        try:
+            modifier = _truncate_modifiers[unit]
+        except KeyError:
+            raise com.UnsupportedOperationError(
+                'Unsupported truncate unit {!r}'.format(unit)
+            )
+        return func(sa_arg, modifier)
+
+    return translator
+
+
 def _now(t, expr):
     return sa.func.datetime('now')
 
@@ -152,6 +175,53 @@ def _log(t, expr):
     return sa.func._ibis_sqlite_log(sa_arg, t.translate(base))
 
 
+def _repeat(t, expr):
+    arg, times = map(t.translate, expr.op().args)
+    f = sa.func
+    return f.replace(
+        f.substr(
+            f.quote(
+                f.zeroblob((times + 1) / 2)
+            ),
+            3,
+            times
+        ),
+        '0',
+        arg
+    )
+
+
+def _generic_pad(arg, length, pad):
+    f = sa.func
+    arg_length = f.length(arg)
+    pad_length = f.length(pad)
+    number_of_zero_bytes = (
+        (length - arg_length - 1 + pad_length) / pad_length + 1) / 2
+    return f.substr(
+        f.replace(
+            f.replace(
+                f.substr(f.quote(f.zeroblob(number_of_zero_bytes)), 3),
+                "'",
+                ''
+            ),
+            '0',
+            pad
+        ),
+        1,
+        length - f.length(arg)
+    )
+
+
+def _lpad(t, expr):
+    arg, length, pad = map(t.translate, expr.op().args)
+    return _generic_pad(arg, length, pad) + arg
+
+
+def _rpad(t, expr):
+    arg, length, pad = map(t.translate, expr.op().args)
+    return arg + _generic_pad(arg, length, pad)
+
+
 _operation_registry.update({
     ops.Cast: _cast,
 
@@ -160,23 +230,12 @@ _operation_registry.update({
 
     ops.StringFind: _string_find,
 
-    ops.StringLength: unary('length'),
-
     ops.Least: varargs(sa.func.min),
     ops.Greatest: varargs(sa.func.max),
     ops.IfNull: fixed_arity(sa.func.ifnull, 2),
 
-    ops.Lowercase: unary('lower'),
-    ops.Uppercase: unary('upper'),
-
-    ops.Strip: unary('trim'),
-    ops.LStrip: unary('ltrim'),
-    ops.RStrip: unary('rtrim'),
-
-    ops.StringReplace: fixed_arity(sa.func.replace, 3),
-    ops.StringSQLLike: _infix_op('LIKE'),
-    ops.RegexSearch: _infix_op('REGEXP'),
-
+    ops.DateTruncate: _truncate(sa.func.date),
+    ops.TimestampTruncate: _truncate(sa.func.datetime),
     ops.Strftime: _strftime,
     ops.ExtractYear: _strftime_int('%Y'),
     ops.ExtractMonth: _strftime_int('%m'),
@@ -192,16 +251,25 @@ _operation_registry.update({
     ops.RegexReplace: fixed_arity(sa.func._ibis_sqlite_regex_replace, 3),
     ops.RegexExtract: fixed_arity(sa.func._ibis_sqlite_regex_extract, 3),
 
-    ops.Sqrt: fixed_arity(sa.func._ibis_sqlite_sqrt, 1),
+    ops.LPad: _lpad,
+    ops.RPad: _rpad,
+    ops.Repeat: _repeat,
+    ops.Reverse: unary(sa.func._ibis_sqlite_reverse),
+
+    ops.StringAscii: unary(sa.func._ibis_sqlite_string_ascii),
+    ops.Capitalize: unary(sa.func._ibis_sqlite_capitalize),
+    ops.Translate: fixed_arity(sa.func._ibis_sqlite_translate, 3),
+
+    ops.Sqrt: unary(sa.func._ibis_sqlite_sqrt),
     ops.Power: fixed_arity(sa.func._ibis_sqlite_power, 2),
-    ops.Exp: fixed_arity(sa.func._ibis_sqlite_exp, 1),
-    ops.Ln: fixed_arity(sa.func._ibis_sqlite_ln, 1),
+    ops.Exp: unary(sa.func._ibis_sqlite_exp),
+    ops.Ln: unary(sa.func._ibis_sqlite_ln),
     ops.Log: _log,
-    ops.Log10: fixed_arity(sa.func._ibis_sqlite_log10, 1),
-    ops.Log2: fixed_arity(sa.func._ibis_sqlite_log2, 1),
-    ops.Floor: fixed_arity(sa.func._ibis_sqlite_floor, 1),
-    ops.Ceil: fixed_arity(sa.func._ibis_sqlite_ceil, 1),
-    ops.Sign: fixed_arity(sa.func._ibis_sqlite_sign, 1),
+    ops.Log10: unary(sa.func._ibis_sqlite_log10),
+    ops.Log2: unary(sa.func._ibis_sqlite_log2),
+    ops.Floor: unary(sa.func._ibis_sqlite_floor),
+    ops.Ceil: unary(sa.func._ibis_sqlite_ceil),
+    ops.Sign: unary(sa.func._ibis_sqlite_sign),
     ops.FloorDivide: fixed_arity(sa.func._ibis_sqlite_floordiv, 2),
 
     ops.Variance: _variance_reduction('_ibis_sqlite_var'),
@@ -234,3 +302,6 @@ compiles = SQLiteExprTranslator.compiles
 class SQLiteDialect(alch.AlchemyDialect):
 
     translator = SQLiteExprTranslator
+
+
+dialect = SQLiteDialect
