@@ -9,7 +9,7 @@ import ibis
 def dbpath():
     # If we haven't defined an environment variable with the path of the SQLite
     # database, assume it's in $PWD
-    path = os.environ.get('IBIS_TEST_SQLITE_DB_PATH', 'ibis_testing.db')
+    path = os.environ.get('IBIS_TEST_SQLITE_DATABASE', 'ibis_testing.db')
     if not os.path.exists(path):
         pytest.skip('SQLite testing db {} does not exist'.format(path))
     else:
@@ -38,9 +38,11 @@ def dialect():
 
 @pytest.fixture
 def translate(dialect):
-    from ibis.sql.sqlite.compiler import SQLiteExprTranslator
+    from ibis.sql.sqlite.compiler import SQLiteDialect
+    ibis_dialect = SQLiteDialect()
+    context = ibis_dialect.make_context()
     return lambda expr: str(
-        SQLiteExprTranslator(expr).get_result().compile(
+        ibis_dialect.translator(expr, context).get_result().compile(
             dialect=dialect,
             compile_kwargs=dict(literal_binds=True)
         )
