@@ -25,7 +25,7 @@ from ibis.expr.tests.mocks import MockConnection
 pytest.importorskip('sqlalchemy')
 pytest.importorskip('impala.dbapi')
 
-from ibis.impala.compiler import build_ast, to_sql  # noqa: E402
+from ibis.impala.compiler import build_ast, to_sql, ImpalaDialect  # noqa: E402
 from ibis import impala  # noqa: E402
 
 
@@ -47,8 +47,7 @@ class TestASTBuilder(unittest.TestCase):
         joined = table2.inner_join(table3, [join_pred])
         result = joined[[table3, table2['value']]]
 
-        ast = build_ast(result)
-        stmt = ast.queries[0]
+        stmt = _get_query(result)
 
         def foo():
             table3 = table[filter_pred]
@@ -82,8 +81,7 @@ class TestASTBuilder(unittest.TestCase):
         result = joined.aggregate([met1, table3['f'].sum().name('bar')],
                                   by=[table3['g'], table2['key']])
 
-        ast = build_ast(result)
-        stmt = ast.queries[0]
+        stmt = _get_query(result)
 
         # #790, this behavior was different before
         ex_pred = [table3['g'] == table2['key']]
@@ -123,8 +121,7 @@ class TestNonTabularResults(unittest.TestCase):
 
         expr = table[table.c > 0].f.sum()
 
-        ast = build_ast(expr)
-        query = ast.queries[0]
+        query = _get_query(expr)
 
         sql_query = query.compile()
         expected = """SELECT sum(`f`) AS `sum`
@@ -191,8 +188,7 @@ FROM (
         agged = table[table.c > 0].group_by('g').aggregate([m])
         expr = agged.g
 
-        ast = build_ast(expr)
-        query = ast.queries[0]
+        query = _get_query(expr)
 
         sql_query = query.compile()
         expected = """\
@@ -266,7 +262,7 @@ FROM alltypes"""
 
 
 def _get_query(expr):
-    ast = build_ast(expr)
+    ast = build_ast(expr, ImpalaDialect.make_context())
     return ast.queries[0]
 
 
@@ -1059,9 +1055,9 @@ WHERE `a` > 0 AND
         expected = """\
 SELECT count(*) AS `count`
 FROM functional_alltypes
-WHERE `timestamp_col` < months_add('2010-01-01 00:00:00', 3) AND
-      `timestamp_col` < days_add(now(), 10)"""
-        assert result == expected
+WHERE `timestamp_col` < date_add(cast({} as timestamp), INTERVAL 3 MONTH) AND
+      `timestamp_col` < date_add(cast(now() as timestamp), INTERVAL 10 DAY)"""
+        assert result == expected.format("'2010-01-01 00:00:00'")
 
     def test_bug_duplicated_where(self):
         # GH #539
@@ -1077,16 +1073,21 @@ WHERE `timestamp_col` < months_add('2010-01-01 00:00:00', 3) AND
         worst = tmp2.limit(10)
 
         result = to_sql(worst)
+
+        # TODO(cpcloud): We should be able to flatten the second subquery into
+        # the first
         expected = """\
-SELECT *
+SELECT t0.*
 FROM (
-  SELECT `arrdelay`, `dest`,
-         avg(`arrdelay`) OVER (PARTITION BY `dest`) AS `dest_avg`,
+  SELECT *, avg(`arrdelay`) OVER (PARTITION BY `dest`) AS `dest_avg`,
          `arrdelay` - avg(`arrdelay`) OVER (PARTITION BY `dest`) AS `dev`
-  FROM airlines
+  FROM (
+    SELECT `arrdelay`, `dest`
+    FROM airlines
+  ) t2
 ) t0
-WHERE `dev` IS NOT NULL
-ORDER BY `dev` DESC
+WHERE t0.`dev` IS NOT NULL
+ORDER BY t0.`dev` DESC
 LIMIT 10"""
         assert result == expected
 
@@ -1272,9 +1273,10 @@ WHERE `value` > 0"""
 
     def test_bug_project_multiple_times(self):
         # 108
-        customer = self.con.table('tpch_customer')
-        nation = self.con.table('tpch_nation')
-        region = self.con.table('tpch_region')
+        con = self.con
+        customer = con.table('tpch_customer')
+        nation = con.table('tpch_nation')
+        region = con.table('tpch_region')
 
         joined = (
             customer.inner_join(nation,
@@ -1295,27 +1297,27 @@ WHERE `value` > 0"""
         # it works!
         result = to_sql(expr)
         expected = """\
-SELECT t0.`c_name`, t2.`r_name`, t1.`n_name`
-FROM tpch_customer t0
-  INNER JOIN tpch_nation t1
-    ON t0.`c_nationkey` = t1.`n_nationkey`
-  INNER JOIN tpch_region t2
-    ON t1.`n_regionkey` = t2.`r_regionkey`
+WITH t0 AS (
+  SELECT t2.*, t3.`n_name`, t4.`r_name`
+  FROM tpch_customer t2
+    INNER JOIN tpch_nation t3
+      ON t2.`c_nationkey` = t3.`n_nationkey`
+    INNER JOIN tpch_region t4
+      ON t3.`n_regionkey` = t4.`r_regionkey`
+)
+SELECT `c_name`, `r_name`, `n_name`
+FROM t0
   LEFT SEMI JOIN (
     SELECT *
     FROM (
-      SELECT t1.`n_name`, sum(CAST(t0.`c_acctbal` AS double)) AS `sum`
-      FROM tpch_customer t0
-        INNER JOIN tpch_nation t1
-          ON t0.`c_nationkey` = t1.`n_nationkey`
-        INNER JOIN tpch_region t2
-          ON t1.`n_regionkey` = t2.`r_regionkey`
+      SELECT `n_name`, sum(CAST(`c_acctbal` AS double)) AS `sum`
+      FROM t0
       GROUP BY 1
-    ) t4
+    ) t2
     ORDER BY `sum` DESC
     LIMIT 10
-  ) t3
-    ON t1.`n_name` = t3.`n_name`"""
+  ) t1
+    ON t0.`n_name` = t1.`n_name`"""
         assert result == expected
 
     def test_aggregate_projection_subquery(self):
@@ -1672,28 +1674,27 @@ FROM tbl t0
 
         result = to_sql(expr)
         expected = """\
-SELECT t0.*, t1.`n_name`, t2.`r_name`
-FROM customer t0
-  INNER JOIN nation t1
-    ON t0.`c_nationkey` = t1.`n_nationkey`
-  INNER JOIN region t2
-    ON t1.`n_regionkey` = t2.`r_regionkey`
+WITH t0 AS (
+  SELECT t2.*, t3.`n_name`, t4.`r_name`
+  FROM customer t2
+    INNER JOIN nation t3
+      ON t2.`c_nationkey` = t3.`n_nationkey`
+    INNER JOIN region t4
+      ON t3.`n_regionkey` = t4.`r_regionkey`
+)
+SELECT t0.*
+FROM t0
   LEFT SEMI JOIN (
     SELECT *
     FROM (
-      SELECT t1.`n_name`, sum(t0.`c_acctbal`) AS `sum`
-      FROM customer t0
-        INNER JOIN nation t1
-          ON t0.`c_nationkey` = t1.`n_nationkey`
-        INNER JOIN region t2
-          ON t1.`n_regionkey` = t2.`r_regionkey`
+      SELECT `n_name`, sum(`c_acctbal`) AS `sum`
+      FROM t0
       GROUP BY 1
-    ) t4
+    ) t2
     ORDER BY `sum` DESC
     LIMIT 10
-  ) t3
-    ON t1.`n_name` = t3.`n_name`"""
-
+  ) t1
+    ON t0.`n_name` = t1.`n_name`"""
         assert result == expected
 
     def test_topk_analysis_bug(self):
@@ -1862,7 +1863,7 @@ WHERE EXISTS (
 
         cond = t1.key1 == t2.key1
         expr = t1[cond.any()]
-        stmt = build_ast(expr).queries[0]
+        stmt = _get_query(expr)
 
         repr(stmt.where[0])
 
@@ -2053,7 +2054,7 @@ ORDER BY `string_col`"""
         t = self.con.table('functional_alltypes')
 
         expr = t.limit(20).limit(10)
-        stmt = build_ast(expr).queries[0]
+        stmt = _get_query(expr)
 
         assert stmt.limit['n'] == 10
 
@@ -2258,7 +2259,7 @@ FROM functional_alltypes"""
 
         result = to_sql(expr)
         expected = """\
-SELECT `string_col`, COUNT(DISTINCT `int_col`) AS `nunique`
+SELECT `string_col`, count(DISTINCT `int_col`) AS `nunique`
 FROM functional_alltypes
 WHERE `bigint_col` > 0
 GROUP BY 1"""
@@ -2276,8 +2277,8 @@ GROUP BY 1"""
 
         result = to_sql(expr)
         expected = """\
-SELECT `string_col`, COUNT(DISTINCT `int_col`) AS `int_card`,
-       COUNT(DISTINCT `smallint_col`) AS `smallint_card`
+SELECT `string_col`, count(DISTINCT `int_col`) AS `int_card`,
+       count(DISTINCT `smallint_col`) AS `smallint_card`
 FROM functional_alltypes
 GROUP BY 1"""
         assert result == expected
@@ -2334,3 +2335,97 @@ WHERE `b` = 'm'
 GROUP BY 1
 HAVING max(`a`) = 2"""
     assert result == expected
+
+
+def test_simple_agg_filter():
+    t = ibis.table([('a', 'int64'), ('b', 'string')], name='my_table')
+    filt = t[t.a < 100]
+    expr = filt[filt.a == filt.a.max()]
+    result = to_sql(expr)
+    expected = """\
+SELECT *
+FROM (
+  SELECT *
+  FROM my_table
+  WHERE `a` < 100
+) t0
+WHERE `a` = (
+  SELECT max(`a`) AS `max`
+  FROM my_table
+  WHERE `a` < 100
+)"""
+    assert result == expected
+
+
+def test_agg_and_non_agg_filter():
+    t = ibis.table([('a', 'int64'), ('b', 'string')], name='my_table')
+    filt = t[t.a < 100]
+    expr = filt[filt.a == filt.a.max()]
+    expr = expr[expr.b == 'a']
+    result = to_sql(expr)
+    expected = """\
+SELECT *
+FROM (
+  SELECT *
+  FROM my_table
+  WHERE `a` < 100
+) t0
+WHERE `a` = (
+  SELECT max(`a`) AS `max`
+  FROM my_table
+  WHERE `a` < 100
+) AND
+      `b` = 'a'"""
+    assert result == expected
+
+
+def test_agg_filter():
+    t = ibis.table([('a', 'int64'), ('b', 'int64')], name='my_table')
+    t = t.mutate(b2=t.b * 2)
+    t = t[['a', 'b2']]
+    filt = t[t.a < 100]
+    expr = filt[filt.a == filt.a.max().name('blah')]
+    result = to_sql(expr)
+    expected = """\
+WITH t0 AS (
+  SELECT *, `b` * 2 AS `b2`
+  FROM my_table
+),
+t1 AS (
+  SELECT t0.`a`, t0.`b2`
+  FROM t0
+  WHERE t0.`a` < 100
+)
+SELECT t1.*
+FROM t1
+WHERE t1.`a` = (
+  SELECT max(`a`) AS `blah`
+  FROM t1
+)"""
+    assert result == expected
+
+
+def test_agg_filter_with_alias():
+    t = ibis.table([('a', 'int64'), ('b', 'int64')], name='my_table')
+    t = t.mutate(b2=t.b * 2)
+    t = t[['a', 'b2']]
+    filt = t[t.a < 100]
+    expr = filt[filt.a.name('A') == filt.a.max().name('blah')]
+    result = to_sql(expr)
+    expected = """\
+WITH t0 AS (
+  SELECT *, `b` * 2 AS `b2`
+  FROM my_table
+),
+t1 AS (
+  SELECT t0.`a`, t0.`b2`
+  FROM t0
+  WHERE t0.`a` < 100
+)
+SELECT t1.*
+FROM t1
+WHERE t1.`a` = (
+  SELECT max(`a`) AS `blah`
+  FROM t1
+)"""
+    assert result == expected
