@@ -12,22 +12,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from six import BytesIO
+import unittest
 
 from posixpath import join as pjoin
 from os import path as osp
 import os
 import shutil
 
+from six import BytesIO
+
 import pytest
 
 from ibis.filesystems import HDFS
-from ibis.compat import unittest
 from ibis.impala.tests.common import IbisTestEnv
 import ibis.compat as compat
 import ibis.util as util
 import ibis
 
+pytest.importorskip('hdfs')
 
 ENV = IbisTestEnv()
 
@@ -90,10 +92,7 @@ class TestHDFSE2E(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        try:
-            cls.hdfs.rmdir(cls.tmp_dir)
-        except:
-            pass
+        cls.hdfs.rmdir(cls.tmp_dir)
 
     def setUp(self):
         self.test_files = []
@@ -428,10 +427,7 @@ class TestSuperUserHDFSE2E(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        try:
-            cls.hdfs.rmdir(cls.tmp_dir)
-        except:
-            pass
+        cls.hdfs.rmdir(cls.tmp_dir)
 
     def setUp(self):
         self.test_files = []
